@@ -83,7 +83,7 @@ KColorCm::KColorCm(QWidget *parent, const QVariantList &)
     m_config = KSharedConfig::openConfig("kdeglobals");
 
     setupUi(this);
-    schemeKnsButton->setIcon( KIcon("get-hot-new-stuff") );
+    //schemeKnsButton->setIcon( KIcon("get-hot-new-stuff") );
     connect(colorSet, SIGNAL(currentIndexChanged(int)), this, SLOT(updateColorTable()));
     connect(schemeList, SIGNAL(currentRowChanged(int)), this, SLOT(loadScheme()));
     connect(applyToAlien, SIGNAL(toggled(bool)), this, SLOT(emitChanged()));
@@ -102,9 +102,8 @@ void KColorCm::populateSchemeList()
     // clear the list in case this is being called from reset button click
     schemeList->clear();
     // add current scheme entry
-    QIcon icon = createSchemePreviewIcon(KGlobalSettings::createApplicationPalette(m_config),
-                                         WindecoColors(m_config));
-    QListWidgetItem *currentitem = new QListWidgetItem(icon, i18n("Current"));
+    QIcon icon = createSchemePreviewIcon(m_config);
+    QListWidgetItem *currentitem = new QListWidgetItem(icon, i18nc("Current color scheme", "Current"));
     schemeList->addItem(currentitem);
     schemeList->blockSignals(true); // don't emit changed signals
     schemeList->setCurrentItem(currentitem);
@@ -112,9 +111,8 @@ void KColorCm::populateSchemeList()
 
     // add default entry
     m_config->setReadDefaults(true);
-    icon = createSchemePreviewIcon(KGlobalSettings::createApplicationPalette(m_config),
-                                         WindecoColors(m_config));
-    schemeList->addItem(new QListWidgetItem(icon, i18n("Default")));
+    icon = createSchemePreviewIcon(m_config);
+    schemeList->addItem(new QListWidgetItem(icon, i18nc("Default color scheme", "Default")));
     m_config->setReadDefaults(false);
 
     QStringList schemeFiles = KGlobal::dirs()->findAllResources("data", "color-schemes/*.colors", KStandardDirs::NoDuplicates);
@@ -126,8 +124,7 @@ void KColorCm::populateSchemeList()
 
         // add the entry
         KSharedConfigPtr config = KSharedConfig::openConfig(filename);
-        icon = createSchemePreviewIcon(KGlobalSettings::createApplicationPalette(config),
-                                             WindecoColors(config));
+        icon = createSchemePreviewIcon(config);
         KConfigGroup group(config, "General");
         QString name = group.readEntry("Name", info.baseName());
         QListWidgetItem * newItem = new QListWidgetItem(icon, name);
@@ -141,6 +138,7 @@ void KColorCm::updatePreviews()
 {
     schemePreview->setPalette(m_config);
     colorPreview->setPalette(m_config);
+    setPreview->setPalette(m_config, (KColorScheme::ColorSet)(colorSet->currentIndex() - 1));
     inactivePreview->setPalette(m_config, QPalette::Inactive);
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 }
@@ -220,7 +218,7 @@ void KColorCm::loadScheme()
     {
         QString name = schemeList->currentItem()->text();
         QString fileBaseName = schemeList->currentItem()->data(Qt::UserRole).toString();
-        if (name == i18n("Default"))
+        if (name == i18nc("Default color scheme", "Default"))
         {
             schemeRemoveButton->setEnabled(false);
 
@@ -231,7 +229,7 @@ void KColorCm::loadScheme()
             // load the default scheme
             emit changed(true);
         }
-        else if (name == i18n("Current"))
+        else if (name == i18nc("Current color scheme", "Current"))
         {
             schemeRemoveButton->setEnabled(false);
             loadInternal(false);
@@ -376,8 +374,7 @@ void KColorCm::saveScheme(const QString &name)
         m_config->sync();
 
         QList<QListWidgetItem*> foundItems = schemeList->findItems(name, Qt::MatchExactly);
-        QIcon icon = createSchemePreviewIcon(KGlobalSettings::createApplicationPalette(m_config),
-                                             WindecoColors(m_config));
+        QIcon icon = createSchemePreviewIcon(m_config);
         if (foundItems.size() < 1)
         {
             // add it to the list since it's not in there already
@@ -433,8 +430,9 @@ void KColorCm::variesClicked()
     }
 }
 
-QPixmap KColorCm::createSchemePreviewIcon(const QPalette &pal, const WindecoColors &wm)
+QPixmap KColorCm::createSchemePreviewIcon(const KSharedConfigPtr &config)
 {
+    const WindecoColors wm(config);
     const uchar bits1[] = { 0xff, 0xff, 0xff, 0x2c, 0x16, 0x0b };
     const uchar bits2[] = { 0x68, 0x34, 0x1a, 0xff, 0xff, 0xff };
     const QSize bitsSize(24,2);
@@ -446,20 +444,24 @@ QPixmap KColorCm::createSchemePreviewIcon(const QPalette &pal, const WindecoColo
 
     QPainter p(&pixmap);
 
-    p.fillRect( 1,  1, 7, 7, pal.brush(QPalette::Window));
-    p.fillRect( 2,  2, 5, 2, QBrush(pal.color(QPalette::WindowText), b1));
+    KColorScheme windowScheme(QPalette::Active, KColorScheme::Window, config);
+    p.fillRect( 1,  1, 7, 7, windowScheme.background());
+    p.fillRect( 2,  2, 5, 2, QBrush(windowScheme.foreground().color(), b1));
 
-    p.fillRect( 8,  1, 7, 7, pal.brush(QPalette::Button));
-    p.fillRect( 9,  2, 5, 2, QBrush(pal.color(QPalette::ButtonText), b1));
+    KColorScheme buttonScheme(QPalette::Active, KColorScheme::Button, config);
+    p.fillRect( 8,  1, 7, 7, buttonScheme.background());
+    p.fillRect( 9,  2, 5, 2, QBrush(buttonScheme.foreground().color(), b1));
 
     p.fillRect(15,  1, 7, 7, wm.color(WindecoColors::ActiveBackground));
     p.fillRect(16,  2, 5, 2, QBrush(wm.color(WindecoColors::ActiveForeground), b1));
 
-    p.fillRect( 1,  8, 7, 7, pal.brush(QPalette::Base));
-    p.fillRect( 2, 12, 5, 2, QBrush(pal.color(QPalette::Text), b2));
+    KColorScheme viewScheme(QPalette::Active, KColorScheme::View, config);
+    p.fillRect( 1,  8, 7, 7, viewScheme.background());
+    p.fillRect( 2, 12, 5, 2, QBrush(viewScheme.foreground().color(), b2));
 
-    p.fillRect( 8,  8, 7, 7, pal.brush(QPalette::Highlight));
-    p.fillRect( 9, 12, 5, 2, QBrush(pal.color(QPalette::HighlightedText), b2));
+    KColorScheme selectionScheme(QPalette::Active, KColorScheme::Selection, config);
+    p.fillRect( 8,  8, 7, 7, selectionScheme.background());
+    p.fillRect( 9, 12, 5, 2, QBrush(selectionScheme.foreground().color(), b2));
 
     p.fillRect(15,  8, 7, 7, wm.color(WindecoColors::InactiveBackground));
     p.fillRect(16, 12, 5, 2, QBrush(wm.color(WindecoColors::InactiveForeground), b2));
@@ -698,7 +700,8 @@ void KColorCm::updateColorTable()
     if (currentSet == -1)
     {
         // common colors is selected
-        stackedWidget->setCurrentIndex(0);
+        stackColors->setCurrentIndex(0);
+        stackPreview->setCurrentIndex(0);
 
         KColorButton * button;
         foreach (button, m_commonColorButtons)
@@ -743,7 +746,9 @@ void KColorCm::updateColorTable()
     else
     {
         // a real color set is selected
-        stackedWidget->setCurrentIndex(1);
+        setPreview->setPalette(m_config, (KColorScheme::ColorSet)currentSet);
+        stackColors->setCurrentIndex(1);
+        stackPreview->setCurrentIndex(1);
 
         for (int i = KColorScheme::NormalBackground; i <= KColorScheme::AlternateBackground; ++i)
         {
@@ -1012,7 +1017,9 @@ void KColorCm::load()
     KConfig      cfg("kcmdisplayrc", KConfig::NoGlobals);
     KConfigGroup group(&cfg, "X11");
 
+    applyToAlien->blockSignals(true); // don't emit SIGNAL(toggled(bool)) which would call SLOT(emitChanged())
     applyToAlien->setChecked(group.readEntry("exportKDEColors", true));
+    applyToAlien->blockSignals(false);
 }
 
 void KColorCm::loadInternal(bool loadOptions)
@@ -1086,7 +1093,7 @@ void KColorCm::save()
 void KColorCm::defaults()
 {
     for(int i = 0; i < schemeList->count(); ++i) {
-        if(schemeList->item(i)->text() == i18n("Default")) {
+        if(schemeList->item(i)->text() == i18nc("Default color scheme", "Default")) {
             schemeList->setCurrentItem(schemeList->item(i));
             break;
         }
