@@ -48,8 +48,7 @@
 namespace KFI
 {
 
-K_GLOBAL_STATIC(CFcEngine, theInstance)
-
+bool      CFcEngine::theirFcDirty(true);
 const int CFcEngine::constScalableSizes[]={8, 10, 12, 24, 36, 48, 64, 72, 96, 0 };
 const int CFcEngine::constDefaultAlphaSize=24;
 
@@ -285,10 +284,12 @@ static bool drawAllGlyphs(XftDraw *xftDraw, XftFont *xftFont, XftColor *xftCol,
                     if(r.height()>0)
                     {
                         if(used)
+                        {
                             if(used->isEmpty())
                                 *used=r;
                             else
                                 *used=used->united(r);
+                        }
                         if(max>0 && ++drawn>=max)
                             break;
                     }
@@ -333,17 +334,13 @@ static QString usableStr(XftFont *font, QString &str)
     return newStr;
 }
 
-CFcEngine * CFcEngine::instance()
-{
-    return theInstance;
-}
-
-CFcEngine::CFcEngine()
-         : itsFcDirty(true),
-           itsIndex(-1),
+CFcEngine::CFcEngine(bool init)
+         : itsIndex(-1),
            itsIndexCount(1),
+           itsAlphaSizeIndex(-1),
            itsPreviewString(getDefaultPreviewString())
 {
+    if(init)
         reinit();
 }
 
@@ -570,8 +567,6 @@ bool CFcEngine::draw(const KUrl &url, int w, int h, QPixmap &pix, int faceNo, bo
         bgndCol.setAlphaF(0.0);
         pix.fill(bgndCol);
 
-        QPainter painter(&pix);
-
         if(itsSizes.size())
         {
             const QX11Info &x11Info(pix.x11Info());
@@ -675,9 +670,9 @@ bool CFcEngine::draw(const KUrl &url, int w, int h, QPixmap &pix, int faceNo, bo
                             uppercase(getUppercaseLetters()),
                             punctuation(getPunctuation());
 
-                    drawName(painter, xftDraw, &xftCol, x, y, w, h);
+                    drawName(pix, xftDraw, &xftCol, x, y, w, h);
 
-                    xftFont=getFont(itsAlphaSize);
+                    xftFont=getFont(alphaSize());
                     if(xftFont)
                     {
                         bool lc(hasStr(xftFont, lowercase)),
@@ -699,7 +694,7 @@ bool CFcEngine::draw(const KUrl &url, int w, int h, QPixmap &pix, int faceNo, bo
                                 drawString(xftDraw, xftFont, &xftCol, validPunc, x, y, h);
                             XftFontClose(QX11Info::display(), xftFont);
                             if(lc || uc || punc)
-                                painter.drawLine(0, y, w-1, y);
+                                drawLine(pix, 0, y+2, w-1, 1);
                             y+=8;
                         }
 
@@ -732,9 +727,9 @@ bool CFcEngine::draw(const KUrl &url, int w, int h, QPixmap &pix, int faceNo, bo
                 {
                     if(range.first().null())
                     {
-                        drawName(painter, xftDraw, &xftCol, x, y, w, h);
+                        drawName(pix, xftDraw, &xftCol, x, y, w, h);
 
-                        if((xftFont=getFont(itsAlphaSize)))
+                        if((xftFont=getFont(alphaSize())))
                         {
                             int fontHeight=xftFont->ascent+xftFont->descent;
 
@@ -744,7 +739,7 @@ bool CFcEngine::draw(const KUrl &url, int w, int h, QPixmap &pix, int faceNo, bo
                             XftFontClose(QX11Info::display(), xftFont);
                         }
                     }
-                    else if((xftFont=getFont(itsAlphaSize*2)))
+                    else if((xftFont=getFont(alphaSize()*2)))
                     {
                         QRect r;
                         rv=drawChar32Centre(xftDraw, xftFont, &xftCol, (*(range.begin())).from,
@@ -756,11 +751,11 @@ bool CFcEngine::draw(const KUrl &url, int w, int h, QPixmap &pix, int faceNo, bo
                     QList<TRange>::ConstIterator it(range.begin()),
                                                  end(range.end());
 
-                    if((xftFont=getFont(itsAlphaSize)))
+                    if((xftFont=getFont(alphaSize())))
                     {
                         rv=true;
-                        drawName(painter, xftDraw, &xftCol, x, y, w, h);
-                        y+=itsAlphaSize;
+                        drawName(pix, xftDraw, &xftCol, x, y, w, h);
+                        y+=alphaSize();
 
                         bool  stop=false;
                         int   fontHeight=xftFont->ascent+xftFont->descent, xOrig(x), yOrig(y);
@@ -1126,7 +1121,7 @@ XftFont * CFcEngine::queryFont()
     if(itsInstalled && !f)
     {
         // Perhaps its a newly installed font? If so try re-initialising fontconfig...
-        itsFcDirty=true;
+        theirFcDirty=true;
         reinit();
 
         f=getFont(constQuerySize);
@@ -1264,13 +1259,13 @@ void CFcEngine::getSizes()
 #ifdef KFI_FC_DEBUG
     kDebug();
 #endif
-
     XftFont *f=queryFont();
+    int     alphaSize(itsSizes.size()>itsAlphaSizeIndex && itsAlphaSizeIndex>=0 ? itsSizes[itsAlphaSizeIndex] : constDefaultAlphaSize);
 
     itsScalable=FcTrue;
 
     itsSizes.clear();
-    itsAlphaSize=0;
+    itsAlphaSizeIndex=0;
 
     if(f)
     {
@@ -1311,6 +1306,7 @@ void CFcEngine::getSizes()
 
                 if (set)
                 {
+                    int size(0);
 #ifdef KFI_FC_DEBUG
                     kDebug() << "got fixed sizes: " << set->nfont;
 #endif
@@ -1324,8 +1320,9 @@ void CFcEngine::getSizes()
 #ifdef KFI_FC_DEBUG
                             kDebug() << "got fixed: " << px;
 #endif
-                            if (px<=constDefaultAlphaSize)
-                                itsAlphaSize=(int)px;
+                            if (px<=alphaSize)
+                                itsAlphaSizeIndex=size;
+                            size++;
                         }
                     FcFontSetDestroy(set);
                 }
@@ -1362,8 +1359,8 @@ void CFcEngine::getSizes()
 #endif
                         itsSizes.push_back((int)px);
 
-                        if (px<=constDefaultAlphaSize)
-                            itsAlphaSize=(int)px;
+                        if (px<=alphaSize)
+                            itsAlphaSizeIndex=size;
                     }
                 }
                 XftUnlockFace(f);
@@ -1378,18 +1375,29 @@ void CFcEngine::getSizes()
         itsSizes.reserve(sizeof(constScalableSizes)/sizeof(int));
 
         for (int i=0; constScalableSizes[i]; ++i)
-            itsSizes.push_back(point2Pixel(constScalableSizes[i]));
-        itsAlphaSize=constDefaultAlphaSize;
+        {
+            int px=point2Pixel(constScalableSizes[i]);
+
+            if (px<=alphaSize)
+                itsAlphaSizeIndex=i;
+            itsSizes.push_back(px);
+        }
     }
 
-    if(0==itsAlphaSize && itsSizes.count())
-        itsAlphaSize=itsSizes[0];
 #ifdef KFI_FC_DEBUG
     kDebug() << "end";
 #endif
 }
 
-void CFcEngine::drawName(QPainter &painter, XftDraw *xftDraw, XftColor *xftCol, int x, int &y, int w, int h)
+void CFcEngine::drawLine(QPixmap &pix, int x, int y, int w, int h)
+{
+    QPainter painter(&pix);
+    painter.setPen(theirTextCol);
+    painter.drawLine(x, y, x+w-1, y+h-1);
+    painter.end();
+}
+
+void CFcEngine::drawName(QPixmap &pix, XftDraw *xftDraw, XftColor *xftCol, int x, int &y, int w, int h)
 {
     QString title(itsDescriptiveName.isEmpty()
                     ? i18n("ERROR: Could not determine font's name.")
@@ -1401,8 +1409,7 @@ void CFcEngine::drawName(QPainter &painter, XftDraw *xftDraw, XftColor *xftCol,
     drawString(xftDraw, xftCol, title, x, y, h);
 
     y+=4;
-    painter.setPen(theirTextCol);
-    painter.drawLine(0, y, w-1, y);
+    drawLine(pix, x, y, w-1, 1);
     y+=8;
 }
 
@@ -1418,11 +1425,11 @@ void CFcEngine::addFontFile(const QString &file)
 
 void CFcEngine::reinit()
 {
-    if(itsFcDirty)
+    if(theirFcDirty)
     {
         FcInitLoadConfigAndFonts();
         FcInitReinitialize();
-        itsFcDirty=false;
+        theirFcDirty=false;
     }
 }
 
