@@ -60,6 +60,7 @@
 #include <kscreensaver_interface.h>
 #include <KPluginFactory>
 #include <KPluginLoader>
+#include <KMessageBox>
 
 template class QList<SaverConfig*>;
 
@@ -238,6 +239,21 @@ KScreenSaver::KScreenSaver(QWidget *parent, const QVariantList&)
     mLockLbl->setWhatsThis( wltstr );
     mWaitLockEdit->setWhatsThis( wltstr );
 
+    mPlasmaCheckBox = new QCheckBox(i18n("Allow widgets on screen saver"), mSaverGroup);
+    mPlasmaCheckBox->setChecked(mPlasmaEnabled);
+    mPlasmaCheckBox->setWhatsThis(i18n("Add widgets to your screensaver"));
+    connect(mPlasmaCheckBox, SIGNAL(toggled(bool)), this, SLOT(slotEnablePlasma(bool)));
+    groupLayout->addWidget(mPlasmaCheckBox);
+
+    hbox = new QHBoxLayout();
+    groupLayout->addLayout(hbox);
+    hbox->addSpacing(30);
+    mPlasmaSetup = new QPushButton(i18n("Setup..."), mSaverGroup);
+    mPlasmaSetup->setEnabled(mPlasmaEnabled);
+    connect(mPlasmaSetup, SIGNAL(clicked()), this, SLOT(slotPlasmaSetup()));
+    hbox->addWidget(mPlasmaSetup);
+    hbox->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
+
     // right column
     QBoxLayout* rightColumnLayout = new QVBoxLayout();
     topLayout->addItem( rightColumnLayout );
@@ -389,6 +405,7 @@ void KScreenSaver::readSettings()
     mLockTimeout = config.readEntry("LockGrace", 60000);
     mLock = config.readEntry("Lock", false);
     mSaver = config.readEntry("Saver");
+    mPlasmaEnabled = config.readEntry("PlasmaEnabled", false);
 
     if (mTimeout < 60) mTimeout = 60;
     if (mLockTimeout < 0) mLockTimeout = 0;
@@ -432,6 +449,8 @@ void KScreenSaver::defaults()
     slotLockTimeoutChanged( 60 );
     slotLock( false );
     mEnabledCheckBox->setChecked(false);
+    mPlasmaCheckBox->setChecked(false);
+    mPlasmaSetup->setEnabled(false);
 
     updateValues();
 
@@ -451,6 +470,7 @@ void KScreenSaver::save()
     config.writeEntry("Timeout", mTimeout);
     config.writeEntry("LockGrace", mLockTimeout);
     config.writeEntry("Lock", mLock);
+    config.writeEntry("PlasmaEnabled", mPlasmaEnabled);
 
     if ( !mSaver.isEmpty() )
         config.writeEntry("Saver", mSaver);
@@ -475,8 +495,8 @@ void KScreenSaver::findSavers()
         else
             mLoadTimer->start( 50 );
     }
-    for( KService::List::const_iterator it = mSaverServices.begin();
-        it != mSaverServices.end(); it++,mNumLoaded++)
+    for( KService::List::const_iterator it = mSaverServices.constBegin();
+        it != mSaverServices.constEnd(); it++,mNumLoaded++)
     {
       SaverConfig *saver = new SaverConfig;
       QString file = KStandardDirs::locate("services", (*it)->entryPath());
@@ -615,6 +635,23 @@ void KScreenSaver::slotEnable(bool e)
     emit changed(true);
 }
 
+void KScreenSaver::slotEnablePlasma(bool enable)
+{
+    mPlasmaEnabled = enable;
+    //FIXME even though the button's enabled, plasma isn't until the user hits apply
+    //so the button will just show the screensaver, no plasma.
+    //what should I do about this?
+    mPlasmaSetup->setEnabled(mPlasmaEnabled);
+    mChanged = true;
+    emit changed(true);
+}
+
+void KScreenSaver::slotPlasmaSetup()
+{
+    org::kde::screensaver kscreensaver("org.kde.screensaver", "/ScreenSaver", QDBusConnection::sessionBus());
+    kscreensaver.setupPlasma();
+}
+
 
 //---------------------------------------------------------------------------
 //
