@@ -4,6 +4,7 @@
 //
 // Copyright 1999 Martin R. Jones <mjones@kde.org>
 // Copyright 2003 Oswald Buddenhagen <ossi@kde.org>
+// Copyright 2008 Chani Armitage <chanika@gmail.com>
 //
 
 //krunner keeps running and checks user inactivity
@@ -15,6 +16,7 @@
 //crashes
 
 #include "lockprocess.h"
+#include "lockprocessadaptor.h"
 
 #include <config-workspace.h>
 #include <config-X11.h>
@@ -32,7 +34,7 @@
 #include <KMessageBox>
 #include <KGlobalSettings>
 #include <KLocale>
-#include <KLibLoader>
+#include <KLibrary>
 #include <KPushButton>
 #include <KStandardGuiItem>
 #include <KAuthorized>
@@ -52,6 +54,9 @@
 #include <QX11Info>
 #include <QTextStream>
 #include <QPainter>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QDBusInterface>
 
 #include <QDateTime>
 
@@ -116,6 +121,8 @@ int XSelectInput( Display* dpy, Window w, long e )
 }
 #endif
 
+const int TIMEOUT_CODE = 2; //from PasswordDlg
+
 //===========================================================================
 //
 // Screen saver handling process.  Handles screensaver window,
@@ -123,6 +130,10 @@ int XSelectInput( Display* dpy, Window w, long e )
 //
 LockProcess::LockProcess(bool child, bool useBlankOnly)
     : QWidget(0L, Qt::X11BypassWindowManagerHint),
+      mLocked(false),
+      mBusy(false),
+      mPlasmaDBus(0),
+      mSetupMode(false),
       mOpenGLVisual(false),
       child_saver(child),
       mParent(0),
@@ -131,11 +142,15 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
       mVisibility(false),
       mRestoreXF86Lock(false),
       mForbidden(false),
-      mAutoLogout(false)
+      mAutoLogoutTimerId(0)
 {
     setObjectName("save window");
     setupSignals();
 
+    new LockProcessAdaptor(this);
+    QDBusConnection::sessionBus().registerService("org.kde.krunner_lock");
+    QDBusConnection::sessionBus().registerObject("/LockProcess", this);
+
     kapp->installX11EventFilter(this);
 
     // Get root window size
@@ -163,7 +178,7 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
 
     QStringList dmopt =
         QString::fromLatin1( ::getenv( "XDM_MANAGED" )).split(QChar(','), QString::SkipEmptyParts);
-    for (QStringList::ConstIterator it = dmopt.begin(); it != dmopt.end(); ++it)
+    for (QStringList::ConstIterator it = dmopt.constBegin(); it != dmopt.constEnd(); ++it)
         if ((*it).startsWith("method="))
             mMethod = (*it).mid(7);
 
@@ -185,6 +200,9 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
 #endif
 
     greetPlugin.library = 0;
+
+    mSuppressUnlock.setSingleShot(true);
+    connect(&mSuppressUnlock, SIGNAL(timeout()), SLOT(deactivatePlasma()));
 }
 
 //---------------------------------------------------------------------------
@@ -308,6 +326,14 @@ bool LockProcess::defaultSave()
     return false;
 }
 
+bool LockProcess::startSetup()
+{
+    mPlasmaEnabled = true; //force it on in case the user didn't click apply yet
+    mLocked = false;
+    mSetupMode = true;
+    return startSaver();
+    //plasma startup will handle the suppressunlock bit
+}
 //---------------------------------------------------------------------------
 bool LockProcess::dontLock()
 {
@@ -340,7 +366,6 @@ void LockProcess::configure()
     }
 
     if ( KScreenSaverSettings::autoLogout() ) {
-        mAutoLogout = true;
         mAutoLogoutTimeout = KScreenSaverSettings::autoLogoutTimeout();
         mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000); // in milliseconds
     }
@@ -360,6 +385,11 @@ void LockProcess::configure()
 
     readSaver();
 
+    mPlasmaEnabled = KScreenSaverSettings::plasmaEnabled();
+
+    mSuppressUnlockTimeout = qMax(0, KScreenSaverSettings::timeout() * 1000);
+    mSuppressUnlockTimeout = qMax(mSuppressUnlockTimeout, 30 * 1000); //min. 30 secs FIXME is this a good idea?
+
     mPlugins = KScreenSaverSettings::pluginsUnlock();
     if (mPlugins.isEmpty()) {
         mPlugins << "classic" << "generic";
@@ -723,6 +753,7 @@ bool LockProcess::startSaver()
 
     setVRoot( winId(), winId() );
     startHack();
+    startPlasma();
     return true;
 }
 
@@ -734,6 +765,7 @@ void LockProcess::stopSaver()
 {
     kDebug(1204) << "LockProcess: stopping saver";
     resume( true );
+    stopPlasma();
     stopHack();
     hideSaverWindow();
     mVisibility = false;
@@ -742,7 +774,7 @@ void LockProcess::stopSaver()
             KDisplayManager().setLock( false );
         ungrabInput();
         const char *out = "GOAWAY!";
-        for (QList<int>::ConstIterator it = child_sockets.begin(); it != child_sockets.end(); ++it)
+        for (QList<int>::ConstIterator it = child_sockets.constBegin(); it != child_sockets.constEnd(); ++it)
             write(*it, out, sizeof(out));
     }
 }
@@ -752,8 +784,8 @@ QVariant LockProcess::getConf(void *ctx, const char *key, const QVariant &dflt)
 {
     LockProcess *that = (LockProcess *)ctx;
     QString fkey = QLatin1String( key ) + '=';
-    for (QStringList::ConstIterator it = that->mPluginOptions.begin();
-         it != that->mPluginOptions.end(); ++it)
+    for (QStringList::ConstIterator it = that->mPluginOptions.constBegin();
+         it != that->mPluginOptions.constEnd(); ++it)
         if ((*it).startsWith( fkey ))
             return (*it).mid( fkey.length() );
     return dflt;
@@ -775,39 +807,58 @@ i18n("<i>kcheckpass</i> is unable to operate. Possibly it is not setuid root.");
 //
 bool LockProcess::startLock()
 {
-    for (QStringList::ConstIterator it = mPlugins.begin(); it != mPlugins.end(); ++it) {
+    if (loadGreetPlugin()) {
+        mLocked = true;
+        KDisplayManager().setLock(true);
+        lockPlasma();
+        return true;
+    }
+    return false;
+}
+
+bool LockProcess::loadGreetPlugin()
+{
+    if (greetPlugin.library) {
+        //we were locked once before, so all the plugin loading's done already
+        //FIXME should I be unloading the plugin on unlock instead?
+        return true;
+    }
+    for (QStringList::ConstIterator it = mPlugins.constBegin(); it != mPlugins.constEnd(); ++it) {
         GreeterPluginHandle plugin;
-        QString path = KLibLoader::self()->findLibrary(
-                    ((*it)[0] == '/' ? *it : "kgreet_" + *it ).toLatin1() );
-        if (path.isEmpty()) {
+        KLibrary *lib = new KLibrary( (*it)[0] == '/' ? *it : "kgreet_" + *it );
+        if (lib->fileName().isEmpty()) {
             kWarning(1204) << "GreeterPlugin " << *it << " does not exist" ;
+            delete lib;
             continue;
         }
-        if (!(plugin.library = KLibLoader::self()->library( path.toLatin1() ))) {
-            kWarning(1204) << "Cannot load GreeterPlugin " << *it << " (" << path << ")" ;
+        if (!lib->load()) {
+            kWarning(1204) << "Cannot load GreeterPlugin " << *it << " (" << lib->fileName() << ")" ;
+            delete lib;
             continue;
         }
-        plugin.info = (KGreeterPluginInfo *)plugin.library->resolveSymbol( "kgreeterplugin_info" );
+        plugin.library = lib;
+        plugin.info = (KGreeterPluginInfo *)lib->resolveSymbol( "kgreeterplugin_info" );
         if (!plugin.info ) {
-            kWarning(1204) << "GreeterPlugin " << *it << " (" << path << ") is no valid greet widget plugin" ;
-            plugin.library->unload();
+            kWarning(1204) << "GreeterPlugin " << *it << " (" << lib->fileName() << ") is no valid greet widget plugin" ;
+            lib->unload();
+            delete lib;
             continue;
         }
         if (plugin.info->method && !mMethod.isEmpty() && mMethod != plugin.info->method) {
-            kDebug(1204) << "GreeterPlugin " << *it << " (" << path << ") serves " << plugin.info->method << ", not " << mMethod;
-            plugin.library->unload();
+            kDebug(1204) << "GreeterPlugin " << *it << " (" << lib->fileName() << ") serves " << plugin.info->method << ", not " << mMethod;
+            lib->unload();
+            delete lib;
             continue;
         }
         if (!plugin.info->init( mMethod, getConf, this )) {
-            kDebug(1204) << "GreeterPlugin " << *it << " (" << path << ") refuses to serve " << mMethod;
-            plugin.library->unload();
+            kDebug(1204) << "GreeterPlugin " << *it << " (" << lib->fileName() << ") refuses to serve " << mMethod;
+            lib->unload();
+            delete lib;
             continue;
         }
         kDebug(1204) << "GreeterPlugin " << *it << " (" << plugin.info->method << ", " << plugin.info->name << ") loaded";
         greetPlugin = plugin;
-	mLocked = true;
-	KDisplayManager().setLock( true );
-	return true;
+        return true;
     }
     cantLock( i18n("No appropriate greeter plugin configured.") );
     return false;
@@ -868,6 +919,174 @@ void LockProcess::hackExited()
     XClearWindow(QX11Info::display(), winId());
 }
 
+bool LockProcess::startPlasma()
+{
+    if (!mPlasmaEnabled) {
+        return false;
+    }
+
+    if (mSetupMode) {
+        mSuppressUnlock.start(mSuppressUnlockTimeout);
+        XChangeActivePointerGrab(QX11Info::display(), GRABEVENTS,
+                                 QCursor(Qt::ArrowCursor).handle(), CurrentTime);
+    }
+
+    kDebug() << "looking for plasma-overlay";
+    if (!mPlasmaDBus) {
+        //try to get it, in case it's already running somehow
+        //FIXME I don't like hardcoded strings
+        //mPlasmaDBus = new QDBusInterface("org.kde.plasma-overlay", "/MainApplication", QString(),
+        mPlasmaDBus = new org::kde::plasmaoverlay::App("org.kde.plasma-overlay", "/App",
+                                                       QDBusConnection::sessionBus(), this);
+        //FIXME this might-already-be-running stuff seems really really Wrong.
+    }
+
+    if (mPlasmaDBus->isValid()) {
+        kDebug() << "weird, plasma-overlay is already running";
+        mPlasmaDBus->call(QDBus::NoBlock, "setup", mSetupMode);
+        return true;
+    }
+
+    kDebug () << "...not found" << "starting plasma-overlay";
+    delete mPlasmaDBus;
+    mPlasmaDBus = 0;
+
+    connect(QDBusConnection::sessionBus().interface(), SIGNAL(serviceOwnerChanged(QString, QString, QString)),
+            this, SLOT(newService(QString, QString, QString)));
+
+    KProcess *plasmaProc = new KProcess;
+    plasmaProc->setProgram("plasma-overlay");
+    if (mSetupMode) {
+        *plasmaProc << "--setup";
+    }
+
+    //make sure it goes away when it's done (and not before)
+    connect(plasmaProc, SIGNAL(finished(int,QProcess::ExitStatus)), plasmaProc, SLOT(deleteLater()));
+
+    plasmaProc->start();
+    kDebug() << "process begun";
+
+    //plasma gets 15 seconds to load, or we assume it failed
+    QTimer::singleShot(15 * 1000, this, SLOT(checkPlasma()));
+    return true;
+}
+
+void LockProcess::checkPlasma()
+{
+    if (!mPlasmaEnabled) {
+        kDebug() << "You're Doing It Wrong!";
+        return;
+    }
+    if (mPlasmaDBus && mPlasmaDBus->isValid()) {
+        //hooray, looks like it started ok
+        kDebug() << "success!";
+        //...but just in case, make sure we're not waiting on it
+        mSetupMode = false;
+        return;
+    }
+
+    kDebug() << "ohnoes. plasma = teh fail.";
+    disablePlasma();
+}
+
+bool LockProcess::isPlasmaValid()
+{
+    //FIXME I'm assuming that if it's valid, calls will succeed. so if that's not the case we'll
+    //need to change things so that plasma's disabled properly if it fails
+    //damn. isValid is not quite enough. a call may still fail, and then we need to bail.
+    if (!(mPlasmaEnabled && mPlasmaDBus)) {
+        return false; //no plasma, at least not yet
+    }
+    if (mPlasmaDBus->isValid()) {
+        return true;
+    }
+    //oh crap, it ran away on us.
+    disablePlasma();
+    return false;
+}
+
+void LockProcess::disablePlasma()
+{
+    kDebug();
+    mPlasmaEnabled = false;
+    mSetupMode = false;
+    mSuppressUnlock.stop(); //FIXME we might need to start the lock timer ala deactivatePlasma()
+    //actually we could be lazy and just call deactivatePlasma() TODO check that this'll really work
+    delete mPlasmaDBus;
+    mPlasmaDBus=0;
+}
+
+void LockProcess::stopPlasma()
+{
+    if (mPlasmaDBus && mPlasmaDBus->isValid()) {
+        mPlasmaDBus->call(QDBus::NoBlock, "quit");
+    } else {
+        kDebug() << "cannot stop plasma-overlay";
+    }
+}
+
+void LockProcess::newService(QString name, QString oldOwner, QString newOwner)
+{
+    Q_UNUSED(oldOwner);
+    if (name != "org.kde.plasma-overlay") {
+        return;
+    }
+
+    if (mPlasmaDBus) {
+        if (newOwner.isEmpty()) {
+            kDebug() << "plasma ran away?";
+            disablePlasma();
+        } else {
+            kDebug() << "I'm confused!!";
+        }
+        return;
+    }
+
+    kDebug() << "plasma! yaay!";
+    mPlasmaDBus = new org::kde::plasmaoverlay::App(name, "/App",
+            QDBusConnection::sessionBus(), this);
+
+    //XXX this isn't actually used any more iirc
+    connect(mPlasmaDBus, SIGNAL(hidden()), SLOT(unSuppressUnlock()));
+
+    if (!mDialogs.isEmpty()) {
+        //whoops, activation probably failed earlier
+        mPlasmaDBus->call(QDBus::NoBlock, "setActive", true);
+    }
+}
+
+void LockProcess::deactivatePlasma()
+{
+    if (isPlasmaValid()) {
+        mPlasmaDBus->call(QDBus::NoBlock, "setActive", false);
+    }
+    if (!mLocked && mLockGrace >=0) {
+        QTimer::singleShot(mLockGrace, this, SLOT(startLock())); //this is only ok because any activity will quit
+    }
+}
+
+void LockProcess::lockPlasma()
+{
+    if (isPlasmaValid()) {
+        mPlasmaDBus->call(QDBus::NoBlock, "lock");
+    }
+}
+
+void LockProcess::unSuppressUnlock()
+{
+    //note: suppressing unlock also now means suppressing quit-on-activity
+    //maybe some var renaming is in order.
+    mSuppressUnlock.stop();
+}
+
+void LockProcess::quit()
+{
+    mSuppressUnlock.stop();
+    if (!mLocked || checkPass()) {
+        quitSaver();
+    }
+}
+
 void LockProcess::suspend()
 {
     if( !mSuspended && mHackProc.state() == QProcess::Running )
@@ -904,10 +1123,22 @@ void LockProcess::resume( bool force )
 bool LockProcess::checkPass()
 {
     killTimer(mAutoLogoutTimerId);
-    PasswordDlg passDlg( this, &greetPlugin);
 
+    if (isPlasmaValid()) {
+        mPlasmaDBus->call(QDBus::NoBlock, "setActive", true);
+    }
+
+    PasswordDlg passDlg( this, &greetPlugin);
     int ret = execDialog( &passDlg );
 
+    if (isPlasmaValid()) {
+        if (ret == QDialog::Rejected) {
+            mSuppressUnlock.start(mSuppressUnlockTimeout);
+        } else if (ret == TIMEOUT_CODE) {
+            mPlasmaDBus->call(QDBus::NoBlock, "setActive", false);
+        }
+    }
+
     XWindowAttributes rootAttr;
     XGetWindowAttributes(QX11Info::display(), QX11Info::appRootWindow(), &rootAttr);
     if(( rootAttr.your_event_mask & SubstructureNotifyMask ) == 0 )
@@ -920,6 +1151,38 @@ bool LockProcess::checkPass()
     return ret == QDialog::Accepted;
 }
 
+bool LockProcess::checkPass(const QString &reason)
+{
+    if (! mLocked) {
+        //we were never locked... how can we unlock?!
+        //if anyone finds a use case for checking the password while unlocked, they'll have to load
+        //the greetplugin n'stuff
+        return false;
+    }
+    PasswordDlg passDlg(this, &greetPlugin, reason);
+    int ret = execDialog( &passDlg );
+    kDebug() << ret;
+
+    //FIXME do we need to copy&paste that SubstructureNotifyMask code above?
+    if (ret == QDialog::Accepted) {
+        //we don't quit on a custom checkpass, but we do unlock
+        //so that the user doesn't have to type their password twice
+        mLocked = false;
+        KDisplayManager().setLock(false);
+        //FIXME while suppressUnlock *should* always be running, if it isn't
+        //(say if someone's doing things they shouldn't with dbus) then it won't get started by this
+        //which means that a successful unlock will never re-lock
+        //in fact, the next bit of activity would lead to the screensaver quitting.
+        //possible solutions:
+        //-treat this function like activity: quit if already unlocked, ensure suppress is started
+        //if we're locked and the dialog's rejected
+        //-return true if already unlocked, without doing anything, same as above if locked
+        //-let it quit, and tell people not to do such silly things :P
+        return true;
+    }
+    return false;
+}
+
 static void fakeFocusIn( WId window )
 {
     // We have keyboard grab, so this application will
@@ -973,11 +1236,9 @@ int LockProcess::execDialog( QDialog *dlg )
     if (pos != -1)
         mDialogs.remove( pos );
     if( mDialogs.isEmpty() ) {
-        XChangeActivePointerGrab( QX11Info::display(), GRABEVENTS,
-                QCursor(Qt::BlankCursor).handle(), CurrentTime);
         resume( false );
-    } else
-        fakeFocusIn( mDialogs.first()->winId());
+    }
+    updateFocus();
 
     dlg->removeEventFilter(this);
     mFrames.remove(dlg);
@@ -998,7 +1259,21 @@ void LockProcess::cleanupPopup()
 
     int pos = mDialogs.indexOf( dlg );
     mDialogs.remove( pos );
-    fakeFocusIn( mDialogs.first()->winId() );
+    updateFocus();
+}
+
+void LockProcess::updateFocus()
+{
+    if (mDialogs.isEmpty()) {
+        if (mForeignInputWindows.isEmpty()) {
+            XChangeActivePointerGrab( QX11Info::display(), GRABEVENTS,
+                    QCursor(Qt::BlankCursor).handle(), CurrentTime);
+        } else {
+            fakeFocusIn(mForeignInputWindows.first());
+        }
+    } else {
+        fakeFocusIn(mDialogs.first()->winId());
+    }
 }
 
 //---------------------------------------------------------------------------
@@ -1007,52 +1282,118 @@ void LockProcess::cleanupPopup()
 //
 bool LockProcess::x11Event(XEvent *event)
 {
+    bool ret = false;
     switch (event->type)
     {
-        case KeyPress:
         case ButtonPress:
-        case MotionNotify:
-            if (mBusy || !mDialogs.isEmpty())
+            if (!mDialogs.isEmpty() && event->xbutton.window == event->xbutton.root) {
+                //kDebug() << "close" << mDialogs.first()->effectiveWinId();
+                KDialog *dlg = qobject_cast<KDialog*>(mDialogs.first());
+                if (dlg) {
+                    //kDebug() << "casting success";
+                    dlg->reject();
+                }
                 break;
-            mBusy = true;
-            if (!mLocked || checkPass())
-            {
-                stopSaver();
-                qApp->quit();
             }
-            else if (mAutoLogout) // we need to restart the auto logout countdown
+        case KeyPress:
+        case MotionNotify:
+            if (mBusy || !mDialogs.isEmpty()) {
+                //kDebug() << "busy";
+                //FIXME shouldn't we be resetting some timers?
+                break;
+            }
+            mBusy = true;
+            //something happened. do we quit, ask for a password or forward it to plasma?
+            //if we're supposed to be forwarding, we check that there's actually a plasma window up
+            //so that the user isn't trapped if plasma crashes or is slow to load.
+            //however, if plasma started in setup mode, we don't want to let anything happen until
+            //it has a chance to load.
+            //note: mSetupMode should end when we either get a winid or hit the checkPlasma timeout
+            if (mSuppressUnlock.isActive() && (mSetupMode || !mForeignInputWindows.isEmpty())) {
+                mSuppressUnlock.start(); //help, help, I'm being suppressed!
+            } else if (!mLocked || checkPass()) {
+                quitSaver();
+                mBusy = false;
+                return true; //it's better not to forward any input while quitting, right?
+            }
+            if (mAutoLogoutTimerId) // we need to restart the auto logout countdown
             {
                 killTimer(mAutoLogoutTimerId);
-                mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout);
+                mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000);
             }
             mBusy = false;
-            return true;
+            ret = true;
+            break;
 
         case VisibilityNotify:
             if( event->xvisibility.window == winId())
             {  // mVisibility == false means the screensaver is not visible at all
                // e.g. when switched to text console
+               // ...or when plasma's over it non-compositely?
+               // hey, this gives me free "suspend saver when plasma obscures it"
                 mVisibility = !(event->xvisibility.state == VisibilityFullyObscured);
-                if(!mVisibility)
+                if (!mVisibility) {
                     mSuspendTimer.start(2000);
-		else
-                {
+                    kDebug() << "fully obscured";
+                } else {
+                    kDebug() << "not fully obscured";
                     mSuspendTimer.stop();
                     resume( false );
                 }
-                if (event->xvisibility.state != VisibilityUnobscured)
+                if (mForeignWindows.isEmpty() && event->xvisibility.state != VisibilityUnobscured) {
+                    kDebug() << "no plasma; saver obscured";
                     stayOnTop();
+                }
+            } else if (!mForeignWindows.isEmpty() && event->xvisibility.window == mForeignWindows.last() &&
+                    event->xvisibility.state != VisibilityUnobscured) {
+                //FIXME now that we have several plasma winids this doesn't feel valid
+                //but I don't know what to do about it!
+                kDebug() << "plasma obscured!";
+                stayOnTop();
             }
             break;
 
         case ConfigureNotify: // from SubstructureNotifyMask on the root window
-            if(event->xconfigure.event == QX11Info::appRootWindow())
+            if(event->xconfigure.event == QX11Info::appRootWindow()) {
+                //kDebug() << "ConfigureNotify:"; 
+                //the stacking order changed, so let's change the stacking order!
                 stayOnTop();
+            }
             break;
         case MapNotify: // from SubstructureNotifyMask on the root window
-            if( event->xmap.event == QX11Info::appRootWindow())
+            if( event->xmap.event == QX11Info::appRootWindow()) {
+                kDebug() << "MapNotify:" << event->xmap.window;
+                WindowType type = windowType(event->xmap.window);
+                if (type != IgnoreWindow) {
+                    if (mForeignWindows.contains(event->xmap.window)) {
+                        kDebug() << "uhoh! duplicate!";
+                    } else {
+                        //ordered youngest-on-top
+                        mForeignWindows.prepend(event->xmap.window);
+                    }
+                    if (type & InputWindow) {
+                        kDebug() << "input window";
+                        if (mForeignInputWindows.contains(event->xmap.window)) {
+                            kDebug() << "uhoh! duplicate again"; //never happens
+                        } else {
+                            //ordered youngest-on-top
+                            mForeignInputWindows.prepend(event->xmap.window);
+                            fakeFocusIn(event->xmap.window);
+                        }
+                        mSetupMode = false; //no more waiting for plasma
+                    }
+                }
                 stayOnTop();
+            }
             break;
+        case UnmapNotify:
+            if (event->xmap.event == QX11Info::appRootWindow()) {
+                kDebug() << "UnmapNotify:" << event->xunmap.window;
+                mForeignWindows.removeAll(event->xunmap.window);
+                if (mForeignInputWindows.removeAll(event->xunmap.window)) {
+                    updateFocus();
+                }
+            }
     }
 
     // We have grab with the grab window being the root window.
@@ -1063,41 +1404,131 @@ bool LockProcess::x11Event(XEvent *event)
     // Qt seems to be quite hard to persuade to redirect the event,
     // so let's simply dupe it with correct destination window,
     // and ignore the original one.
-    if(!mDialogs.isEmpty() && ( event->type == KeyPress || event->type == KeyRelease)
-        && event->xkey.window != mDialogs.first()->winId())
-    {
-        XEvent ev2 = *event;
-        ev2.xkey.window = ev2.xkey.subwindow = mDialogs.first()->winId();
-        qApp->x11ProcessEvent( &ev2 );
-        return true;
+    if (!mDialogs.isEmpty()) {
+        if ((event->type == KeyPress || event->type == KeyRelease) &&
+                event->xkey.window != mDialogs.first()->winId()) {
+            //kDebug() << "forward to dialog";
+            XEvent ev2 = *event;
+            ev2.xkey.window = ev2.xkey.subwindow = mDialogs.first()->winId();
+            qApp->x11ProcessEvent( &ev2 );
+            ret = true;
+        }
+    } else if (!mForeignInputWindows.isEmpty()) {
+        //when there are no dialogs, forward some events to plasma
+        switch (event->type) {
+        case KeyPress:
+        case KeyRelease:
+        case ButtonPress:
+        case ButtonRelease:
+        case MotionNotify:
+            {
+                //kDebug() << "forward to plasma";
+                XEvent ev2 = *event;
+                ev2.xkey.window = ev2.xkey.subwindow = mForeignInputWindows.first();
+                XSendEvent(QX11Info::display(), ev2.xkey.window, False, NoEventMask, &ev2);
+                ret = true;
+            }
+        default:
+            break;
+        }
     }
 
-    return false;
+    return ret;
+}
+
+LockProcess::WindowType LockProcess::windowType(WId id)
+{
+    Atom tag = XInternAtom(QX11Info::display(), "_KDE_SCREENSAVER_OVERRIDE", False);
+    Atom actualType;
+    int actualFormat;
+    unsigned long nitems, remaining;
+    unsigned char *data = 0;
+    Display *display = QX11Info::display();
+
+    int result = XGetWindowProperty(display, id, tag, 0, 1, False, tag, &actualType,
+            &actualFormat, &nitems, &remaining, &data);
+
+    //kDebug() << (result == Success) << (actualType == tag);
+    WindowType type = IgnoreWindow;
+    if (result == Success && actualType == tag) {
+        if (nitems != 1 || actualFormat != 8) {
+            kDebug() << "malformed property";
+        } else {
+            kDebug() << "i can haz plasma window?" << data[0];
+            switch (data[0]) {
+                case 0: //FIXME magic numbers
+                    type = SimpleWindow;
+                    break;
+                case 1:
+                    type = InputWindow;
+                    break;
+                case 2:
+                    type = DefaultWindow;
+                    break;
+            }
+        }
+    }
+    if (data) {
+        XFree(data);
+    }
+    return type;
+/*    if (result != Success) {
+        return false;
+    }
+    if (actualType == tag) {
+        return true;
+    }
+    //managed windows will have a pesky frame we have to bypass
+    XWindowAttributes attr;
+    XGetWindowAttributes(display, id, &attr);
+    if (!attr.override_redirect) {
+        //check the real client window
+        if (Window client = XmuClientWindow(display, id)) {
+            result = XGetWindowProperty(display, client, tag, 0, 0, False, tag, &actualType,
+                    &actualFormat, &nitems, &remaining, &data);
+            kDebug() << (result == Success) << (actualType == tag);
+            if (data) {
+                XFree(data);
+            }
+            return (result == Success) && (actualType == tag);
+*        }
+    }
+    return false;*/
 }
 
 void LockProcess::stayOnTop()
 {
-    if(!mDialogs.isEmpty())
+    if(!(mDialogs.isEmpty() && mForeignWindows.isEmpty()))
     {
         // this restacking is written in a way so that
         // if the stacking positions actually don't change,
         // all restacking operations will be no-op,
         // and no ConfigureNotify will be generated,
         // thus avoiding possible infinite loops
-        XRaiseWindow( QX11Info::display(), mDialogs.first()->winId()); // raise topmost
-        // and stack others below it
-        Window* stack = new Window[ mDialogs.count() + 1 ];
+        Window* stack = new Window[ mDialogs.count() + mForeignWindows.count() + 1 ];
         int count = 0;
-        for( QVector< QWidget* >::ConstIterator it = mDialogs.begin();
-             it != mDialogs.end();
-             ++it )
-            stack[ count++ ] = (*it)->winId();
+        if (!mDialogs.isEmpty()) {
+            XRaiseWindow( QX11Info::display(), mDialogs.first()->winId()); // raise topmost
+            // and stack others below it
+            for( QVector< QWidget* >::ConstIterator it = mDialogs.constBegin();
+                    it != mDialogs.constEnd();
+                    ++it )
+                stack[ count++ ] = (*it)->winId();
+        } else {
+            XRaiseWindow( QX11Info::display(), mForeignWindows.first()); // raise topmost
+        }
+        //now the plasma stuff below the dialogs
+        foreach (const WId w, mForeignWindows) {
+            stack[count++] = w;
+        }
+        //finally, the saver window
         stack[ count++ ] = winId();
         XRestackWindows( x11Info().display(), stack, count );
+        //kDebug() << "restacked" << count;
         delete[] stack;
-    }
-    else
+    } else {
         XRaiseWindow(QX11Info::display(), winId());
+    }
 }
 
 void LockProcess::checkDPMSActive()
