@@ -906,8 +906,8 @@ QStringList KSMServer::sessionList()
 {
     QStringList sessions ( "default" );
     KSharedConfig::Ptr config = KGlobal::config();
-    QStringList groups = config->groupList();
-    for ( QStringList::ConstIterator it = groups.begin(); it != groups.end(); it++ )
+    const QStringList groups = config->groupList();
+    for ( QStringList::ConstIterator it = groups.constBegin(); it != groups.constEnd(); it++ )
         if ( (*it).startsWith( "Session: " ) )
             sessions << (*it).mid( 9 );
     return sessions;
@@ -967,3 +967,9 @@ void KSMServer::selectWm( const QString& kdewm )
     wm = smname.isEmpty() ? cfgwm : smname;
     wmCommands = cfgWmCommands;
 }
+
+void KSMServer::wmChanged()
+{
+    KGlobal::config()->reparseConfiguration();
+    selectWm( "" );
+}
