@@ -24,7 +24,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ******************************************************************/
 
 #include "shutdowndlg.h"
-#include "plasma/svg.h"
+#include "plasma/framesvg.h"
 #include "plasma/theme.h"
 
 #include <QBitmap>
@@ -339,7 +339,8 @@ KSMShutdownDlg::KSMShutdownDlg( QWidget* parent,
     m_btnLogout(0),
     m_btnHalt(0),
     m_btnReboot(0),
-    m_automaticallyDoSeconds(60)
+    m_automaticallyDoSeconds(30),
+    m_pictureWidth(0)
     // this is a WType_Popup on purpose. Do not change that! Not
     // having a popup here has severe side effects.
 {
@@ -357,22 +358,21 @@ KSMShutdownDlg::KSMShutdownDlg( QWidget* parent,
         (unsigned char *)"logoutdialog", strlen( "logoutdialog" ));
 
 //#endif
-    m_svg = new Plasma::Svg(this);
+    m_svg = new Plasma::FrameSvg(this);
     m_svg->setImagePath("dialogs/shutdowndialog");
     connect( m_svg, SIGNAL(repaintNeeded()), this, SLOT(update()) );
     setModal( true );
-    resize(400, 220);
-    KDialog::centerOnScreen(this);
 
     QVBoxLayout *mainLayout = new QVBoxLayout();
 
     mainLayout->setContentsMargins(12, 9, 12, 7);
+
     QVBoxLayout *buttonLayout = new QVBoxLayout();
     QHBoxLayout *buttonMainLayout = new QHBoxLayout();
-    if (m_svg->hasElement("picture")) {
-        buttonMainLayout->addSpacing(m_svg->elementRect("picture").toRect().right() + 12);
-    }
-    buttonMainLayout->addStretch();
+
+    m_automaticallyDoLabel = new QLabel(this);
+    mainLayout->addWidget(m_automaticallyDoLabel, 0, Qt::AlignRight);
+
     buttonMainLayout->addLayout(buttonLayout);
 
     QHBoxLayout *bottomLayout = new QHBoxLayout();
@@ -385,7 +385,8 @@ KSMShutdownDlg::KSMShutdownDlg( QWidget* parent,
 
     m_btnLogout = new KSMPushButton( i18n("&Logout"), this );
     m_btnLogout->setPixmap(KIconLoader::global()->loadIcon("system-log-out", KIconLoader::NoGroup, 32));
-    m_btnLogout->setFocus();
+    if ( sdtype == KWorkSpace::ShutdownTypeLogout )
+        m_btnLogout->setFocus();
     connect(m_btnLogout, SIGNAL(clicked()), SLOT(slotLogout()));
     buttonLayout->addWidget(m_btnLogout, Qt::AlignRight | Qt::AlignTop);
 
@@ -436,7 +437,7 @@ KSMShutdownDlg::KSMShutdownDlg( QWidget* parent,
             m_btnReboot->setPopupMenu( rebootMenu );
 
             int index = 0;
-            for (QStringList::ConstIterator it = rebootOptions.begin(); it != rebootOptions.end(); ++it, ++index) {
+            for (QStringList::ConstIterator it = rebootOptions.constBegin(); it != rebootOptions.constEnd(); ++it, ++index) {
                 QString label = (*it);
                 label=label.replace('&',"&&");
                 QAction* action = new QAction(label, rebootActionGroup);
@@ -449,21 +450,32 @@ KSMShutdownDlg::KSMShutdownDlg( QWidget* parent,
         }
     }
 
-    KSMPushButton* btnBack = new KSMPushButton(i18n("&Cancel"), this, true);
+    if ( sdtype == KWorkSpace::ShutdownTypeLogout ) {
+        m_btnReboot->setHidden(true);
+        m_btnHalt->setHidden(true);
+    }
+    else if ( sdtype == KWorkSpace::ShutdownTypeHalt ) {
+        m_btnReboot->setHidden(true);
+        m_btnLogout->setHidden(true);
+    }
+    else if ( sdtype == KWorkSpace::ShutdownTypeReboot ) {
+        m_btnHalt->setHidden(true);
+        m_btnLogout->setHidden(true);
+    }
+
+    btnBack = new KSMPushButton(i18n("&Cancel"), this, true);
     btnBack->setPixmap(KIconLoader::global()->loadIcon( "dialog-cancel", KIconLoader::NoGroup, 16));
 
-    m_automaticallyDoLabel = new QLabel(this);
     m_automaticallyDoLabel->setPalette(palette);
     fnt.setPixelSize(11);
     m_automaticallyDoLabel->setFont(fnt);
-    m_automaticallyDoLabel->setWordWrap(true);
     automaticallyDoTimeout();
 
     QTimer *automaticallyDoTimer = new QTimer(this);
     connect(automaticallyDoTimer, SIGNAL(timeout()), this, SLOT(automaticallyDoTimeout()));
     automaticallyDoTimer->start(1000);
 
-    bottomLayout->addWidget(m_automaticallyDoLabel, 1, Qt::AlignBottom);
+    bottomLayout->addStretch();
     bottomLayout->addWidget(btnBack);
     connect(btnBack, SIGNAL(clicked()), SLOT(reject()));
 
@@ -471,7 +483,34 @@ KSMShutdownDlg::KSMShutdownDlg( QWidget* parent,
     mainLayout->addSpacing(9);
     mainLayout->addLayout(bottomLayout);
 
-    setLayout( mainLayout );
+    setLayout(mainLayout);
+    adjustSize();
+    if (m_svg->hasElement("picture")) {
+        QRect pictRect = m_svg->elementRect("picture").toRect();
+
+        if (pictRect.height() < 1 || pictRect.width() < 1) {
+            m_pictureWidth = 0;
+        } else if (height() > width()) {
+            m_pictureWidth = width();
+        } else {
+            m_svg->isValid();
+            m_pictureWidth = mainLayout->sizeHint().height() * (pictRect.width() / pictRect.height());
+            //kDebug() << "blurk!" << buttonMainLayout->sizeHint().height() << pictRect;
+        }
+
+        //kDebug() << width() << m_pictureWidth;
+        //FIXME: this spaces will be taken from framesvg borders
+        if (m_pictureWidth > 0) {
+            const int extraSpace = 18;
+            buttonMainLayout->insertSpacing(0, m_pictureWidth + extraSpace);
+        }
+        //resize(width() + m_pictureWidth, height());
+        //kDebug() << width();
+    } else {
+        m_pictureWidth = 0;
+    }
+
+    KDialog::centerOnScreen(this);
 }
 
 void KSMShutdownDlg::automaticallyDoTimeout()
@@ -482,14 +521,14 @@ void KSMShutdownDlg::automaticallyDoTimeout()
                 focusedButton->click();
         // following code is required to provide a clean way to translate strings
         } else if (focusedButton == m_btnLogout) {
-            m_automaticallyDoLabel->setText(i18np("Log out in 1 second.",
-                                            "Log out in %1 seconds.", m_automaticallyDoSeconds));
+            m_automaticallyDoLabel->setText(i18np("Logging out in 1 second.",
+                                            "Logging out in %1 seconds.", m_automaticallyDoSeconds));
         } else if (focusedButton == m_btnHalt) {
-                m_automaticallyDoLabel->setText(i18np("Turn off computer in 1 second.",
-                                                      "Turn off computer in %1 seconds.", m_automaticallyDoSeconds));
+                m_automaticallyDoLabel->setText(i18np("Turning off computer in 1 second.",
+                                                      "Turning off computer in %1 seconds.", m_automaticallyDoSeconds));
         } else if (focusedButton == m_btnReboot) {
-                m_automaticallyDoLabel->setText(i18np("Reboot computer in 1 second.",
-                                                      "Reboot computer in %1 seconds.", m_automaticallyDoSeconds));
+                m_automaticallyDoLabel->setText(i18np("Restarting computer in 1 second.",
+                                                      "Restarting computer in %1 seconds.", m_automaticallyDoSeconds));
         } else {
             m_automaticallyDoLabel->setText(QString());
         }
@@ -509,13 +548,44 @@ void KSMShutdownDlg::paintEvent(QPaintEvent *e)
     p.setClipRect(e->rect());
 
     p.fillRect(QRect(0, 0, width(), height()), Qt::transparent);
-    m_svg->paint(&p, QRect(0, 0, width(), height()), "background");
 
-    if (m_svg->hasElement("picture")) {
-        QRect r = m_svg->elementRect("picture").toRect();
-        r.moveTop(m_btnLogout->geometry().top());
+    if (m_svg->hasElement("center")) {
+        m_svg->resizeFrame(size());
+        m_svg->paintFrame(&p);
+    } else {
+        m_svg->paint(&p, QRect(0, 0, width(), height()), "background");
+    }
+
+    if (m_pictureWidth > 0) { // implies hasElement("picture")
+        QRect r = layout()->geometry();
+        r.setWidth(m_pictureWidth);
+
+        m_svg->resize(m_svg->elementRect("picture").size());
+        QPixmap picture = m_svg->pixmap("picture");
+        m_svg->resize();
+
+        //kDebug() << 1 << r << picture.size();
+        if (r.width() < picture.width()) {
+            picture = picture.scaledToWidth(r.width(), Qt::SmoothTransformation);
+        }
+
+        if (r.height() < picture.height()) {
+            picture = picture.scaledToHeight(r.height(), Qt::SmoothTransformation);
+        }
+
+
+        int left = (r.height() - picture.height())/2;
+        if (QApplication::isLeftToRight()) {
+            r.moveLeft(left);
+        } else {
+            r.moveRight(layout()->geometry().width() - left);
+        }
+
+        //kDebug() << 2 << r << picture.size();
+        QRect dest = picture.rect();
+        dest.moveCenter(r.center());
         p.setCompositionMode( QPainter::CompositionMode_SourceOver );
-        m_svg->paint(&p, r, "picture");
+        p.drawPixmap(dest, picture, picture.rect());
     }
 }
 
