@@ -52,10 +52,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "scene_xrender.h"
 #include "scene_opengl.h"
 #include "compositingprefs.h"
+#include "notifications.h"
 
 #include <stdio.h>
 
 #include <QMenu>
+#include <kaction.h>
+#include <kactioncollection.h>
+#include <klocale.h>
 #include <kxerrorhandler.h>
 
 #include <X11/extensions/shape.h>
@@ -84,7 +88,7 @@ void Workspace::setupCompositing()
         return;
     if( !options->useCompositing && getenv( "KWIN_COMPOSE") == NULL )
         {
-        kDebug( 1212 ) << "Compositing is turned off in options";
+        kDebug( 1212 ) << "Compositing is turned off in options or disabled";
         return;
         }
     else if( compositingSuspended )
@@ -183,13 +187,17 @@ void Workspace::setupCompositing()
     if( rate <= 0 )
         rate = 50;
     // QTimer gives us 1msec (1000Hz) at best, so we ignore anything higher;
-    // however, additional throttling makes prevents very high rates from taking place anyway
+    // however, additional throttling prevents very high rates from taking place anyway
     else if( rate > 1000 )
         rate = 1000;
     kDebug( 1212 ) << "Refresh rate " << rate << "Hz";
     compositeRate = 1000 / rate;
-    compositeTimer.start( compositeRate );
     lastCompositePaint.start();
+    // fake a previous paint, so that the next starts right now
+    nextPaintReference = QTime::currentTime().addMSecs( -compositeRate );
+    compositeTimer.setSingleShot( true );
+    checkCompositeTimer();
+    composite_paint_times.clear();
     XCompositeRedirectSubwindows( display(), rootWindow(), CompositeRedirectManual );
     new EffectsHandlerImpl( scene->compositingType() ); // sets also the 'effects' pointer
     addRepaintFull();
@@ -234,19 +242,19 @@ void Workspace::finishCompositing()
     foreach( Deleted* c, deleted )
         c->finishCompositing();
     XCompositeUnredirectSubwindows( display(), rootWindow(), CompositeRedirectManual );
-    compositeTimer.stop();
     delete effects;
     effects = NULL;
     delete scene;
     scene = NULL;
+    compositeTimer.stop();
     repaints_region = QRegion();
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it )
         { // forward all opacity values to the frame in case there'll be other CM running
         if( (*it)->opacity() != 1.0 )
             {
-            NETWinInfo i( display(), (*it)->frameId(), rootWindow(), 0 );
+            NETWinInfo2 i( display(), (*it)->frameId(), rootWindow(), 0 );
             i.setOpacity( static_cast< unsigned long >((*it)->opacity() * 0xffffffff ));
             }
         }
@@ -266,7 +274,17 @@ void Workspace::lostCMSelection()
 // for the shortcut
 void Workspace::slotToggleCompositing()
     {
-    compositingSuspended = !compositingSuspended;
+    suspendCompositing( !compositingSuspended );
+    }
+    
+void Workspace::suspendCompositing()
+    {
+    suspendCompositing( true );
+    }
+
+void Workspace::suspendCompositing( bool suspend )
+    {
+    compositingSuspended = suspend;
     finishCompositing();
     setupCompositing(); // will do nothing if suspended
     }
@@ -276,6 +294,7 @@ void Workspace::addRepaint( int x, int y, int w, int h )
     if( !compositing())
         return;
     repaints_region += QRegion( x, y, w, h );
+    checkCompositeTimer();
     }
 
 void Workspace::addRepaint( const QRect& r )
@@ -283,6 +302,7 @@ void Workspace::addRepaint( const QRect& r )
     if( !compositing())
         return;
     repaints_region += r;
+    checkCompositeTimer();
     }
     
 void Workspace::addRepaint( const QRegion& r )
@@ -290,6 +310,7 @@ void Workspace::addRepaint( const QRegion& r )
     if( !compositing())
         return;
     repaints_region += r;
+    checkCompositeTimer();
     }
     
 void Workspace::addRepaintFull()
@@ -297,6 +318,7 @@ void Workspace::addRepaintFull()
     if( !compositing())
         return;
     repaints_region = QRegion( 0, 0, displayWidth(), displayHeight());
+    checkCompositeTimer();
     }
 
 void Workspace::performCompositing()
@@ -308,21 +330,27 @@ void Workspace::performCompositing()
     // them - leave at least 1msec time after one repaint is finished and next one
     // is started.
     if( lastCompositePaint.elapsed() < 1 )
+        {
+        compositeTimer.start( 1 );
         return;
+        }
+    if( !scene->waitSyncAvailable())
+        nextPaintReference = QTime::currentTime();
     checkCursorPos();
-    if(( repaints_region.isEmpty() && !windowRepaintsPending()) // no damage
+    if((( repaints_region.isEmpty() && !windowRepaintsPending()) // no damage
         || !overlay_visible ) // nothing is visible anyway
+        // HACK: don't idle during active full screen effect so that mouse events are not dropped (bug #177226)
+        && !static_cast< EffectsHandlerImpl* >( effects )->activeFullScreenEffect() )
         {
         scene->idle();
-        // With vsync, next repaint is scheduled dynamically at the end of this function,
-        // and it can have a very short timeout. If we now idle here, make sure the idling
-        // does not actually caused heavy load by firing the timer often too quickly.
-        if( compositeTimer.interval() != compositeRate )
-            compositeTimer.start( compositeRate );
+        // Note: It would seem here we should undo suspended unredirect, but when scenes need
+        // it for some reason, e.g. transformations or translucency, the next pass that does not
+        // need this anymore and paints normally will also reset the suspended unredirect.
+        // Otherwise the window would not be painted normally anyway.
         return;
         }
     // create a list of all windows in the stacking order
-    ToplevelList windows = compositingStackingOrder();
+    ToplevelList windows = xStackingOrder();
     foreach( EffectWindow* c, static_cast< EffectsHandlerImpl* >( effects )->elevatedWindows())
         {
         Toplevel* t = static_cast< EffectWindowImpl* >( c )->window();
@@ -335,7 +363,7 @@ void Workspace::performCompositing()
 #if 0
     // There is a bug somewhere that prevents this from working properly (#160393), but additionally
     // this cannot be used so carelessly - needs protections against broken clients, the window
-    // should not get focus before it's displayed and so on.
+    // should not get focus before it's displayed, handle unredirected windows properly and so on.
     foreach( Toplevel* c, tmp )
         if( c->readyForPainting())
             windows.append( c );
@@ -354,14 +382,24 @@ void Workspace::performCompositing()
     QRegion repaints = repaints_region;
     // clear all repaints, so that post-pass can add repaints for the next repaint
     repaints_region = QRegion();
+    QTime t = QTime::currentTime();
     scene->paint( repaints, windows );
     if( scene->waitSyncAvailable())
         {
-        // if vsync is used, schedule the next repaint slightly in advance of the next sync,
-        // so that there is still time for the drawing to take place
-        int untilNextSync = compositeRate - ( lastCompositePaint.elapsed() % compositeRate );
-        compositeTimer.start( qMax( 1, untilNextSync - 10 )); // 10 ms in advance - TODO maybe less?
+        // If vsync is used, schedule the next repaint slightly in advance of the next sync,
+        // so that there is still time for the drawing to take place. We have just synced, and
+        // nextPaintReference is time from which multiples of compositeRate should be added,
+        // so set it 10ms back (meaning next paint will be in 'compositeRate - 10').
+        // However, make sure the reserve is smaller than the composite rate.
+        int reserve = compositeRate <= 10 ? compositeRate - 1 : 10;
+        nextPaintReference = QTime::currentTime().addMSecs( -reserve );
         }
+    // Trigger at least one more pass even if there would be nothing to paint, so that scene->idle()
+    // is called the next time. If there would be nothing pending, it will not restart the timer and
+    // checkCompositeTime() would restart it again somewhen later, called from functions that
+    // would again add something pending.
+    checkCompositeTimer();
+    checkCompositePaintTime( t.elapsed());
     lastCompositePaint.start();
 #endif
     }
@@ -383,6 +421,15 @@ bool Workspace::windowRepaintsPending() const
     return false;
     }
 
+void Workspace::setCompositeTimer()
+    {
+    if( !compositing()) // should not really happen, but there may be e.g. some damage events still pending
+        return;
+    // The last paint set nextPaintReference as a reference time to which multiples of compositeRate
+    // should be added for the next paint. qBound() for protection; system time can change without notice.
+    compositeTimer.start( qBound( 0, nextPaintReference.msecsTo( QTime::currentTime() ), 250 ) % compositeRate );
+    }
+
 bool Workspace::createOverlay()
     {
     assert( overlay == None );
@@ -400,13 +447,72 @@ bool Workspace::createOverlay()
 #endif
     }
 
+void Workspace::checkCompositePaintTime( int msec )
+    {
+    if( options->disableCompositingChecks )
+        return;
+    composite_paint_times.prepend( msec );
+    bool tooslow = false;
+    // If last 3 paints were way too slow, disable and warn.
+    // 1 second seems reasonable, it's not that difficult to get relatively high times
+    // with high system load.
+    const int MAX_LONG_PAINT = 1000;
+    if( composite_paint_times.count() >= 3 && composite_paint_times[ 0 ] > MAX_LONG_PAINT
+        && composite_paint_times[ 1 ] > MAX_LONG_PAINT && composite_paint_times[ 2 ] > MAX_LONG_PAINT )
+        {
+        kDebug( 1212 ) << "Too long paint times, suspending";
+        tooslow = true;
+        }
+    // If last 15 seconds all paints (all of them) were quite slow, disable and warn too. Quite slow being 0,1s
+    // should be reasonable, that's 10fps and having constant 10fps is bad.
+    // This may possibly trigger also when activating an expensive effect, so this may need tweaking.
+    const int MAX_SHORT_PAINT = 100;
+    const int SHORT_TIME = 15000; // 15 sec
+    int time = 0;
+    foreach( int t, composite_paint_times )
+        {
+        if( t < MAX_SHORT_PAINT )
+            break;
+        time += t;
+        if( time > SHORT_TIME ) // all paints in the given time were long
+            {
+            kDebug( 1212 ) << "Long paint times for long time, suspending";
+            tooslow = true;
+            break;
+            }
+        }
+    if( composite_paint_times.count() > 1000 )
+        composite_paint_times.removeLast();
+    if( tooslow )
+        {
+        QTimer::singleShot( 0, this, SLOT( suspendCompositing()));
+        QString shortcut, message;
+        if( KAction* action = qobject_cast<KAction*>( keys->action("Suspend Compositing")))
+            shortcut = action->globalShortcut().primary().toString(QKeySequence::NativeText);
+        if (shortcut.isEmpty())
+            message = i18n( "Compositing was too slow and has been suspended.\n"
+                "You can disable functionality checks in advanced compositing settings." );
+        else
+            message = i18n( "Compositing was too slow and has been suspended.\n"
+                "If this was only a temporary problem, you can resume using the '%1' shortcut.\n"
+                "You can also disable functionality checks in advanced compositing settings.", shortcut );
+        Notify::raise( Notify::CompositingSlow, message );
+        compositeTimer.start( 1000 ); // so that it doesn't trigger sooner than suspendCompositing()
+        }
+    }
+
 void Workspace::setupOverlay( Window w )
     {
     assert( overlay != None );
     assert( Extensions::shapeInputAvailable());
-    XShapeCombineRectangles( display(), overlay, ShapeInput, 0, 0, NULL, 0, ShapeSet, Unsorted );
+    XSetWindowBackgroundPixmap( display(), overlay, None );
+    overlay_shape = QRegion();
+    setOverlayShape( QRect( 0, 0, displayWidth(), displayHeight()));
     if( w != None )
+        {
+        XSetWindowBackgroundPixmap( display(), w, None );
         XShapeCombineRectangles( display(), w, ShapeInput, 0, 0, NULL, 0, ShapeSet, Unsorted );
+        }
     XSelectInput( display(), overlay, VisibilityChangeMask );
     }
 
@@ -420,10 +526,46 @@ void Workspace::showOverlay()
     overlay_shown = true;
     }
 
+void Workspace::hideOverlay()
+    {
+    assert( overlay != None );
+    XUnmapWindow( display(), overlay );
+    overlay_shown = false;
+    setOverlayShape( QRect( 0, 0, displayWidth(), displayHeight()));
+    }
+
+void Workspace::setOverlayShape( const QRegion& reg )
+    {
+    // Avoid setting the same shape again, it causes flicker (apparently it is not a no-op
+    // and triggers something).
+    if( reg == overlay_shape )
+        return;
+    QVector< QRect > rects = reg.rects();
+    XRectangle* xrects = new XRectangle[ rects.count() ];
+    for( int i = 0;
+         i < rects.count();
+         ++i )
+        {
+        xrects[ i ].x = rects[ i ].x();
+        xrects[ i ].y = rects[ i ].y();
+        xrects[ i ].width = rects[ i ].width();
+        xrects[ i ].height = rects[ i ].height();
+        }
+    XShapeCombineRectangles( display(), overlay, ShapeBounding, 0, 0,
+        xrects, rects.count(), ShapeSet, Unsorted );
+    delete[] xrects;
+    XShapeCombineRectangles( display(), overlay, ShapeInput, 0, 0, NULL, 0, ShapeSet, Unsorted );
+    overlay_shape = reg;
+    }
+
 void Workspace::destroyOverlay()
     {
     if( overlay == None )
         return;
+    // reset the overlay shape
+    XRectangle rec = { 0, 0, displayWidth(), displayHeight() };
+    XShapeCombineRectangles( display(), overlay, ShapeBounding, 0, 0, &rec, 1, ShapeSet, Unsorted );
+    XShapeCombineRectangles( display(), overlay, ShapeInput, 0, 0, &rec, 1, ShapeSet, Unsorted );
 #ifdef HAVE_XCOMPOSITE_OVERLAY
     XCompositeReleaseOverlayWindow( display(), overlay );
 #endif
@@ -431,6 +573,52 @@ void Workspace::destroyOverlay()
     overlay_shown = false;
     }
 
+bool Workspace::compositingActive()
+    {
+    return compositing();
+    }
+
+// force is needed when the list of windows changes (e.g. a window goes away)
+void Workspace::checkUnredirect( bool force )
+    {
+    if( !compositing() || overlay == None || !options->unredirectFullscreen )
+        return;
+    if( force )
+        forceUnredirectCheck = true;
+    if( !unredirectTimer.isActive())
+        unredirectTimer.start( 0 );
+    }
+
+void Workspace::delayedCheckUnredirect()
+    {
+    if( !compositing() || overlay == None || !options->unredirectFullscreen )
+        return;
+    ToplevelList list;
+    bool changed = forceUnredirectCheck;
+    foreach( Client* c, clients )
+        list.append( c );
+    foreach( Unmanaged* c, unmanaged )
+        list.append( c );
+    foreach( Toplevel* c, list )
+        {
+        if( c->updateUnredirectedState())
+            changed = true;
+        }
+    // no desktops, no Deleted ones
+    if( !changed )
+        return;
+    forceUnredirectCheck = false;
+    // Cut out parts from the overlay window where unredirected windows are,
+    // so that they are actually visible.
+    QRegion reg( 0, 0, displayWidth(), displayHeight());
+    foreach( Toplevel* c, list )
+        {
+        if( c->unredirected())
+            reg -= c->geometry();
+        }
+    setOverlayShape( reg );
+    }
+
 //****************************************
 // Toplevel
 //****************************************
@@ -446,6 +634,8 @@ void Toplevel::setupCompositing()
     damage_region = QRegion( 0, 0, width(), height());
     effect_window = new EffectWindowImpl();
     effect_window->setWindow( this );
+    unredirect = false;
+    workspace()->checkUnredirect( true );
 #endif
     }
 
@@ -454,6 +644,7 @@ void Toplevel::finishCompositing()
 #ifdef KWIN_HAVE_COMPOSITING
     if( damage_handle == None )
         return;
+    workspace()->checkUnredirect( true );
     if( effect_window->window() == this ) // otherwise it's already passed to Deleted, don't free data
         {
         discardWindowPixmap();
@@ -482,9 +673,11 @@ Pixmap Toplevel::createWindowPixmap()
     {
 #ifdef KWIN_HAVE_COMPOSITING
     assert( compositing());
+    if( unredirected())
+        return None;
     grabXServer();
     KXErrorHandler err;
-    window_pix = XCompositeNameWindowPixmap( display(), frameId());
+    Pixmap pix = XCompositeNameWindowPixmap( display(), frameId());
     // check that the received pixmap is valid and actually matches what we
     // know about the window (i.e. size)
     XWindowAttributes attrs;
@@ -493,11 +686,11 @@ Pixmap Toplevel::createWindowPixmap()
         || attrs.width != width() || attrs.height != height() || attrs.map_state != IsViewable )
         {
         kDebug( 1212 ) << "Creating window pixmap failed: " << this;
-        XFreePixmap( display(), window_pix );
-        window_pix = None;
+        XFreePixmap( display(), pix );
+        pix = None;
         }
     ungrabXServer();
-    return window_pix;
+    return pix;
 #else
     return None;
 #endif
@@ -545,7 +738,7 @@ void Toplevel::damageNotifyEvent( XDamageNotifyEvent* e )
             }
         break;
         }
-    foreach( QRect r, damage.rects())
+    foreach( const QRect& r, damage.rects())
         addDamage( r ); 
     }
 
@@ -577,6 +770,7 @@ void Toplevel::addDamage( int x, int y, int w, int h )
     damage_region += r;
     repaints_region += r;
     static_cast<EffectsHandlerImpl*>(effects)->windowDamaged( effectWindow(), r );
+    workspace()->checkCompositeTimer();
     }
 
 void Toplevel::addDamageFull()
@@ -586,6 +780,7 @@ void Toplevel::addDamageFull()
     damage_region = rect();
     repaints_region = rect();
     static_cast<EffectsHandlerImpl*>(effects)->windowDamaged( effectWindow(), rect());
+    workspace()->checkCompositeTimer();
     }
 
 void Toplevel::resetDamage( const QRect& r )
@@ -605,11 +800,13 @@ void Toplevel::addRepaint( int x, int y, int w, int h )
     QRect r( x, y, w, h );
     r &= rect();
     repaints_region += r;
+    workspace()->checkCompositeTimer();
     }
 
 void Toplevel::addRepaintFull()
     {
     repaints_region = rect();
+    workspace()->checkCompositeTimer();
     }
 
 void Toplevel::resetRepaints( const QRect& r )
@@ -632,4 +829,114 @@ void Toplevel::addWorkspaceRepaint( const QRect& r2 )
     workspace()->addRepaint( r );
     }
 
+bool Toplevel::updateUnredirectedState()
+    {
+    assert( compositing());
+    bool should = shouldUnredirect() && !unredirectSuspend && !shape() && !hasAlpha() && opacity() == 1.0 &&
+        !static_cast<EffectsHandlerImpl*>( effects )->activeFullScreenEffect();
+    if( should && !unredirect )
+        {
+        unredirect = true;
+        kDebug( 1212 ) << "Unredirecting:" << this;
+#ifdef HAVE_XCOMPOSITE
+        XCompositeUnredirectWindow( display(), frameId(), CompositeRedirectManual );
+#endif
+        return true;
+        }
+    else if( !should && unredirect )
+        {
+        unredirect = false;
+        kDebug( 1212 ) << "Redirecting:" << this;
+#ifdef HAVE_XCOMPOSITE
+        XCompositeRedirectWindow( display(), frameId(), CompositeRedirectManual );
+#endif
+        discardWindowPixmap();
+        return true;
+        }
+    return false;
+    }
+
+void Toplevel::suspendUnredirect( bool suspend )
+    {
+    if( unredirectSuspend == suspend )
+        return;
+    unredirectSuspend = suspend;
+    workspace()->checkUnredirect();
+    }
+
+//****************************************
+// Client
+//****************************************
+
+void Client::setupCompositing()
+    {
+    Toplevel::setupCompositing();
+    updateVisibility(); // for internalKeep()
+    }
+
+void Client::finishCompositing()
+    {
+    Toplevel::finishCompositing();
+    updateVisibility();
+    }
+
+bool Client::shouldUnredirect() const
+    {
+    if( isActiveFullScreen())
+        {
+        ToplevelList stacking = workspace()->xStackingOrder();
+        for( int pos = stacking.count() - 1;
+             pos >= 0;
+             --pos )
+            {
+            Toplevel* c = stacking.at( pos );
+            if( c == this ) // is not covered by any other window, ok to unredirect
+                return true;
+            if( c->geometry().intersects( geometry()))
+                return false;
+            }
+        abort();
+        }
+    return false;
+    }
+
+//****************************************
+// Unmanaged
+//****************************************
+
+bool Unmanaged::shouldUnredirect() const
+    {
+    // the pixmap is needed for the login effect, a nicer solution would be the login effect increasing
+    // refcount for the window pixmap (which would prevent unredirect), avoiding this hack
+    if( resourceClass() == "ksplashx" || resourceClass() == "ksplashsimple" )
+        return false;
+// it must cover whole display or one xinerama screen, and be the topmost there
+    if( geometry() == workspace()->clientArea( FullArea, geometry().center(), workspace()->currentDesktop())
+        || geometry() == workspace()->clientArea( ScreenArea, geometry().center(), workspace()->currentDesktop()))
+        {
+        ToplevelList stacking = workspace()->xStackingOrder();
+        for( int pos = stacking.count() - 1;
+             pos >= 0;
+             --pos )
+            {
+            Toplevel* c = stacking.at( pos );
+            if( c == this ) // is not covered by any other window, ok to unredirect
+                return true;
+            if( c->geometry().intersects( geometry()))
+                return false;
+            }
+        abort();
+        }
+    return false;
+    }
+
+//****************************************
+// Deleted
+//****************************************
+
+bool Deleted::shouldUnredirect() const
+    {
+    return false;
+    }
+
 } // namespace
