@@ -51,6 +51,7 @@ EffectsHandlerImpl::EffectsHandlerImpl(CompositingType type)
     : EffectsHandler(type)
     , keyboard_grab_effect( NULL )
     , fullscreen_effect( 0 )
+    , next_window_quad_type( EFFECT_QUAD_TYPE_START )
     {
     reconfigure();
     }
@@ -85,14 +86,21 @@ void EffectsHandlerImpl::reconfigure()
         if( shouldbeloaded )
             effectsToBeLoaded.append( plugininfo.pluginName() );
         }
+    QStringList newLoaded;
     // Then load those that should be loaded
     foreach( const QString &effectName, effectsToBeLoaded )
         {
         if( !isEffectLoaded( effectName ))
             {
             loadEffect( effectName );
+            newLoaded.append( effectName );
             }
         }
+    foreach( const EffectPair &ep, loaded_effects )
+        {
+        if( !newLoaded.contains( ep.first )) // don't reconfigure newly loaded effects
+            ep.second->reconfigure( Effect::ReconfigureAll );
+        }
     }
 
 // the idea is that effects call this function again which calls the next one
@@ -169,12 +177,37 @@ void EffectsHandlerImpl::drawWindow( EffectWindow* w, int mask, QRegion region,
         scene->finalDrawWindow( static_cast<EffectWindowImpl*>( w ), mask, region, data );
     }
 
+void EffectsHandlerImpl::buildQuads( EffectWindow* w, WindowQuadList& quadList )
+    {
+    if( current_build_quads < loaded_effects.size())
+        {
+        loaded_effects[current_build_quads++].second->buildQuads( w, quadList );
+        --current_build_quads;
+        }
+    }
+
+bool EffectsHandlerImpl::hasDecorationShadows() const
+    {
+    return Workspace::self()->hasDecorationShadows();
+    }
+
+QList< QList<QImage> > EffectsHandlerImpl::shadowTextures()
+    {
+    return Workspace::self()->decorationShadowTextures();
+    }
+
+int EffectsHandlerImpl::shadowTextureList( ShadowType type ) const
+    {
+    return Workspace::self()->decorationShadowTextureList( type );
+    }
+
 // start another painting pass
 void EffectsHandlerImpl::startPaint()
     {
     assert( current_paint_screen == 0 );
     assert( current_paint_window == 0 );
     assert( current_draw_window == 0 );
+    assert( current_build_quads == 0 );
     assert( current_transform == 0 );
     }
 
@@ -272,6 +305,7 @@ void EffectsHandlerImpl::tabBoxUpdated()
 void EffectsHandlerImpl::setActiveFullScreenEffect( Effect* e )
     {
     fullscreen_effect = e;
+    Workspace::self()->checkUnredirect();
     }
 
 Effect* EffectsHandlerImpl::activeFullScreenEffect() const
@@ -419,6 +453,16 @@ int EffectsHandlerImpl::desktopDown( int desktop, bool wrap ) const
     return Workspace::self()->desktopDown( desktop, wrap );
     }
 
+double EffectsHandlerImpl::animationTimeFactor() const
+    {
+    return options->animationTimeFactor();
+    }
+
+WindowQuadType EffectsHandlerImpl::newWindowQuadType()
+    {
+    return WindowQuadType( next_window_quad_type++ );
+    }
+
 int EffectsHandlerImpl::displayWidth() const
     {
     return KWin::displayWidth();
@@ -607,7 +651,7 @@ void EffectsHandlerImpl::destroyInputWindow( Window w )
             return;
             }
         }
-    assert( false );
+    abort();
     }
 
 bool EffectsHandlerImpl::checkInputWindowEvent( XEvent* e )
@@ -755,6 +799,7 @@ bool EffectsHandlerImpl::loadEffect( const QString& name )
     assert( current_paint_screen == 0 );
     assert( current_paint_window == 0 );
     assert( current_draw_window == 0 );
+    assert( current_build_quads == 0 );
     assert( current_transform == 0 );
 
     if( !name.startsWith("kwin4_effect_") )
@@ -839,7 +884,7 @@ bool EffectsHandlerImpl::loadEffect( const QString& name )
         {
         if( !loadEffect(depName))
             {
-            kError() << "EffectsHandler::loadEffect : Couldn't load dependencies for effect " << name << endl;
+            kError(1212) << "EffectsHandler::loadEffect : Couldn't load dependencies for effect " << name << endl;
             library->unload();
             return false;
             }
@@ -860,6 +905,7 @@ void EffectsHandlerImpl::unloadEffect( const QString& name )
     assert( current_paint_screen == 0 );
     assert( current_paint_window == 0 );
     assert( current_draw_window == 0 );
+    assert( current_build_quads == 0 );
     assert( current_transform == 0 );
 
     for( QMap< int, EffectPair >::iterator it = effect_order.begin(); it != effect_order.end(); ++it)
@@ -882,13 +928,14 @@ void EffectsHandlerImpl::unloadEffect( const QString& name )
     kDebug( 1212 ) << "EffectsHandler::unloadEffect : Effect not loaded : " << name;
     }
 
-void EffectsHandlerImpl::reloadEffect( const QString& name )
+void EffectsHandlerImpl::reconfigureEffect( const QString& name )
     {
-    if( isEffectLoaded( name ))
-        {
-        unloadEffect( name );
-        loadEffect( name );
-        }
+    for( QVector< EffectPair >::iterator it = loaded_effects.begin(); it != loaded_effects.end(); ++it)
+        if ( (*it).first == name )
+            {
+            (*it).second->reconfigure( Effect::ReconfigureAll );
+            return;
+            }
     }
 
 bool EffectsHandlerImpl::isEffectLoaded( const QString& name )
@@ -1144,6 +1191,13 @@ bool EffectWindowImpl::isMovable() const
     return false;
     }
 
+bool EffectWindowImpl::isMovableAcrossScreens() const
+    {
+    if( Client* c = dynamic_cast< Client* >( toplevel ))
+        return c->isMovableAcrossScreens();
+    return false;
+    }
+
 bool EffectWindowImpl::isUserMove() const
     {
     if( Client* c = dynamic_cast< Client* >( toplevel ))
@@ -1283,9 +1337,53 @@ EffectWindowList EffectWindowImpl::mainWindows() const
     return EffectWindowList();
     }
 
-WindowQuadList EffectWindowImpl::buildQuads() const
+QList<QRect> EffectWindowImpl::shadowQuads( ShadowType type ) const
     {
-    return sceneWindow()->buildQuads();
+    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
+        {
+        if( Client* c = dynamic_cast< Client* >( toplevel ))
+            return c->shadowQuads( type );
+        return QList<QRect>();
+        }
+    return toplevel->workspace()->decorationShadowQuads( type, toplevel->size() );
+    }
+
+double EffectWindowImpl::shadowOpacity( ShadowType type ) const
+    {
+    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
+        {
+        if( Client* c = dynamic_cast< Client* >( toplevel ))
+            return c->shadowOpacity( type );
+        return 1.0;
+        }
+    return toplevel->workspace()->decorationShadowOpacity( type );
+    }
+
+double EffectWindowImpl::shadowBrightness( ShadowType type ) const
+    {
+    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
+        {
+        if( Client* c = dynamic_cast< Client* >( toplevel ))
+            return c->shadowBrightness( type );
+        return 1.0;
+        }
+    return toplevel->workspace()->decorationShadowBrightness( type );
+    }
+
+double EffectWindowImpl::shadowSaturation( ShadowType type ) const
+    {
+    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
+        {
+        if( Client* c = dynamic_cast< Client* >( toplevel ))
+            return c->shadowSaturation( type );
+        return 1.0;
+        }
+    return toplevel->workspace()->decorationShadowSaturation( type );
+    }
+
+WindowQuadList EffectWindowImpl::buildQuads( bool force ) const
+    {
+    return sceneWindow()->buildQuads( force );
     }
 
 EffectWindow* effectWindow( Toplevel* w )
