@@ -38,23 +38,20 @@ License. See the file "COPYING" for the exact licensing terms.
 namespace KWin
 {
 
-static const qreal STIFFNESS = 0.06;
-static const qreal DRAG = 0.92;
-static const qreal MOVEFACTOR = 0.1;
+static const qreal STIFFNESS = 0.15;
+static const qreal DRAG = 0.80;
+static const qreal MOVEFACTOR = 0.10;
 
 static const int XTESSELATION = 20;
 static const int YTESSELATION = 20;
 
 static const qreal MINVELOCITY = 0.0;
 static const qreal MAXVELOCITY = 1000.0;
-static const qreal STOPVELOCITY = 3.0;
+static const qreal STOPVELOCITY = 0.5;
 static const qreal MINACCELERATION = 0.0;
 static const qreal MAXACCELERATION = 1000.0;
 static const qreal STOPACCELERATION = 5.0;
 
-static const char* VELOCITYFILTER = "HeightRingLinearMean";
-static const char* ACCELERATIONFILTER = "HeightRingLinearMean";
-
 struct ParameterSet
 {
     qreal stiffness;
@@ -64,9 +61,6 @@ struct ParameterSet
     qreal xTesselation;
     qreal yTesselation;
 
-    WobblyWindowsEffect::GridFilter velocityFilter;
-    WobblyWindowsEffect::GridFilter accelerationFilter;
-
     qreal minVelocity;
     qreal maxVelocity;
     qreal stopVelocity;
@@ -79,35 +73,13 @@ struct ParameterSet
     bool closeEffectEnabled;
 };
 
-ParameterSet set_0 =
-{
-    0.1,
-    0.8,
-    0.1,
-    20.0,
-    20.0,
-    WobblyWindowsEffect::FourRingLinearMean,
-    WobblyWindowsEffect::HeightRingLinearMean,
-    0.0,
-    1000.0,
-    0.5,
-    0.0,
-    1000.0,
-    0.5,
-    true,
-    false,
-    false
-};
-
-ParameterSet set_1 =
+static const ParameterSet set_0 =
 {
     0.15,
-    0.85,
-    0.1,
+    0.80,
+    0.10,
     20.0,
     20.0,
-    WobblyWindowsEffect::HeightRingLinearMean,
-    WobblyWindowsEffect::MeanWithMean,
     0.0,
     1000.0,
     0.5,
@@ -119,15 +91,31 @@ ParameterSet set_1 =
     false
 };
 
-ParameterSet set_2 =
+static const ParameterSet set_1 =
+{
+    0.10,
+    0.85,
+    0.10,
+    20.0,
+    20.0,
+    0.0,
+    1000.0,
+    0.5,
+    0.0,
+    1000.0,
+    0.5,
+    true,
+    false,
+    false
+};
+
+static const ParameterSet set_2 =
 {
     0.06,
-    0.9,
-    0.1,
+    0.90,
+    0.10,
     20.0,
     20.0,
-    WobblyWindowsEffect::HeightRingLinearMean,
-    WobblyWindowsEffect::NoFilter,
     0.0,
     1000.0,
     0.5,
@@ -139,15 +127,13 @@ ParameterSet set_2 =
     false
 };
 
-ParameterSet set_3 =
+static const ParameterSet set_3 =
 {
     0.03,
     0.92,
-    0.2,
+    0.20,
     20.0,
     20.0,
-    WobblyWindowsEffect::HeightRingLinearMean,
-    WobblyWindowsEffect::HeightRingLinearMean,
     0.0,
     1000.0,
     0.5,
@@ -159,15 +145,13 @@ ParameterSet set_3 =
     false
 };
 
-ParameterSet set_4 =
+static const ParameterSet set_4 =
 {
     0.01,
     0.97,
     0.25,
     20.0,
     20.0,
-    WobblyWindowsEffect::HeightRingLinearMean,
-    WobblyWindowsEffect::HeightRingLinearMean,
     0.0,
     1000.0,
     0.5,
@@ -179,11 +163,31 @@ ParameterSet set_4 =
     false
 };
 
-ParameterSet pset[5] = { set_0, set_1, set_2, set_3, set_4 };
+static const ParameterSet pset[5] = { set_0, set_1, set_2, set_3, set_4 };
 
 KWIN_EFFECT(wobblywindows, WobblyWindowsEffect)
 
 WobblyWindowsEffect::WobblyWindowsEffect()
+{
+    reconfigure( ReconfigureAll );
+}
+
+WobblyWindowsEffect::~WobblyWindowsEffect()
+{
+    if (!windows.empty())
+    {
+        // we should be empty at this point...
+        // emit a warning and clean the list.
+        kDebug(1212) << "Windows list not empty. Left items : " << windows.count();
+        QHash< const EffectWindow*,  WindowWobblyInfos >::iterator i;
+        for (i = windows.begin(); i != windows.end(); ++i)
+        {
+            freeWobblyInfo(i.value());
+        }
+    }
+}
+
+void WobblyWindowsEffect::reconfigure( ReconfigureFlags )
 {
     KConfigGroup conf = effects->effectConfig("Wobbly");
 
@@ -191,13 +195,20 @@ WobblyWindowsEffect::WobblyWindowsEffect()
     QString settingsMode = conf.readEntry("Settings", "Auto");
     if (settingsMode != "Custom")
     {
-        unsigned int wobblynessLevel = conf.readEntry("WobblynessLevel", 1);
+        unsigned int wobblynessLevel = conf.readEntry("WobblynessLevel", 0);
         if (wobblynessLevel > 4)
         {
-            kDebug() << "Wrong value for \"WobblynessLevel\" : " << wobblynessLevel;
+            kDebug(1212) << "Wrong value for \"WobblynessLevel\" : " << wobblynessLevel;
             wobblynessLevel = 4;
         }
         setParameterSet(pset[wobblynessLevel]);
+        
+        if (conf.readEntry("AdvancedMode", false))
+        {
+            m_stiffness = conf.readEntry("Stiffness", STIFFNESS * 100.0) / 100.0;
+            m_drag = conf.readEntry("Drag", DRAG * 100.0) / 100.0;
+            m_move_factor = conf.readEntry("MoveFactor", MOVEFACTOR * 100.0) / 100.0;
+        }
     }
     else // Custom method, read all values from config file.
     {
@@ -215,68 +226,17 @@ WobblyWindowsEffect::WobblyWindowsEffect()
         m_maxAcceleration = conf.readEntry("MaxAcceleration", MAXACCELERATION);
         m_stopAcceleration = conf.readEntry("StopAcceleration", STOPACCELERATION);
 
-        QString velFilter = conf.readEntry("VelocityFilter", VELOCITYFILTER);
-        if (velFilter == "NoFilter")
-        {
-            m_velocityFilter = NoFilter;
-        }
-        else if (velFilter == "FourRingLinearMean")
-        {
-            m_velocityFilter = FourRingLinearMean;
-        }
-        else if (velFilter == "HeightRingLinearMean")
-        {
-            m_velocityFilter = HeightRingLinearMean;
-        }
-        else if (velFilter == "MeanWithMean")
-        {
-            m_velocityFilter = MeanWithMean;
-        }
-        else if (velFilter == "MeanWithMedian")
-        {
-            m_velocityFilter = MeanWithMedian;
-        }
-        else
-        {
-            m_velocityFilter = FourRingLinearMean;
-            kDebug() << "Unknown config value for VelocityFilter : " << velFilter;
-        }
-
-        QString accFilter = conf.readEntry("AccelerationFilter", ACCELERATIONFILTER);
-        if (accFilter == "NoFilter")
-        {
-            m_accelerationFilter = NoFilter;
-        }
-        else if (accFilter == "FourRingLinearMean")
-        {
-            m_accelerationFilter = FourRingLinearMean;
-        }
-        else if (accFilter == "HeightRingLinearMean")
-        {
-            m_accelerationFilter = HeightRingLinearMean;
-        }
-        else if (accFilter == "MeanWithMean")
-        {
-            m_accelerationFilter = MeanWithMean;
-        }
-        else if (accFilter == "MeanWithMedian")
-        {
-            m_accelerationFilter = MeanWithMedian;
-        }
-        else
-        {
-            m_accelerationFilter = NoFilter;
-            kDebug() << "Unknown config value for accelerationFilter : " << accFilter;
-        }
-
         m_moveEffectEnabled = conf.readEntry("MoveEffect", true);
         m_openEffectEnabled = conf.readEntry("OpenEffect", false);
         // disable close effect by default for now as it doesn't do what I want.
         m_closeEffectEnabled = conf.readEntry("CloseEffect", false);
     }
 
+    m_moveWobble = conf.readEntry("MoveWobble", true);
+    m_resizeWobble = conf.readEntry("ResizeWobble", true);
+
 #if defined VERBOSE_MODE
-    kDebug() << "Parameters :\n" <<
+    kDebug(1212) << "Parameters :\n" <<
         "move : " << m_moveEffectEnabled << ", open : " << m_openEffectEnabled << ", close : " << m_closeEffectEnabled << "\n"
         "grid(" << m_stiffness << ", " << m_drag << ", " << m_move_factor << ")\n" <<
         "velocity(" << m_minVelocity << ", " << m_maxVelocity << ", " << m_stopVelocity << ")\n" <<
@@ -285,22 +245,8 @@ WobblyWindowsEffect::WobblyWindowsEffect()
 #endif
 }
 
-WobblyWindowsEffect::~WobblyWindowsEffect()
-{
-    if (!windows.empty())
-    {
-        // we should be empty at this point...
-        // emit a warning and clean the list.
-        kDebug() << "Windows list not empty. Left items : " << windows.count();
-        QHash< const EffectWindow*,  WindowWobblyInfos >::iterator i;
-        for (i = windows.begin(); i != windows.end(); ++i)
-        {
-            freeWobblyInfo(i.value());
-        }
-    }
-}
 
-void WobblyWindowsEffect::setParameterSet(ParameterSet& pset)
+void WobblyWindowsEffect::setParameterSet(const ParameterSet& pset)
 {
     m_stiffness = pset.stiffness;
     m_drag = pset.drag;
@@ -309,9 +255,6 @@ void WobblyWindowsEffect::setParameterSet(ParameterSet& pset)
     m_xTesselation = pset.xTesselation;
     m_yTesselation = pset.yTesselation;
 
-    m_velocityFilter = pset.velocityFilter ;
-    m_accelerationFilter =  pset.accelerationFilter;
-
     m_minVelocity =  pset.minVelocity;
     m_maxVelocity =  pset.maxVelocity;
     m_stopVelocity =  pset.stopVelocity;
@@ -339,26 +282,6 @@ void WobblyWindowsEffect::setStiffness(qreal stiffness)
     m_stiffness = stiffness;
 }
 
-void WobblyWindowsEffect::setVelocityFilter(GridFilter filter)
-{
-    m_velocityFilter = filter;
-}
-
-void WobblyWindowsEffect::setAccelerationFilter(GridFilter filter)
-{
-    m_accelerationFilter = filter;
-}
-
-WobblyWindowsEffect::GridFilter WobblyWindowsEffect::velocityFilter() const
-{
-    return m_velocityFilter;
-}
-
-WobblyWindowsEffect::GridFilter WobblyWindowsEffect::accelerationFilter() const
-{
-    return m_accelerationFilter;
-}
-
 void WobblyWindowsEffect::setDrag(qreal drag)
 {
     m_drag = drag;
@@ -392,7 +315,7 @@ void WobblyWindowsEffect::prePaintWindow(EffectWindow* w, WindowPrePaintData& da
         while (!stop && (updateTime > maxTime))
         {
 #if defined VERBOSE_MODE
-            kDebug() << "loop time " << updateTime << " / " << time;
+            kDebug(1212) << "loop time " << updateTime << " / " << time;
 #endif
             stop = !updateWindowWobblyDatas(w, maxTime);
             updateTime -= maxTime;
@@ -442,7 +365,8 @@ void WobblyWindowsEffect::postPaintScreen()
 
 void WobblyWindowsEffect::windowUserMovedResized(EffectWindow* w, bool first, bool last)
 {
-    if (m_moveEffectEnabled && first && !w->isSpecialWindow())
+    if (m_moveEffectEnabled && first && !w->isSpecialWindow() &&
+       ((w->isUserMove() && m_moveWobble) || (w->isUserResize() && m_resizeWobble)))
     {
         if (!windows.contains(w))
         {
@@ -464,18 +388,33 @@ void WobblyWindowsEffect::windowUserMovedResized(EffectWindow* w, bool first, bo
         int pickedPointIndex = indy*wwi.width + indx;
         if (pickedPointIndex < 0)
         {
-            kDebug() << "Picked index == " << pickedPointIndex << " with (" << cursorPos().x() << "," << cursorPos().y() << ")";
+            kDebug(1212) << "Picked index == " << pickedPointIndex << " with (" << cursorPos().x() << "," << cursorPos().y() << ")";
             pickedPointIndex = 0;
         }
         else if (static_cast<unsigned int>(pickedPointIndex) > wwi.count - 1)
         {
-            kDebug() << "Picked index == " << pickedPointIndex << " with (" << cursorPos().x() << "," << cursorPos().y() << ")";
+            kDebug(1212) << "Picked index == " << pickedPointIndex << " with (" << cursorPos().x() << "," << cursorPos().y() << ")";
             pickedPointIndex = wwi.count - 1;
         }
 #if defined VERBOSE_MODE
-        kDebug() << "Original Picked point -- x : " << picked.x << " - y : " << picked.y;
+        kDebug(1212) << "Original Picked point -- x : " << picked.x << " - y : " << picked.y;
 #endif
         wwi.constraint[pickedPointIndex] = true;
+
+        if (w->isUserResize())
+        {
+            if (picked.x > rect.center().x())
+                if (picked.y > rect.center().y())
+                    // picked somewhere in the bottom right, so constrain the top left corner too
+                    wwi.locked[0] = wwi.constraint[0] = true;
+                else
+                    wwi.locked[wwi.count-wwi.width] = wwi.constraint[wwi.count-wwi.width] = true;
+            else
+                if (picked.y > rect.center().y())
+                    wwi.locked[wwi.width-1] = wwi.constraint[wwi.width-1] = true;
+                else
+                    wwi.locked[wwi.count-1] = wwi.constraint[wwi.count-1] = true;
+        }
     }
     else if (m_moveEffectEnabled && last)
     {
@@ -543,6 +482,7 @@ void WobblyWindowsEffect::wobblyOpenInit(WindowWobblyInfos& wwi) const
         {
             unsigned int idx = j*4 + i;
             wwi.constraint[idx] = false;
+            wwi.locked[idx] = false;
             wwi.position[idx].x = (wwi.position[idx].x + 3*middle.x)/4;
             wwi.position[idx].y = (wwi.position[idx].y + 3*middle.y)/4;
         }
@@ -567,6 +507,7 @@ void WobblyWindowsEffect::wobblyCloseInit(WindowWobblyInfos& wwi, EffectWindow*
         {
             unsigned int idx = j*4 + i;
             wwi.constraint[idx] = false;
+            wwi.locked[idx] = false;
         }
     }
     wwi.status = Closing;
@@ -588,6 +529,7 @@ void WobblyWindowsEffect::initWobblyInfo(WindowWobblyInfos& wwi, QRect geometry)
     wwi.acceleration = new Pair[wwi.count];
     wwi.buffer = new Pair[wwi.count];
     wwi.constraint = new bool[wwi.count];
+    wwi.locked = new bool[wwi.count];
 
     wwi.bezierSurface = new Pair[wwi.bezierCount];
 
@@ -611,6 +553,7 @@ void WobblyWindowsEffect::initWobblyInfo(WindowWobblyInfos& wwi, QRect geometry)
             wwi.position[idx] = initValue;
             wwi.velocity[idx] = nullPair;
             wwi.constraint[idx] = false;
+            wwi.locked[idx] = false;
             if (i != 4-2) // x grid count - 2, i.e. not the last point
             {
                 initValue.x += x_increment;
@@ -643,6 +586,7 @@ void WobblyWindowsEffect::freeWobblyInfo(WindowWobblyInfos& wwi) const
     delete[] wwi.acceleration;
     delete[] wwi.buffer;
     delete[] wwi.constraint;
+    delete[] wwi.locked;
 
     delete[] wwi.bezierSurface;
 }
@@ -666,7 +610,7 @@ WobblyWindowsEffect::Pair WobblyWindowsEffect::computeBezierPoint(const WindowWo
 //    ASSERT1(ty >= 0);
 //    ASSERT1(ty <= 1);
 
-    // compute polinomial coeff
+    // compute polynomial coeff
 
     qreal px[4];
     px[0] = (1-tx)*(1-tx)*(1-tx);
@@ -694,6 +638,7 @@ WobblyWindowsEffect::Pair WobblyWindowsEffect::computeBezierPoint(const WindowWo
 
     return res;
 }
+
 namespace
 {
 
@@ -768,8 +713,8 @@ bool WobblyWindowsEffect::updateWindowWobblyDatas(EffectWindow* w, qreal time)
     qreal y_length = rect.height() / (wwi.height-1.0);
 
 #if defined VERBOSE_MODE
-    kDebug() << "time " << time;
-    kDebug() << "increment x " << x_length << " // y" <<  y_length;
+    kDebug(1212) << "time " << time;
+    kDebug(1212) << "increment x " << x_length << " // y" <<  y_length;
 #endif
 
     Pair origine = {rect.x(), rect.y()};
@@ -1064,30 +1009,7 @@ bool WobblyWindowsEffect::updateWindowWobblyDatas(EffectWindow* w, qreal time)
         }
     }
 
-    switch (m_accelerationFilter)
-    {
-    case NoFilter:
-        break;
-
-    case FourRingLinearMean:
-        fourRingLinearMean(&wwi.acceleration, wwi);
-        break;
-
-    case HeightRingLinearMean:
-        heightRingLinearMean(&wwi.acceleration, wwi);
-        break;
-
-    case MeanWithMean:
-        meanWithMean(&wwi.acceleration, wwi);
-        break;
-
-    case MeanWithMedian:
-        meanWithMedian(&wwi.acceleration, wwi);
-        break;
-
-    default:
-        ASSERT1(false);
-    }
+    heightRingLinearMean(&wwi.acceleration, wwi);
 
 #if defined COMPUTE_STATS
     Pair accBound = {m_maxAcceleration, m_minAcceleration};
@@ -1111,31 +1033,7 @@ bool WobblyWindowsEffect::updateWindowWobblyDatas(EffectWindow* w, qreal time)
         acc_sum += fabs(acc.x) + fabs(acc.y);
     }
 
-
-    switch (m_velocityFilter)
-    {
-    case NoFilter:
-        break;
-
-    case FourRingLinearMean:
-        fourRingLinearMean(&wwi.velocity, wwi);
-        break;
-
-    case HeightRingLinearMean:
-        heightRingLinearMean(&wwi.velocity, wwi);
-        break;
-
-    case MeanWithMean:
-        meanWithMean(&wwi.velocity, wwi);
-        break;
-
-    case MeanWithMedian:
-        meanWithMedian(&wwi.velocity, wwi);
-        break;
-
-    default:
-        ASSERT1(false);
-    }
+    heightRingLinearMean(&wwi.velocity, wwi);
 
     Pair topLeftCorner = {-10000.0, -10000.0};
     Pair bottomRightCorner = {10000.0, 10000.0};
@@ -1173,20 +1071,24 @@ bool WobblyWindowsEffect::updateWindowWobblyDatas(EffectWindow* w, qreal time)
 
         vel_sum += fabs(vel.x) + fabs(vel.y);
 
+        if (wwi.locked[i])
+        {
+            wwi.position[i] = wwi.origin[i];
+        }
 #if defined VERBOSE_MODE
         if (wwi.constraint[i])
         {
-            kDebug() << "Constraint point ** vel : " << vel.x << "," << vel.y << " ** move : " << vel.x*time << "," << vel.y*time;
+            kDebug(1212) << "Constraint point ** vel : " << vel.x << "," << vel.y << " ** move : " << vel.x*time << "," << vel.y*time;
         }
 #endif
     }
 
 #if defined VERBOSE_MODE
 #   if defined COMPUTE_STATS
-        kDebug() << "Acceleration bounds (" << accBound.x << ", " << accBound.y << ")";
-        kDebug() << "Velocity bounds (" << velBound.x << ", " << velBound.y << ")";
+        kDebug(1212) << "Acceleration bounds (" << accBound.x << ", " << accBound.y << ")";
+        kDebug(1212) << "Velocity bounds (" << velBound.x << ", " << velBound.y << ")";
 #   endif
-    kDebug() << "sum_acc : " << acc_sum << "  ***  sum_vel :" << vel_sum;
+    kDebug(1212) << "sum_acc : " << acc_sum << "  ***  sum_vel :" << vel_sum;
 #endif
 
     if (wwi.status != Moving && acc_sum < m_stopAcceleration && vel_sum < m_stopVelocity)
@@ -1214,204 +1116,6 @@ bool WobblyWindowsEffect::updateWindowWobblyDatas(EffectWindow* w, qreal time)
     return true;
 }
 
-void WobblyWindowsEffect::fourRingLinearMean(Pair** datas_pointer, WindowWobblyInfos& wwi)
-{
-    Pair* datas = *datas_pointer;
-    Pair neibourgs[4];
-
-    // for corners
-
-    // top-left
-    {
-        Pair& res = wwi.buffer[0];
-        Pair vit = datas[0];
-        neibourgs[0] = datas[1];
-        neibourgs[1] = datas[wwi.width];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + 2.0*vit.x) / 4.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + 2.0*vit.y) / 4.0;
-    }
-
-
-    // top-right
-    {
-        Pair& res = wwi.buffer[wwi.width-1];
-        Pair vit = datas[wwi.width-1];
-        neibourgs[0] = datas[wwi.width-2];
-        neibourgs[1] = datas[2*wwi.width-1];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + 2.0*vit.x) / 4.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + 2.0*vit.y) / 4.0;
-    }
-
-
-    // bottom-left
-    {
-        Pair& res = wwi.buffer[wwi.width*(wwi.height-1)];
-        Pair vit = datas[wwi.width*(wwi.height-1)];
-        neibourgs[0] = datas[wwi.width*(wwi.height-1)+1];
-        neibourgs[1] = datas[wwi.width*(wwi.height-2)];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + 2.0*vit.x) / 4.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + 2.0*vit.y) / 4.0;
-    }
-
-
-    // bottom-right
-    {
-        Pair& res = wwi.buffer[wwi.count-1];
-        Pair vit = datas[wwi.count-1];
-        neibourgs[0] = datas[wwi.count-2];
-        neibourgs[1] = datas[wwi.width*(wwi.height-1)-1];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + 2.0*vit.x) / 4.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + 2.0*vit.y) / 4.0;
-    }
-
-
-    // for borders
-
-    // top border
-    for (unsigned int i=1; i<wwi.width-1; ++i)
-    {
-        Pair& res = wwi.buffer[i];
-        Pair vit = datas[i];
-        neibourgs[0] = datas[i-1];
-        neibourgs[1] = datas[i+1];
-        neibourgs[2] = datas[i+wwi.width];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + neibourgs[2].x + 3.0*vit.x) / 6.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + neibourgs[2].y + 3.0*vit.y) / 6.0;
-    }
-
-    // bottom border
-    for (unsigned int i=wwi.width*(wwi.height-1)+1; i<wwi.count-1; ++i)
-    {
-        Pair& res = wwi.buffer[i];
-        Pair vit = datas[i];
-        neibourgs[0] = datas[i-1];
-        neibourgs[1] = datas[i+1];
-        neibourgs[2] = datas[i-wwi.width];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + neibourgs[2].x + 3.0*vit.x) / 6.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + neibourgs[2].y + 3.0*vit.y) / 6.0;
-    }
-
-    // left border
-    for (unsigned int i=wwi.width; i<wwi.width*(wwi.height-1); i+=wwi.width)
-    {
-        Pair& res = wwi.buffer[i];
-        Pair vit = datas[i];
-        neibourgs[0] = datas[i+1];
-        neibourgs[1] = datas[i-wwi.width];
-        neibourgs[2] = datas[i+wwi.width];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + neibourgs[2].x + 3.0*vit.x) / 6.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + neibourgs[2].y + 3.0*vit.y) / 6.0;
-    }
-
-    // right border
-    for (unsigned int i=2*wwi.width-1; i<wwi.count-1; i+=wwi.width)
-    {
-        Pair& res = wwi.buffer[i];
-        Pair vit = datas[i];
-        neibourgs[0] = datas[i-1];
-        neibourgs[1] = datas[i-wwi.width];
-        neibourgs[2] = datas[i+wwi.width];
-
-        res.x = (neibourgs[0].x + neibourgs[1].x + neibourgs[2].x + 3.0*vit.x) / 6.0;
-        res.y = (neibourgs[0].y + neibourgs[1].y + neibourgs[2].y + 3.0*vit.y) / 6.0;
-    }
-
-    // for the inner points
-    for (unsigned int j=1; j<wwi.height-1; ++j)
-    {
-        for (unsigned int i=1; i<wwi.width-1; ++i)
-        {
-            unsigned int index = i+j*wwi.width;
-
-            Pair& res = wwi.buffer[index];
-            Pair& vit = datas[index];
-            neibourgs[0] = datas[index-1];
-            neibourgs[1] = datas[index+1];
-            neibourgs[2] = datas[index-wwi.width];
-            neibourgs[3] = datas[index+wwi.width];
-
-            res.x = (neibourgs[0].x + neibourgs[1].x + neibourgs[2].x + neibourgs[3].x + 4.0*vit.x) / 8.0;
-            res.y = (neibourgs[0].y + neibourgs[1].y + neibourgs[2].y + neibourgs[3].y + 4.0*vit.y) / 8.0;
-        }
-    }
-
-    Pair* tmp = datas;
-    *datas_pointer = wwi.buffer;
-    wwi.buffer = tmp;
-}
-
-void WobblyWindowsEffect::meanWithMean(Pair** datas_pointer, WindowWobblyInfos& wwi)
-{
-    Pair* datas = *datas_pointer;
-
-    Pair mean = {0.0, 0.0};
-    for (unsigned int i = 0; i < wwi.count; ++i)
-    {
-        mean.x += datas[i].x;
-        mean.y += datas[i].y;
-    }
-
-    mean.x /= wwi.count;
-    mean.y /= wwi.count;
-
-    for (unsigned int i = 0; i < wwi.count; ++i)
-    {
-        wwi.buffer[i].x = (datas[i].x + mean.x) / 2.0;
-        wwi.buffer[i].y = (datas[i].y + mean.y) / 2.0;
-    }
-
-    Pair* tmp = datas;
-    *datas_pointer = wwi.buffer;
-    wwi.buffer = tmp;
-}
-
-void WobblyWindowsEffect::meanWithMedian(Pair** datas_pointer, WindowWobblyInfos& wwi)
-{
-    Pair* datas = *datas_pointer;
-
-    qreal xmin = datas[0].x, ymin = datas[0].y;
-    qreal xmax = datas[0].x, ymax = datas[0].y;
-    for (unsigned int i = 1; i < wwi.count; ++i)
-    {
-        if (datas[i].x < xmin)
-        {
-            xmin = datas[i].x;
-        }
-        if (datas[i].x > xmax)
-        {
-            xmax = datas[i].x;
-        }
-
-        if (datas[i].y < ymin)
-        {
-            ymin = datas[i].y;
-        }
-        if (datas[i].y > ymax)
-        {
-            ymax = datas[i].y;
-        }
-    }
-
-    Pair median = {(xmin + xmax)/2.0, (ymin + ymax)/2.0};
-
-    for (unsigned int i = 0; i < wwi.count; ++i)
-    {
-        wwi.buffer[i].x = (datas[i].x + median.x) / 2.0;
-        wwi.buffer[i].y = (datas[i].y + median.y) / 2.0;
-    }
-
-    Pair* tmp = datas;
-    *datas_pointer = wwi.buffer;
-    wwi.buffer = tmp;
-}
-
 void WobblyWindowsEffect::heightRingLinearMean(Pair** datas_pointer, WindowWobblyInfos& wwi)
 {
     Pair* datas = *datas_pointer;
