@@ -64,11 +64,14 @@ WindowPaintData::WindowPaintData( EffectWindow* w )
     , decoration_opacity( 1.0 )
     , xScale( 1 )
     , yScale( 1 )
+    , zScale( 1 )
     , xTranslate( 0 )
     , yTranslate( 0 )
+    , zTranslate( 0 )
     , saturation( 1 )
     , brightness( 1 )
     , shader( NULL )
+    , rotation( NULL )
     {
     quads = w->buildQuads();
     }
@@ -76,8 +79,20 @@ WindowPaintData::WindowPaintData( EffectWindow* w )
 ScreenPaintData::ScreenPaintData()
     : xScale( 1 )
     , yScale( 1 )
+    , zScale( 1 )
     , xTranslate( 0 )
     , yTranslate( 0 )
+    , zTranslate( 0 )
+    , rotation( NULL )
+    {
+    }
+
+RotationData::RotationData()
+    : axis( ZAxis )
+    , angle( 0.0 )
+    , xRotationPoint( 0.0 )
+    , yRotationPoint( 0.0 )
+    , zRotationPoint( 0.0 )
     {
     }
 
@@ -93,6 +108,10 @@ Effect::~Effect()
     {
     }
 
+void Effect::reconfigure( ReconfigureFlags )
+    {
+    }
+
 void Effect::windowUserMovedResized( EffectWindow* , bool, bool )
     {
     }
@@ -205,6 +224,11 @@ void Effect::drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintD
     effects->drawWindow( w, mask, region, data );
     }
 
+void Effect::buildQuads( EffectWindow* w, WindowQuadList& quadList )
+    {
+    effects->buildQuads( w, quadList );
+    }
+
 QRect Effect::transformWindowDamage( EffectWindow* w, const QRect& r )
     {
     return effects->transformWindowDamage( w, r );
@@ -241,6 +265,17 @@ QPoint Effect::cursorPos()
     return effects->cursorPos();
     }
 
+double Effect::animationTime( const KConfigGroup& cfg, const QString& key, int defaultTime )
+    {
+    int time = cfg.readEntry( key, 0 );
+    return time != 0 ? time : qMax( defaultTime * effects->animationTimeFactor(), 1. );
+    }
+
+double Effect::animationTime( int defaultTime )
+    { // at least 1ms, otherwise 0ms times can break some things
+    return qMax( defaultTime * effects->animationTimeFactor(), 1. );
+    }
+
 //****************************************
 // EffectsHandler
 //****************************************
@@ -249,6 +284,7 @@ EffectsHandler::EffectsHandler(CompositingType type)
     : current_paint_screen( 0 )
     , current_paint_window( 0 )
     , current_draw_window( 0 )
+    , current_build_quads( 0 )
     , current_transform( 0 )
     , compositing_type( type )
     {
@@ -307,7 +343,7 @@ bool EffectsHandler::saturationSupported() const
 
 void EffectsHandler::sendReloadMessage( const QString& effectname )
     {
-    QDBusMessage message = QDBusMessage::createMethodCall("org.kde.kwin", "/KWin", "org.kde.KWin", "reloadEffect");
+    QDBusMessage message = QDBusMessage::createMethodCall("org.kde.kwin", "/KWin", "org.kde.KWin", "reconfigureEffect");
     message << QString("kwin4_effect_" + effectname);
     QDBusConnection::sessionBus().send(message);
     }
@@ -318,21 +354,21 @@ KConfigGroup EffectsHandler::effectConfig( const QString& effectname )
     return kwinconfig->group( "Effect-" + effectname );
     }
 
-bool EffectsHandler::paintText( const QString& text, const QPoint& center, int maxwidth,
-        const QColor& color, const QFont& font )
+bool EffectsHandler::paintText( const QString& text, const QRect& rect, const QColor& color,
+        const QFont& font, const Qt::Alignment& alignment )
 {
     QPainter p;
     // Calculate size of the text
     QFontMetrics fm( font );
-    QString painttext = fm.elidedText( text, Qt::ElideRight, maxwidth );
+    QString painttext = fm.elidedText( text, Qt::ElideRight, rect.width() );
     QRect textrect = fm.boundingRect( painttext );
 
-    // Create temporary QPixmap where the text will be drawn onto
-    QPixmap textPixmap( textrect.width(), textrect.height());
-    textPixmap.fill( Qt::transparent );
+    // Create temporary QImage where the text will be drawn onto
+    QImage textImage( textrect.width(), textrect.height(), QImage::Format_ARGB32 );
+    textImage.fill( Qt::transparent );
 
     // Draw the text
-    p.begin( &textPixmap );
+    p.begin( &textImage );
     p.setFont( font );
     p.setRenderHint( QPainter::TextAntialiasing );
     p.setPen( color );
@@ -340,32 +376,30 @@ bool EffectsHandler::paintText( const QString& text, const QPoint& center, int m
     p.end();
 
     // Area covered by text
-    QRect area( center.x() - textrect.width() / 2, center.y() - textrect.height() / 2,
-                 textrect.width(), textrect.height() );
+    int rectX, rectY;
+    if( alignment & Qt::AlignLeft )
+        rectX = rect.x();
+    else if( alignment & Qt::AlignRight )
+        rectX = rect.right() - textrect.width();
+    else
+        rectX = rect.center().x() - textrect.width() / 2;
+    if( alignment & Qt::AlignTop )
+        rectY = rect.y();
+    else if( alignment & Qt::AlignBottom )
+        rectY = rect.bottom() - textrect.height();
+    else
+        rectY = rect.center().y() - textrect.height() / 2;
+    QRect area( rectX, rectY, textrect.width(), textrect.height() );
 
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( effects->compositingType() == OpenGLCompositing )
         {
-        GLTexture textTexture( textPixmap, GL_TEXTURE_RECTANGLE_ARB );
+        GLTexture textTexture( textImage, GL_TEXTURE_RECTANGLE_ARB );
         glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT );
         glEnable( GL_BLEND );
         glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
         textTexture.bind();
-        const float verts[ 4 * 2 ] =
-            {
-            area.x(), area.y(),
-            area.x(), area.y() + area.height(),
-            area.x() + area.width(), area.y() + area.height(),
-            area.x() + area.width(), area.y()
-            };
-        const float texcoords[ 4 * 2 ] =
-            {
-            0, textPixmap.height(),
-            0, 0,
-            textPixmap.width(), 0,
-            textPixmap.width(), textPixmap.height()
-            };
-        renderGLGeometry( 4, verts, texcoords );
+        textTexture.render( infiniteRegion(), area );
         textTexture.unbind();
         glPopAttrib();
         return true;
@@ -374,8 +408,8 @@ bool EffectsHandler::paintText( const QString& text, const QPoint& center, int m
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if( effects->compositingType() == XRenderCompositing )
         {
-        XRenderPicture textPicture( textPixmap );
-        XRenderComposite( display(), textPixmap.depth() == 32 ? PictOpOver : PictOpSrc,
+        XRenderPicture textPicture( QPixmap::fromImage( textImage ));
+        XRenderComposite( display(), textImage.depth() == 32 ? PictOpOver : PictOpSrc,
             textPicture, None, effects->xrenderBufferPicture(),
             0, 0, 0, 0, area.x(), area.y(), area.width(), area.height());
         return true;
@@ -384,17 +418,29 @@ bool EffectsHandler::paintText( const QString& text, const QPoint& center, int m
     return false;
 }
 
-bool EffectsHandler::paintTextWithBackground( const QString& text, const QPoint& center, int maxwidth,
-        const QColor& color, const QColor& bgcolor, const QFont& font )
+bool EffectsHandler::paintTextWithBackground( const QString& text, const QRect& rect, const QColor& color,
+        const QColor& bgcolor, const QFont& font, const Qt::Alignment& alignment )
 {
     // Calculate size of the text
     QFontMetrics fm( font );
-    QString painttext = fm.elidedText( text, Qt::ElideRight, maxwidth );
+    QString painttext = fm.elidedText( text, Qt::ElideRight, rect.width() );
     QRect textrect = fm.boundingRect( painttext );
 
     // Area covered by text
-    QRect area( center.x() - textrect.width() / 2, center.y() - textrect.height() / 2,
-                textrect.width(), textrect.height() );
+    int rectX, rectY;
+    if( alignment & Qt::AlignLeft )
+        rectX = rect.x();
+    else if( alignment & Qt::AlignRight )
+        rectX = rect.right() - textrect.width();
+    else
+        rectX = rect.center().x() - textrect.width() / 2;
+    if( alignment & Qt::AlignTop )
+        rectY = rect.y();
+    else if( alignment & Qt::AlignBottom )
+        rectY = rect.bottom() - textrect.height();
+    else
+        rectY = rect.center().y() - textrect.height() / 2;
+    QRect area( rectX, rectY, textrect.width(), textrect.height() );
 
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( effects->compositingType() == OpenGLCompositing )
@@ -402,14 +448,14 @@ bool EffectsHandler::paintTextWithBackground( const QString& text, const QPoint&
         glColor4f( bgcolor.redF(), bgcolor.greenF(), bgcolor.blueF(), bgcolor.alphaF() );
         renderRoundBox( area.adjusted( -8, -3, 8, 3 ), 5 );
 
-        return paintText( text, center, maxwidth, color, font );
+        return paintText( text, rect, color, font, alignment );
     }
 #endif
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if( effects->compositingType() == XRenderCompositing )
     {
         xRenderRoundBox( effects->xrenderBufferPicture(), area.adjusted( -8, -3, 8, 3 ), 5, bgcolor );
-        return paintText( text, center, maxwidth, color, font );
+        return paintText( text, rect, color, font, alignment );
     }
 #endif
     return false;
@@ -529,7 +575,7 @@ bool WindowQuad::smoothNeeded() const
 WindowQuadList WindowQuadList::splitAtX( double x ) const
     {
     WindowQuadList ret;
-    foreach( WindowQuad quad, *this )
+    foreach( const WindowQuad &quad, *this )
         {
 #ifndef NDEBUG
         if( quad.isTransformed())
@@ -551,6 +597,11 @@ WindowQuadList WindowQuadList::splitAtX( double x ) const
             ret.append( quad );
             continue;
             }
+        if( quad.left() == quad.right() ) // quad has no size
+            {
+            ret.append( quad );
+            continue;
+            }
         ret.append( quad.makeSubQuad( quad.left(), quad.top(), x, quad.bottom()));
         ret.append( quad.makeSubQuad( x, quad.top(), quad.right(), quad.bottom()));
         }
@@ -560,7 +611,7 @@ WindowQuadList WindowQuadList::splitAtX( double x ) const
 WindowQuadList WindowQuadList::splitAtY( double y ) const
     {
     WindowQuadList ret;
-    foreach( WindowQuad quad, *this )
+    foreach( const WindowQuad &quad, *this )
         {
 #ifndef NDEBUG
         if( quad.isTransformed())
@@ -582,6 +633,11 @@ WindowQuadList WindowQuadList::splitAtY( double y ) const
             ret.append( quad );
             continue;
             }
+        if( quad.top() == quad.bottom() ) // quad has no size
+            {
+            ret.append( quad );
+            continue;
+            }
         ret.append( quad.makeSubQuad( quad.left(), quad.top(), quad.right(), y ));
         ret.append( quad.makeSubQuad( quad.left(), y, quad.right(), quad.bottom()));
         }
@@ -597,7 +653,7 @@ WindowQuadList WindowQuadList::makeGrid( int maxquadsize ) const
     double right = first().right();
     double top = first().top();
     double bottom = first().bottom();
-    foreach( WindowQuad quad, *this )
+    foreach( const WindowQuad &quad, *this )
         {
 #ifndef NDEBUG
         if( quad.isTransformed())
@@ -617,7 +673,7 @@ WindowQuadList WindowQuadList::makeGrid( int maxquadsize ) const
              y < bottom;
              y += maxquadsize )
             {
-            foreach( WindowQuad quad, *this )
+            foreach( const WindowQuad &quad, *this )
                 {
                 if( QRectF( QPointF( quad.left(), quad.top()), QPointF( quad.right(), quad.bottom()))
                     .intersects( QRectF( x, y, maxquadsize, maxquadsize )))
@@ -640,7 +696,7 @@ WindowQuadList WindowQuadList::makeRegularGrid( int xSubdivisions, int ySubdivis
     double right = first().right();
     double top = first().top();
     double bottom = first().bottom();
-    foreach( WindowQuad quad, *this )
+    foreach( const WindowQuad &quad, *this )
         {
 #ifndef NDEBUG
         if( quad.isTransformed())
@@ -663,7 +719,7 @@ WindowQuadList WindowQuadList::makeRegularGrid( int xSubdivisions, int ySubdivis
              x < right;
              x +=  xincrement)
             {
-            foreach( WindowQuad quad, *this )
+            foreach( const WindowQuad &quad, *this )
                 {
                 if( QRectF( QPointF( quad.left(), quad.top()), QPointF( quad.right(), quad.bottom()))
                     .intersects( QRectF( x, y, xincrement, yincrement )))
@@ -699,14 +755,14 @@ void WindowQuadList::makeArrays( float** vertices, float** texcoords ) const
 
 WindowQuadList WindowQuadList::select( WindowQuadType type ) const
     {
-    foreach( WindowQuad q, *this )
+    foreach( const WindowQuad &q, *this )
         {
-        if( q.type != type ) // something else than ones to select, make a copy and filter
+        if( q.type() != type ) // something else than ones to select, make a copy and filter
             {
             WindowQuadList ret;
-            foreach( WindowQuad q, *this )
+            foreach( const WindowQuad &q, *this )
                 {
-                if( q.type == type )
+                if( q.type() == type )
                     ret.append( q );
                 }
             return ret;
@@ -717,14 +773,14 @@ WindowQuadList WindowQuadList::select( WindowQuadType type ) const
 
 WindowQuadList WindowQuadList::filterOut( WindowQuadType type ) const
     {
-    foreach( WindowQuad q, *this )
+    foreach( const WindowQuad &q, *this )
         {
-        if( q.type == type ) // something to filter out, make a copy and filter
+        if( q.type() == type ) // something to filter out, make a copy and filter
             {
             WindowQuadList ret;
-            foreach( WindowQuad q, *this )
+            foreach( const WindowQuad &q, *this )
                 {
-                if( q.type != type )
+                if( q.type() != type )
                     ret.append( q );
                 }
             return ret;
@@ -735,7 +791,7 @@ WindowQuadList WindowQuadList::filterOut( WindowQuadType type ) const
 
 bool WindowQuadList::smoothNeeded() const
     {
-    foreach( WindowQuad q, *this )
+    foreach( const WindowQuad &q, *this )
         if( q.smoothNeeded())
             return true;
     return false;
@@ -743,7 +799,7 @@ bool WindowQuadList::smoothNeeded() const
 
 bool WindowQuadList::isTransformed() const
     {
-    foreach( WindowQuad q, *this )
+    foreach( const WindowQuad &q, *this )
         if( q.isTransformed())
             return true;
     return false;
@@ -798,7 +854,7 @@ QRegion PaintClipper::paintArea()
     {
     assert( areas != NULL ); // can be called only with clip() == true
     QRegion ret = QRegion( 0, 0, displayWidth(), displayHeight());
-    foreach( QRegion r, *areas )
+    foreach( const QRegion &r, *areas )
         ret &= r;
     return ret;
     }
@@ -900,9 +956,8 @@ QRect PaintClipper::Iterator::boundingRect() const
 TimeLine::TimeLine(const int duration)
     {
     m_Time = 0;
-    m_CurveShape = TimeLine::EaseInCurve;
     m_Duration = duration;
-    m_TimeLine = new QTimeLine(m_Duration);
+    m_TimeLine = new QTimeLine(m_Duration ? m_Duration : 1); // (avoid QTimeLine warning)
     m_TimeLine->setFrameRange(0, m_Duration);
     m_TimeLine->setCurveShape(QTimeLine::EaseInCurve);
     }
@@ -910,12 +965,12 @@ TimeLine::TimeLine(const int duration)
 TimeLine::TimeLine(const TimeLine &other)
     {
     m_Time = other.m_Time;
-    m_CurveShape = other.m_CurveShape;
     m_Duration = other.m_Duration;
-    m_TimeLine = new QTimeLine(m_Duration);
+    m_TimeLine = new QTimeLine(m_Duration ? m_Duration : 1);
     m_TimeLine->setFrameRange(0, m_Duration);
-    setProgress(m_Progress);
     setCurveShape(m_CurveShape);
+    if( m_Duration != 0 )
+        setProgress(m_Progress);
     }
 
 TimeLine::~TimeLine()
@@ -931,16 +986,19 @@ int TimeLine::duration() const
 void TimeLine::setDuration(const int msec)
     {
     m_Duration = msec;
-    m_TimeLine->setDuration(msec);
+    m_TimeLine->setDuration(m_Duration);
+    m_TimeLine->setFrameRange(0, m_Duration);
     }
 
 double TimeLine::value() const
     {
+    Q_ASSERT( m_Duration != 0 );
     return valueForTime(m_Time);
     }
 
 double TimeLine::valueForTime(const int msec) const
     {
+    Q_ASSERT( m_Duration != 0 );
     // Catch non QTimeLine CurveShapes here, (but there are none right now)
 
 
@@ -950,34 +1008,40 @@ double TimeLine::valueForTime(const int msec) const
 
 void TimeLine::addTime(const int msec)
     {
+    Q_ASSERT( m_Duration != 0 );
     m_Time = qMin(m_Duration, m_Time + msec);
     m_Progress = (double)m_Time / m_Duration;
     }
 
 void TimeLine::removeTime(const int msec)
     {
+    Q_ASSERT( m_Duration != 0 );
     m_Time = qMax(0, m_Time - msec);
     m_Progress = (double)m_Time / m_Duration;
     }
 
 void TimeLine::setProgress(const double progress)
     {
+    Q_ASSERT( m_Duration != 0 );
     m_Progress = progress;
     m_Time = qRound(m_Duration * progress);
     }
 
 double TimeLine::progress() const
     {
+    Q_ASSERT( m_Duration != 0 );
     return m_Progress;
     }
 
 int TimeLine::time() const
     {
+    Q_ASSERT( m_Duration != 0 );
     return m_Time;
     }
 
 void TimeLine::addProgress(const double progress)
     {
+    Q_ASSERT( m_Duration != 0 );
     m_Progress += progress;
     m_Time = (int)(m_Duration * m_Progress);
     }
@@ -1005,4 +1069,201 @@ void TimeLine::setCurveShape(CurveShape curveShape)
         m_CurveShape = curveShape;
     }
 
+/***************************************************************
+ WindowMotionManager
+***************************************************************/
+
+WindowMotionManager::WindowMotionManager( bool useGlobalAnimationModifier )
+    :   m_useGlobalAnimationModifier( useGlobalAnimationModifier )
+    ,   m_movingWindows( 0 )
+    { // TODO: Allow developer to modify motion attributes
+    } // TODO: What happens when the window moves by an external force?
+
+WindowMotionManager::~WindowMotionManager()
+    {
+    }
+
+void WindowMotionManager::manage( EffectWindow *w )
+    {
+    if( m_managedWindows.contains( w ))
+        return;
+
+    double strength = 0.9;
+    double decay = 0.75;
+    if( m_useGlobalAnimationModifier )
+        {
+        if( effects->animationTimeFactor() ) // If == 0 we just skip the calculation
+            strength = 0.9 / effects->animationTimeFactor();
+        decay = effects->animationTimeFactor() / ( effects->animationTimeFactor() + 0.3333333 );
+        }
+
+    m_managedWindows[ w ] = WindowMotion();
+    m_managedWindows[ w ].translation.setStrengthDecay( strength, decay );
+    m_managedWindows[ w ].scale.setStrengthDecay( strength, decay / 2.0 );
+
+    m_managedWindows[ w ].translation.setValue( w->pos() );
+    m_managedWindows[ w ].scale.setValue( QPointF( 1.0, 1.0 ));
+    }
+
+void WindowMotionManager::unmanage( EffectWindow *w )
+    {
+    if( !m_managedWindows.contains( w ))
+        return;
+
+    QPointF diffT = m_managedWindows[ w ].translation.distance();
+    QPointF diffS = m_managedWindows[ w ].scale.distance();
+    if( diffT.x() != 0.0 || diffT.y() != 0.0 || diffS.x() != 0.0 || diffS.y() != 0.0 )
+        m_movingWindows--; // Window was moving
+
+    m_managedWindows.remove( w );
+    }
+
+void WindowMotionManager::unmanageAll()
+    {
+    m_managedWindows.clear();
+    m_movingWindows = 0;
+    }
+
+void WindowMotionManager::calculate( int time )
+    {
+    if( !effects->animationTimeFactor() )
+        { // Just skip it completely if the user wants no animation
+        m_movingWindows = 0;
+        QHash<EffectWindow*, WindowMotion>::iterator it = m_managedWindows.begin();
+        for(; it != m_managedWindows.end(); it++ )
+            {
+            WindowMotion *motion = &it.value();
+            motion->translation.finish();
+            motion->scale.finish();
+            }
+        }
+
+    QHash<EffectWindow*, WindowMotion>::iterator it = m_managedWindows.begin();
+    for(; it != m_managedWindows.end(); it++ )
+        {
+        WindowMotion *motion = &it.value();
+        bool stopped = false;
+
+        // TODO: What happens when distance() == 0 but we are still moving fast?
+        // TODO: Motion needs to be calculated from the window's center
+
+        QPointF diffT = motion->translation.distance();
+        if( diffT != QPoint( 0.0, 0.0 ))
+            { // Still moving
+            motion->translation.calculate( time );
+            diffT = motion->translation.distance();
+            if( qAbs( diffT.x() ) < 0.5 && qAbs( motion->translation.velocity().x() ) < 0.2 &&
+                qAbs( diffT.y() ) < 0.5 && qAbs( motion->translation.velocity().y() ) < 0.2 )
+                { // Hide tiny oscillations
+                motion->translation.finish();
+                diffT = QPoint( 0.0, 0.0 );
+                stopped = true;
+                }
+            }
+
+        QPointF diffS = motion->scale.distance();
+        if( diffS != QPoint( 0.0, 0.0 ))
+            { // Still scaling
+            motion->scale.calculate( time );
+            diffS = motion->scale.distance();
+            if( qAbs( diffS.x() ) < 0.002 && qAbs( motion->scale.velocity().x() ) < 0.05 &&
+                qAbs( diffS.y() ) < 0.002 && qAbs( motion->scale.velocity().y() ) < 0.05 )
+                { // Hide tiny oscillations
+                motion->scale.finish();
+                diffS = QPoint( 0.0, 0.0 );
+                stopped = true;
+                }
+            }
+
+        // We just finished this window's motion
+        if( stopped && diffT == QPoint( 0.0, 0.0 ) && diffS == QPoint( 0.0, 0.0 ))
+            m_movingWindows--;
+        }
+    }
+
+void WindowMotionManager::reset()
+    {
+    if( !m_managedWindows.count() )
+        return;
+
+    EffectWindowList windows = m_managedWindows.keys();
+
+    for( int i = 0; i < windows.size(); i++ )
+        {
+        EffectWindow *w = windows.at( i );
+        m_managedWindows[ w ].translation.setTarget( w->pos() );
+        m_managedWindows[ w ].translation.finish();
+        m_managedWindows[ w ].scale.setTarget( QPointF( 1.0, 1.0 ));
+        m_managedWindows[ w ].scale.finish();
+        }
+    }
+
+void WindowMotionManager::reset( EffectWindow *w )
+    {
+    if( !m_managedWindows.contains( w ))
+        return;
+
+    m_managedWindows[ w ].translation.setTarget( w->pos() );
+    m_managedWindows[ w ].translation.finish();
+    m_managedWindows[ w ].scale.setTarget( QPointF( 1.0, 1.0 ));
+    m_managedWindows[ w ].scale.finish();
+    }
+
+void WindowMotionManager::apply( EffectWindow *w, WindowPaintData &data )
+    {
+    if( !m_managedWindows.contains( w ))
+        return;
+
+    // TODO: Take into account existing scale so that we can work with multiple managers (E.g. Present windows + grid)
+    data.xTranslate += m_managedWindows[ w ].translation.value().x() - w->x();
+    data.yTranslate += m_managedWindows[ w ].translation.value().y() - w->y();
+    data.xScale *= m_managedWindows[ w ].scale.value().x();
+    data.yScale *= m_managedWindows[ w ].scale.value().y();
+    }
+
+void WindowMotionManager::moveWindow( EffectWindow *w, QPoint target, double scale, double yScale )
+    {
+    if( !m_managedWindows.contains( w ))
+        abort(); // Notify the effect author that they did something wrong
+
+    if( yScale == 0.0 )
+        yScale = scale;
+    QPointF scalePoint( scale, yScale );
+
+    if( m_managedWindows[ w ].translation.value() == target &&
+        m_managedWindows[ w ].scale.value() == scalePoint )
+        return; // Window already at that position
+
+    m_managedWindows[ w ].translation.setTarget( target );
+    m_managedWindows[ w ].scale.setTarget( scalePoint );
+
+    if( m_managedWindows[ w ].translation.velocity() == QPointF( 0.0, 0.0 ) &&
+        m_managedWindows[ w ].scale.velocity() == QPointF( 0.0, 0.0 ))
+        m_movingWindows++;
+    }
+
+QRectF WindowMotionManager::transformedGeometry( EffectWindow *w ) const
+    {
+    QRectF geometry( w->geometry() );
+
+    // TODO: Take into account existing scale so that we can work with multiple managers (E.g. Present windows + grid)
+    geometry.moveTo( m_managedWindows[ w ].translation.value() );
+    geometry.setWidth( geometry.width() * m_managedWindows[ w ].scale.value().x() );
+    geometry.setHeight( geometry.height() * m_managedWindows[ w ].scale.value().y() );
+
+    return geometry;
+    }
+
+EffectWindow* WindowMotionManager::windowAtPoint( QPoint point, bool useStackingOrder ) const
+    {
+    // TODO: Stacking order uses EffectsHandler::stackingOrder() then filters by m_managedWindows
+    EffectWindowList windows = m_managedWindows.keys();
+
+    for( int i = 0; i < windows.size(); i++ )
+        if( transformedGeometry( windows.at( i )).contains( point ))
+            return windows.at( i );
+
+    return NULL;
+    }
+
 } // namespace
