@@ -227,7 +227,7 @@ void Workspace::clientPopupAboutToShow()
         }
 
     mResizeOpAction->setEnabled( active_popup_client->isResizable() );
-    mMoveOpAction->setEnabled( active_popup_client->isMovable() );
+    mMoveOpAction->setEnabled( active_popup_client->isMovableAcrossScreens() );
     mMaximizeOpAction->setEnabled( active_popup_client->isMaximizable() );
     mMaximizeOpAction->setChecked( active_popup_client->maximizeMode() == Client::MaximizeFull );
     mShadeOpAction->setEnabled( active_popup_client->isShadeable() );
@@ -416,17 +416,27 @@ void Workspace::clientShortcutUpdated( Client* c )
         {
         if( action == NULL ) // new shortcut
             {
-            action = client_keys->addAction( key );
-            connect( action, SIGNAL(triggered(bool)), c, SLOT(shortcutActivated()));
+            action = client_keys->addAction(QString( key ));
+            action->setText( i18n("Activate Window (%1)", c->caption()) );
+            connect( action, SIGNAL(triggered(bool)), c, SLOT(shortcutActivated()) );
             }
+
+        KAction *kaction = qobject_cast<KAction*>( action );
         // no autoloading, since it's configured explicitly here and is not meant to be reused
         // (the key is the window id anyway, which is kind of random)
-        qobject_cast< KAction* >( action )->setGlobalShortcut(
-            c->shortcut(), KAction::ActiveShortcut, KAction::NoAutoloading );
-        action->setEnabled( true );
+        kaction->setGlobalShortcut(
+                c->shortcut(), KAction::ActiveShortcut, KAction::NoAutoloading );
+        kaction->setEnabled( true );
         }
     else
+        {
+        KAction *kaction = qobject_cast<KAction*>( action );
+        if( kaction )
+            {
+            kaction->forgetGlobalShortcut();
+            }
         delete action;
+        }
     }
 
 void Workspace::clientPopupActivated( QAction *action )
@@ -564,15 +574,15 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             break;
         case Options::MouseShade :
             toggleShade();
-            cancelShadeHover();
+            cancelShadeHoverTimer();
             break;
         case Options::MouseSetShade:
             setShade( ShadeNormal );
-            cancelShadeHover();
+            cancelShadeHoverTimer();
             break;
         case Options::MouseUnsetShade:
             setShade( ShadeNone );
-            cancelShadeHover();
+            cancelShadeHoverTimer();
             break;
         case Options::MouseOperationsMenu:
             if ( isActive() && options->clickRaise )
@@ -612,13 +622,13 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             workspace()->raiseClient( this );
             workspace()->requestFocus( this );
             workspace()->setActiveScreenMouse( globalPos );
-            if( options->moveMode == Options::Transparent && isMovable())
+            if( options->moveMode == Options::Transparent && isMovableAcrossScreens())
                 move_faked_activity = workspace()->fakeRequestedActivity( this );
         // fallthrough
         case Options::MouseMove:
         case Options::MouseUnrestrictedMove:
             {
-            if (!isMovable())
+            if (!isMovableAcrossScreens())
                 break;
             if( moveResizeMode )
                 finishMoveResize( false );
@@ -694,10 +704,12 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             workspace()->windowToNextDesktop( this );
             break;
         case Options::MouseOpacityMore:
-            setOpacity( qMin( opacity() + 0.1, 1.0 ));
+            if( !isDesktop() ) // No point in changing the opacity of the desktop
+                setOpacity( qMin( opacity() + 0.1, 1.0 ));
             break;
         case Options::MouseOpacityLess:
-            setOpacity( qMax( opacity() - 0.1, 0.0 ));
+            if( !isDesktop() ) // No point in changing the opacity of the desktop
+                setOpacity( qMax( opacity() - 0.1, 0.0 ));
             break;
         case Options::MouseNothing:
             replay = true;
@@ -730,10 +742,10 @@ void Workspace::unloadEffect( const QString& name )
         static_cast<EffectsHandlerImpl*>(effects)->unloadEffect( name );
     }
 
-void Workspace::reloadEffect( const QString& name )
+void Workspace::reconfigureEffect( const QString& name )
     {
     if( effects )
-        static_cast<EffectsHandlerImpl*>(effects)->reloadEffect( name );
+        static_cast<EffectsHandlerImpl*>(effects)->reconfigureEffect( name );
     }
 
 QStringList Workspace::loadedEffects() const
@@ -1217,8 +1229,8 @@ void Client::setShortcut( const QString& _cut )
         }
     QList< KShortcut > keys;
     QStringList groups = cut.split( ' ');
-    for( QStringList::ConstIterator it = groups.begin();
-         it != groups.end();
+    for( QStringList::ConstIterator it = groups.constBegin();
+         it != groups.constEnd();
          ++it )
         {
         QRegExp reg( "(.*\\+)\\((.*)\\)" );
@@ -1236,15 +1248,15 @@ void Client::setShortcut( const QString& _cut )
                 }
             }
         }
-    for( QList< KShortcut >::ConstIterator it = keys.begin();
-         it != keys.end();
+    for( QList< KShortcut >::ConstIterator it = keys.constBegin();
+         it != keys.constEnd();
          ++it )
         {
         if( _shortcut == *it ) // current one is in the list
             return;
         }
-    for( QList< KShortcut >::ConstIterator it = keys.begin();
-         it != keys.end();
+    for( QList< KShortcut >::ConstIterator it = keys.constBegin();
+         it != keys.constEnd();
          ++it )
         {
         if( workspace()->shortcutAvailable( *it, this ))
@@ -1280,8 +1292,8 @@ void Client::delayedSetShortcut()
 bool Workspace::shortcutAvailable( const KShortcut& cut, Client* ignore ) const
     {
     // TODO check global shortcuts etc.
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it )
         {
         if( (*it) != ignore && (*it)->shortcut() == cut )
