@@ -73,7 +73,7 @@ class ProgressBarItemDelegate : public QStyledItemDelegate
 {
   public:
 	ProgressBarItemDelegate(QObject *parent) : QStyledItemDelegate(parent), startProgressColor(0x00, 0x71, 0xBC, 100), endProgressColor(0x83, 0xDD, 0xF5, 100), totalMemory(-1), numCpuCores(-1) {}
-  
+
   	virtual void paint(QPainter *painter, const QStyleOptionViewItem &opt, const QModelIndex &index) const
 	{
 		QStyleOptionViewItemV4 option = opt;
@@ -109,19 +109,19 @@ class ProgressBarItemDelegate : public QStyledItemDelegate
 					percentage = 0;
 			}
 		} else
-			percentage = 0;
+			percentage = -1;
 
-		if (percentage > 0)
+		if (percentage >= 0)
 			drawPercentageDisplay(painter,option,index.data(Qt::DisplayRole).toString());
-		else
+		else 
 			QStyledItemDelegate::paint(painter, option, index);
 	}
 
   private:
-	void drawPercentageDisplay(QPainter *painter, const QStyleOptionViewItem& option, const QString& text) const 
+	void drawPercentageDisplay(QPainter *painter, const QStyleOptionViewItem& option, const QString& text) const
 	{
 		QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem,&option,painter);
-		
+
 		const QRect rect = option.rect;
 		if(percentage * rect.width() > 100 ) { //make sure the line will have a width of more than 1 pixel
 			QPen old = painter->pen();
@@ -132,9 +132,9 @@ class ProgressBarItemDelegate : public QStyledItemDelegate
 			painter->fillRect( rect.x(), rect.y(), rect.width() * percentage /100 , rect.height(), QBrush(linearGrad));
 			painter->setPen( old );
 		}
-		painter->drawText(rect,option.displayAlignment,text);
+		painter->drawText(rect,option.displayAlignment,text + ' ');
 	}
-	
+
 	mutable int percentage;
 	QColor startProgressColor;
 	QColor endProgressColor;
@@ -153,7 +153,7 @@ struct KSysGuardProcessListPrivate {
 		selectTracer = new QAction(i18n("Jump to Process Debugging This One"), q);
 		window = new QAction(i18n("Show Application Window"), q);
 #ifdef WITH_MONITOR_PROCESS_IO
-		monitorio = new QAction(i18n("Monitor input and output"), q);
+		monitorio = new QAction(i18n("Monitor Input && Output"), q);
 #else
 		monitorio = 0;
 #endif
@@ -246,7 +246,7 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingTty);
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingCommand);
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingPid);
-	d->mFilterModel.setFilterKeyColumn(0);
+	d->mFilterModel.setFilterKeyColumn(-1);
 
 	d->mUi->treeView->header()->resizeSection(ProcessModel::HeadingCPUUsage, d->mUi->treeView->header()->sectionSizeHint(ProcessModel::HeadingCPUUsage));
 	d->mUi->treeView->header()->resizeSection(ProcessModel::HeadingMemory, d->mUi->treeView->header()->sectionSizeHint(ProcessModel::HeadingMemory));
@@ -263,6 +263,9 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 	//so don't let the user change it
 	d->mUi->treeView->header()->setResizeMode(0, QHeaderView::ResizeToContents);
 	d->mUi->treeView->header()->setStretchLastSection(true);
+	d->mUi->treeView->setDragEnabled(true);
+	d->mUi->treeView->setDragDropMode(QAbstractItemView::DragOnly);
+
 
 	//Sort by username by default
 	d->mUi->treeView->sortByColumn(ProcessModel::HeadingUser, Qt::AscendingOrder);
@@ -299,7 +302,6 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 //	mTimer->start(4000);
 // QT BUG?  We have to disable the sorting for now because there seems to be a bug in Qt introduced in Qt 4.4beta which makes the view scroll back to the top
         d->mModel.update(d->mUpdateIntervalMSecs);
-
 }
 
 KSysGuardProcessList::~KSysGuardProcessList()
@@ -711,13 +713,14 @@ bool KSysGuardProcessList::reniceProcesses(const QList<long long> &pids, int nic
 	if(!d->mModel.isLocalhost()) return false; //We can't use kdesu to renice non-localhost processes
 
 	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
 	arguments << "--" << "renice" << QString::number(niceValue);
 
         for (int i = 0; i < unreniced_pids.size(); ++i) {
 		arguments << QString::number(unreniced_pids.at(i));
 	}
 
-	QString su = KStandardDirs::findExe("kdesu");
+	QString su = KStandardDirs::findExe("kdesu");  //kdesu is a libexec program, so it will not be in the path.  findExe will find it correctly anyway
 	if(su.isEmpty()) return false;  //Cannot find kdesu
 
 	QProcess *reniceProcess = new QProcess(NULL);
@@ -865,6 +868,7 @@ bool KSysGuardProcessList::changeIoScheduler(const QList< long long> &pids, KSys
 	//We must use kdesu to kill the process
 
 	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
 	if(unchanged_pids.size() == 1) {
 		arguments << "--" << "ionice" << "-p" << QString::number(unchanged_pids.at(0)) << "-c";
 		switch(newIoSched) {
@@ -927,9 +931,47 @@ bool KSysGuardProcessList::changeCpuScheduler(const QList< long long> &pids, KSy
 		}
 	}
 	if(unchanged_pids.isEmpty()) return true;
-	if(!d->mModel.isLocalhost()) return false; //We can't use kdesu to kill non-localhost processes
+	if(!d->mModel.isLocalhost()) {
+		KMessageBox::sorry(this, i18n("No."));
+		return false; //We can't use kdesu to kill non-localhost processes
+	}
 
-	return false;  //TODO Must implement some form of kdesu or something
+	QString su = KStandardDirs::findExe("kdesu");
+	if(su.isEmpty()) {
+		KMessageBox::sorry(this, i18n("Could not find kdesu executable"));
+		return false;  //Cannot find kdesu
+	}
+	QString setscheduler = KStandardDirs::findExe("setscheduler");
+	if(setscheduler.isEmpty()) {
+		KMessageBox::sorry(this, i18n("Could not find setscheduler executable.  This should have been installed alongside system monitor."));
+		return false;
+	}
+
+	//We must use kdesu to kill the process
+
+	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
+	if(unchanged_pids.size() == 1) {
+		arguments << "--" << setscheduler << QString::number(unchanged_pids.at(0)) << QString::number((int)newCpuSched) << QString::number(newCpuSchedPriority);
+	} else {
+		//Cope with multiple pids by doing a for loop
+		arguments << "--" << "sh" << "-c";
+		QString sh("for f in ");
+
+	        for (int i = 0; i < unchanged_pids.size(); ++i) {
+			sh += QString::number(unchanged_pids.at(i)) + " ";
+		}
+		sh += "; do " + setscheduler + "\"$f\" " + newCpuSched + " " + newCpuSchedPriority;
+		sh += "; done";
+
+		arguments << sh;
+	}
+
+	QProcess *process = new QProcess(NULL);
+	connect(process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(ioniceFailed()));
+	connect(process, SIGNAL(finished( int, QProcess::ExitStatus) ), this, SLOT(updateList()));
+	process->start(su, arguments);
+	return true;  //assume it ran successfully :(  We cannot seem to actually check if it did.  There must be a better solution
 }
 
 bool KSysGuardProcessList::killProcesses(const QList< long long> &pids, int sig)
@@ -947,8 +989,10 @@ bool KSysGuardProcessList::killProcesses(const QList< long long> &pids, int sig)
 	QString su = KStandardDirs::findExe("kdesu");
 	if(su.isEmpty()) return false;  //Cannot find kdesu
 
+	kDebug() << "running " << su;
 	//We must use kdesu to kill the process
 	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
 	arguments << "--" << "kill";
 	if(sig != SIGTERM) {
 		arguments << ('-' + QString::number(sig));
