@@ -27,7 +27,14 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 // Qt
 #include <QMimeData>
+#include <QTime>
 #include <QTimer>
+#include <QApplication>
+#include <QDesktopWidget>
+
+#ifdef Q_WS_X11
+#include <QX11Info>
+#endif
 
 // KDE
 #include <KDebug>
@@ -35,11 +42,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <KLocale>
 
 #include "taskmanager.h"
-#include "taskmanager_p.h"
 
 namespace TaskManager
 {
 
+static const unsigned long windowInfoFlags = NET::WMState | NET::XAWMState | NET::WMDesktop |
+                                             NET::WMVisibleName | NET::WMGeometry |
+                                             NET::WMWindowType | NET::WM2AllowedActions;
+static const unsigned long windowInfoFlags2 = NET::WM2AllowedActions;
+
 class Task::Private
 {
 public:
@@ -47,17 +58,13 @@ public:
      : active(false),
        win(w),
        frameId(w),
-       info(KWindowSystem::windowInfo(w,
-            NET::WMState | NET::XAWMState | NET::WMDesktop |
-            NET::WMVisibleName | NET::WMGeometry | NET::WMWindowType,
-            NET::WM2AllowedActions)),
+       info(KWindowSystem::windowInfo(w, windowInfoFlags, windowInfoFlags2)),
        lastWidth(0),
        lastHeight(0),
        lastResize(false),
        lastIcon(),
-       thumbSize(0.2),
-       thumb(),
-       grab()
+       cachedChanges(0),
+       cachedChangesTimerId(0)
     {
     }
 
@@ -73,27 +80,26 @@ public:
     int lastHeight;
     bool lastResize;
     QPixmap lastIcon;
+    QIcon icon;
 
-    double thumbSize;
-    QPixmap thumb;
-    QPixmap grab;
     QRect iconGeometry;
 
-    Pixmap windowPixmap;
+    QTime lastUpdate;
+    unsigned int cachedChanges;
+    int cachedChangesTimerId;
 };
 
 Task::Task(WId w, QObject *parent, const char *name)
   : QObject(parent),
     d(new Private(w))
 {
-    setObjectName( name );
+    setObjectName(name);
 
     // try to load icon via net_wm
     d->pixmap = KWindowSystem::icon(d->win, 16, 16, true);
 
     // try to guess the icon from the classhint
-    if (d->pixmap.isNull())
-    {
+    if (d->pixmap.isNull()) {
         KIconLoader::global()->loadIcon(className().toLower(),
                                                     KIconLoader::Small,
                                                     KIconLoader::Small,
@@ -102,71 +108,26 @@ Task::Task(WId w, QObject *parent, const char *name)
     }
 
     // load the icon for X applications
-    if (d->pixmap.isNull())
-    {
+    if (d->pixmap.isNull()) {
         d->pixmap = SmallIcon("xorg");
     }
-
-#ifdef THUMBNAILING_POSSIBLE
-    d->windowPixmap = 0;
-    findWindowFrameId();
-
-    if (KWindowSystem::compositingActive())
-    {
-        updateWindowPixmap();
-    }
-#endif // THUMBNAILING_POSSIBLE
 }
 
 Task::~Task()
 {
-#ifdef THUMBNAILING_POSSIBLE
-    //be sure we have something to delete
-    if (d->windowPixmap && QX11Info::display())
-    {
-        XFreePixmap(QX11Info::display(), d->windowPixmap);
-    }
-#endif // THUMBNAILING_POSSIBLE
     delete d;
 }
 
-// Task::findWindowFrameId()
-// Code was copied from Kompose.
-// Copyright (C) 2004 Hans Oischinger
-// Permission granted on 2005-04-27.
-void Task::findWindowFrameId()
+void Task::timerEvent(QTimerEvent *)
 {
-#ifdef THUMBNAILING_POSSIBLE
-    Window targetWin, parent, root;
-    Window *children;
-    uint nchildren;
-
-    targetWin = d->win;
-    for (;;)
-    {
-        if (!XQueryTree(QX11Info::display(), targetWin, &root,
-                        &parent, &children, &nchildren))
-        {
-            break;
-        }
-
-        if (children)
-        {
-            XFree(children); // it's a list, that's deallocated!
-        }
-
-        if (!parent || parent == root)
-        {
-            break;
-        }
-        else
-        {
-            targetWin = parent;
-        }
+    if (d->cachedChanges) {
+        d->lastUpdate = QTime();
+        refresh(d->cachedChanges);
+        d->cachedChanges = 0;
     }
 
-    d->frameId = targetWin;
-#endif // THUMBNAILING_POSSIBLE
+    killTimer(d->cachedChangesTimerId);
+    d->cachedChangesTimerId = 0;
 }
 
 void Task::refreshIcon()
@@ -175,109 +136,134 @@ void Task::refreshIcon()
     d->pixmap = KWindowSystem::icon(d->win, 16, 16, true);
 
     // try to guess the icon from the classhint
-    if(d->pixmap.isNull())
-    {
+    if (d->pixmap.isNull()) {
         KIconLoader::global()->loadIcon(className().toLower(),
                                                     KIconLoader::Small,
                                                     KIconLoader::Small,
                                                     KIconLoader::DefaultState,
                                                     QStringList(), 0, true);
-    }
 
-    // load the icon for X applications
-    if (d->pixmap.isNull())
-    {
-        d->pixmap = SmallIcon("xorg");
+        // load the icon for X applications
+        if (d->pixmap.isNull()) {
+            d->pixmap = SmallIcon("xorg");
+        }
     }
 
     d->lastIcon = QPixmap();
-    emit iconChanged();
+    d->icon = QIcon();
+    emit changed(IconChanged);
 }
 
-void Task::refresh(unsigned int dirty)
+::TaskManager::TaskChanges Task::refresh(unsigned int dirty)
 {
-    QString name = visibleName();
-    d->info = KWindowSystem::windowInfo(d->win,
-        NET::WMState | NET::XAWMState | NET::WMDesktop | NET::WMVisibleName | NET::WMGeometry | NET::WMWindowType,
-        NET::WM2AllowedActions);
+    if (!d->lastUpdate.isNull() && d->lastUpdate.elapsed() < 200) {
+        d->cachedChanges |= dirty;
 
-    if (dirty != NET::WMName || name != visibleName())
-    {
-        emit changed();
+        if (!d->cachedChangesTimerId) {
+            d->cachedChangesTimerId = startTimer(200 - d->lastUpdate.elapsed());
+        }
+
+        d->lastUpdate.restart();
+        return TaskUnchanged;
     }
+
+    d->lastUpdate.restart();
+    KWindowInfo info = KWindowSystem::windowInfo(d->win, windowInfoFlags, windowInfoFlags2);
+    TaskChanges changes = TaskUnchanged;
+
+    if (d->info.visibleName() != info.visibleName() ||
+        d->info.visibleNameWithState() != info.visibleNameWithState() ||
+        d->info.name() != info.name()) {
+        changes |= NameChanged;
+    }
+
+    d->info = info;
+
+    if (dirty & NET::WMState || dirty & NET::XAWMState) {
+        changes |= StateChanged;
+    }
+
+    if (dirty & NET::WMDesktop) {
+        changes |= DesktopChanged;
+    }
+
+    if (dirty & NET::WMGeometry) {
+        changes |= GeometryChanged;
+    }
+
+    if (dirty & NET::WMWindowType) {
+        changes |= WindowTypeChanged;
+    }
+
+    if (dirty & NET::WM2AllowedActions) {
+        changes |= ActionsChanged;
+    }
+
+    if (dirty & NET::WMIcon) {
+        refreshIcon();
+    }
+
+    if (changes != TaskUnchanged) {
+        emit changed(changes);
+    }
+
+    return changes;
 }
 
 void Task::setActive(bool a)
 {
     d->active = a;
-    emit changed();
-    if ( a )
+    emit changed(StateChanged);
+    if (a) {
       emit activated();
-    else
+    } else {
       emit deactivated();
-}
-
-double Task::thumbnailSize() const { return d->thumbSize; }
-
-
-void Task::setThumbnailSize( double size )
-{
-    d->thumbSize = size;
-}
-
-bool Task::hasThumbnail() const
-{
-    return !d->thumb.isNull();
-}
-
-QPixmap Task::thumbnail() const
-{
-    return d->thumb;
+    }
 }
 
 bool Task::isMaximized() const
 {
-    return d->info.valid() && (d->info.state() & NET::Max);
+    return d->info.valid(true) && (d->info.state() & NET::Max);
 }
 
 bool Task::isMinimized() const
 {
-    return d->info.valid() && d->info.isMinimized();
+    return d->info.valid(true) && d->info.isMinimized();
 }
 
 bool Task::isIconified() const
 {
-    return d->info.valid() && d->info.isMinimized();
+    return d->info.valid(true) && d->info.isMinimized();
 }
 
 bool Task::isAlwaysOnTop() const
 {
-    return d->info.valid() && (d->info.state() & NET::StaysOnTop);
+    return d->info.valid(true) && (d->info.state() & NET::StaysOnTop);
 }
 
 bool Task::isKeptBelowOthers() const
 {
-    return d->info.valid() && (d->info.state() & NET::KeepBelow);
+    return d->info.valid(true) && (d->info.state() & NET::KeepBelow);
 }
 
 bool Task::isFullScreen() const
 {
-    return d->info.valid() && (d->info.state() & NET::FullScreen);
+    return d->info.valid(true) && (d->info.state() & NET::FullScreen);
 }
 
 bool Task::isShaded() const
 {
-    return d->info.valid() && (d->info.state() & NET::Shaded);
+    return d->info.valid(true) && (d->info.state() & NET::Shaded);
 }
 
 bool Task::isOnCurrentDesktop() const
 {
-    return d->info.valid() && d->info.isOnCurrentDesktop();
+    return d->info.valid(true) && d->info.isOnCurrentDesktop();
 }
 
 bool Task::isOnAllDesktops() const
 {
-    return d->info.valid() && d->info.onAllDesktops();
+    return d->info.valid(true) && d->info.onAllDesktops();
 }
 
 bool Task::isActive() const
@@ -293,7 +279,7 @@ bool Task::isOnTop() const
 bool Task::isModified() const
 {
   static QString modStr = QString::fromUtf8("[") +
-                          i18n("modified") +
+                          i18nc("marks that a task has been modified", "modified") +
                           QString::fromUtf8("]");
   int modStrPos = d->info.visibleName().indexOf(modStr);
 
@@ -308,7 +294,7 @@ int Task::desktop() const
 bool Task::demandsAttention() const
 {
     return (d->info.valid() && (d->info.state() & NET::DemandsAttention)) ||
-            d->transientsDemandingAttention.count() > 0;
+            !d->transientsDemandingAttention.isEmpty();
 }
 
 bool Task::isOnScreen( int screen ) const
@@ -333,43 +319,37 @@ QRect Task::geometry() const
 
 void Task::updateDemandsAttentionState( WId w )
 {
-    if (window() != w)
-    {
+    if (window() != w) {
         // 'w' is a transient for this task
         NETWinInfo i( QX11Info::display(), w, QX11Info::appRootWindow(), NET::WMState );
-        if(i.state() & NET::DemandsAttention)
-        {
-            if (!d->transientsDemandingAttention.contains(w))
-            {
-                d->transientsDemandingAttention.append(w);
+        if (i.state() & NET::DemandsAttention) {
+            if (!d->transientsDemandingAttention.contains(w)) {
+                d->transientsDemandingAttention.insert(w);
             }
-        }
-        else
-        {
-            d->transientsDemandingAttention.removeAll( w );
+        } else {
+            d->transientsDemandingAttention.remove(w);
         }
     }
 }
 
 void Task::addTransient( WId w, const NETWinInfo& info )
 {
-    d->transients.append(w);
-    if (info.state() & NET::DemandsAttention)
-    {
-        d->transientsDemandingAttention.append(w);
-        emit changed();
+    d->transients.insert(w);
+    if (info.state() & NET::DemandsAttention) {
+        d->transientsDemandingAttention.insert(w);
+        emit changed(TransientsChanged);
     }
 }
 
 void Task::removeTransient(WId w)
 {
-    d->transients.removeAll(w);
-    d->transientsDemandingAttention.removeAll(w);
+    d->transients.remove(w);
+    d->transientsDemandingAttention.remove(w);
 }
 
 bool Task::hasTransient(WId w) const
 {
-    return d->transients.indexOf(w) != -1;
+    return d->transients.contains(w);
 }
 
 WId Task::window() const
@@ -397,7 +377,7 @@ QString Task::name() const
     return d->info.name();
 }
 
-QString Task::className()
+QString Task::className() const
 {
     XClassHint hint;
     if(XGetClassHint(QX11Info::display(), d->win, &hint)) {
@@ -409,7 +389,7 @@ QString Task::className()
     return QString();
 }
 
-QString Task::classClass()
+QString Task::classClass() const
 {
     XClassHint hint;
     if(XGetClassHint(QX11Info::display(), d->win, &hint)) {
@@ -440,6 +420,19 @@ QPixmap Task::icon( int width, int height, bool allowResize )
   return newIcon;
 }
 
+QIcon Task::icon()
+{
+    if ( !d->icon.isNull() )
+        return d->icon;
+
+    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeSmall, KIconLoader::SizeSmall, false));
+    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium, false));
+    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeMedium, KIconLoader::SizeMedium, false));
+    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeLarge, KIconLoader::SizeLarge, false));
+
+    return d->icon;
+}
+
 WindowList Task::transients() const
 {
     return d->transients;
@@ -700,25 +693,23 @@ void Task::activate()
 {
 //    kDebug(1210) << "Task::activate():" << name();
     WId w = d->win;
-    if (d->transientsDemandingAttention.count() > 0)
-    {
-        w = d->transientsDemandingAttention.last();
+    if (!d->transientsDemandingAttention.isEmpty()) {
+        WindowList::const_iterator it = d->transientsDemandingAttention.end();
+        --it;
+        w = *it;
     }
-    KWindowSystem::forceActiveWindow( w );
+
+    KWindowSystem::forceActiveWindow(w);
 }
 
 void Task::activateRaiseOrIconify()
 {
-    if (!isActive() || isIconified())
-    {
+    //kDebug() << isActive() << isIconified() << isOnTop();
+    if (!isActive() || isIconified()) {
         activate();
-    }
-    else if (!isOnTop())
-    {
+    } else if (!isActive() && !isOnTop()) {
        raise();
-    }
-    else
-    {
+    } else {
        setIconified(true);
     }
 }
@@ -726,23 +717,22 @@ void Task::activateRaiseOrIconify()
 void Task::toDesktop(int desk)
 {
     NETWinInfo ni(QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMDesktop);
-    if (desk == 0)
-    {
-        if (d->info.valid() && d->info.onAllDesktops())
-        {
+    if (desk == 0) {
+        if (isOnAllDesktops()) {
             ni.setDesktop(KWindowSystem::currentDesktop());
             KWindowSystem::forceActiveWindow(d->win);
-        }
-        else
-        {
+        } else {
             ni.setDesktop(NETWinInfo::OnAllDesktops);
         }
 
         return;
     }
+
     ni.setDesktop(desk);
-    if(desk == KWindowSystem::currentDesktop())
+
+    if (desk == KWindowSystem::currentDesktop()) {
         KWindowSystem::forceActiveWindow(d->win);
+    }
 }
 
 void Task::toCurrentDesktop()
@@ -837,219 +827,107 @@ void Task::publishIconGeometry(QRect rect)
     ni.setIconGeometry(r);
 }
 
-void Task::updateThumbnail()
+void Task::addMimeData(QMimeData *mimeData) const
 {
-    if ( !d->info.valid() ||
-            !isOnCurrentDesktop() ||
-            !isActive() ||
-            !d->grab.isNull() ) // We're already processing one...
-    {
-        return;
-    }
+    Q_ASSERT(mimeData);
 
-    //
-    // We do this as a two stage process to remove the delay caused
-    // by the thumbnail generation. This makes things much smoother
-    // on slower machines.
-    //
-    QWidget *rootWin = qApp->desktop();
-    QRect geom = d->info.geometry();
-    d->grab = QPixmap::grabWindow(rootWin->winId(),
-                                geom.x(), geom.y(),
-                                geom.width(), geom.height());
-
-    if (!d->grab.isNull())
-    {
-       QTimer::singleShot(200, this, SLOT(generateThumbnail()));
-    }
-}
-
-void Task::generateThumbnail()
-{
-   if ( d->grab.isNull() )
-      return;
-
-   double width = d->grab.width();
-   double height = d->grab.height();
-   width = width * d->thumbSize;
-   height = height * d->thumbSize;
-
-   d->thumb = d->grab.scaled( qRound(width), qRound(height), Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
-   d->grab = QPixmap(); // Makes grab a null image.
-
-   emit thumbnailChanged();
-}
-
-#ifdef THUMBNAILING_POSSIBLE
-QPixmap Task::thumbnail(int maxDimension)
-{
-    if (!KWindowSystem::compositingActive() || !d->windowPixmap)
-    {
-        return QPixmap();
-    }
-
-    Display *dpy = QX11Info::display();
-
-    XWindowAttributes winAttr;
-    XGetWindowAttributes(dpy, d->frameId, &winAttr);
-    XRenderPictFormat *format = XRenderFindVisualFormat(dpy, winAttr.visual);
-
-    XRenderPictureAttributes picAttr;
-    ::memset(&picAttr, 0, sizeof(picAttr));
-    picAttr.subwindow_mode = IncludeInferiors; // Don't clip child widgets
-
-    Picture picture = XRenderCreatePicture(dpy, d->windowPixmap, format,
-                                           CPSubwindowMode, &picAttr);
-
-    // Get shaped windows handled correctly.
-    XserverRegion region = XFixesCreateRegionFromWindow(dpy, d->frameId,
-                                                        WindowRegionBounding);
-    XFixesSetPictureClipRegion(dpy, picture, 0, 0, region);
-    XFixesDestroyRegion(dpy, region);
-
-    double factor;
-    if (winAttr.width > winAttr.height)
-    {
-        factor = (double)maxDimension / (double)winAttr.width;
-    }
-    else
-    {
-        factor = (double)maxDimension / (double)winAttr.height;
-    }
-    int thumbnailWidth = (int)(winAttr.width * factor);
-    int thumbnailHeight = (int)(winAttr.height * factor);
-
-    QPixmap thumbnail(thumbnailWidth, thumbnailHeight);
-    thumbnail.fill(QApplication::palette().color(QPalette::Active,QPalette::Background));
-
-#if 0 // QImage::smoothScale() scaling
-    QPixmap full(winAttr.width, winAttr.height);
-    full.fill(QApplication::palette().active().background());
-
-    bool hasAlpha = format->type == PictTypeDirect && format->direct.alphaMask;
-
-    XRenderComposite(dpy,
-                     hasAlpha ? PictOpOver : PictOpSrc,
-                     picture, // src
-                     None, // mask
-                     full.x11RenderHandle(), // dst
-                     0, 0, // src offset
-                     0, 0, // mask offset
-                     0, 0, // dst offset
-                     winAttr.width, winAttr.height);
-
-    KPixmapIO io;
-    QImage image = io.toImage(full);
-    thumbnail = io.convertToPixmap(image.smoothScale(thumbnailWidth,
-                                                     thumbnailHeight));
-#else // XRENDER scaling
-    // Scaling matrix
-    XTransform transformation = {{
-        { XDoubleToFixed(1), XDoubleToFixed(0), XDoubleToFixed(     0) },
-        { XDoubleToFixed(0), XDoubleToFixed(1), XDoubleToFixed(     0) },
-        { XDoubleToFixed(0), XDoubleToFixed(0), XDoubleToFixed(factor) }
-    }};
-
-    XRenderSetPictureTransform(dpy, picture, &transformation);
-    XRenderSetPictureFilter(dpy, picture, FilterBest, 0, 0);
-
-    XRenderComposite(QX11Info::display(),
-                     PictOpOver, // we're filtering, alpha values are probable
-                     picture, // src
-                     None, // mask
-                     thumbnail.x11PictureHandle(), // dst
-                     0, 0, // src offset
-                     0, 0, // mask offset
-                     0, 0, // dst offset
-                     thumbnailWidth, thumbnailHeight);
-#endif
-    XRenderFreePicture(dpy, picture);
-
-    return thumbnail;
-}
-#else // THUMBNAILING_POSSIBLE
-QPixmap Task::thumbnail(int /* maxDimension */)
-{
-    return QPixmap();
-}
-#endif // THUMBNAILING_POSSIBLE
-
-void Task::updateWindowPixmap()
-{
-#ifdef THUMBNAILING_POSSIBLE
-    if (!KWindowSystem::compositingActive() || !isOnCurrentDesktop() ||
-        isMinimized())
-    {
-        return;
-    }
-
-    Display *dpy = QX11Info::display();
-
-    if (d->windowPixmap)
-    {
-        XFreePixmap(dpy, d->windowPixmap);
-    }
-
-    d->windowPixmap = XCompositeNameWindowPixmap(dpy, d->frameId);
-#endif // THUMBNAILING_POSSIBLE
-}
-
-int TaskManager::currentDesktop() const
-{
-    return KWindowSystem::currentDesktop();
-}
-
-TaskDrag::TaskDrag(const TaskList& tasks, QWidget* source)
-  : QDrag(source),
-    d(0)
-{
     QByteArray data;
-    QDataStream stream(&data, QIODevice::WriteOnly);
+    data.resize(sizeof(WId));
+    memcpy(data.data(), &d->win, sizeof(WId));
+    mimeData->setData(mimetype(), data);
+}
 
-    stream.setVersion(QDataStream::Qt_3_1);
+QString Task::mimetype()
+{
+    return "windowsystem/winid";
+}
 
-    TaskList::const_iterator itEnd = tasks.constEnd();
-    for (TaskList::const_iterator it = tasks.constBegin(); it != itEnd; ++it)
-    {
-        stream << (quint32)(*it)->window();
+QString Task::groupMimetype()
+{
+    return "windowsystem/multiple-winids";
+}
+
+QList<WId> Task::idsFromMimeData(const QMimeData *mimeData, bool *ok)
+{
+    Q_ASSERT(mimeData);
+    QList<WId> ids;
+
+    if (ok) {
+        *ok = false;
     }
 
-    QMimeData* mimeData = new QMimeData();
-    mimeData->setData("taskbar/task", data);
-    setMimeData(mimeData);
-}
+    if (!mimeData->hasFormat(groupMimetype())) {
+        // try to grab a singular id if it exists
+        //kDebug() << "not group type";
+        bool singularOk;
+        WId id = idFromMimeData(mimeData, &singularOk);
 
-TaskDrag::~TaskDrag()
-{
-}
-
-bool TaskDrag::canDecode(const QMimeData* e)
-{
-    return e->hasFormat("taskbar/task");
-}
-
-TaskList TaskDrag::decode( const QMimeData* e )
-{
-    QByteArray data(e->data("taskbar/task"));
-    TaskList tasks;
-
-    if (data.size())
-    {
-        QDataStream stream(data);
-        while (!stream.atEnd())
-        {
-            quint32 id;
-            stream >> id;
-            if (TaskPtr task = TaskManager::self()->findTask(id))
-            {
-                tasks.append(task);
-            }
+        if (ok) {
+            *ok = singularOk;
         }
+
+        if (singularOk) {
+            //kDebug() << "and singular failed, too";
+            ids << id;
+        }
+
+        return ids;
     }
 
-    return tasks;
+    QByteArray data(mimeData->data(groupMimetype()));
+    if ((unsigned int)data.size() < sizeof(int) + sizeof(WId)) {
+        //kDebug() << "wrong size" << data.size() << sizeof(int) + sizeof(WId);
+        return ids;
+    }
+
+    int count = 0;
+    memcpy(&count, data.data(), sizeof(int));
+    if (count < 1 || (unsigned int)data.size() < sizeof(int) + sizeof(WId) * count) {
+        //kDebug() << "wrong size, 2" << data.size() << count << sizeof(int) + sizeof(WId) * count;
+        return ids;
+    }
+
+    WId id;
+    for (int i = 0; i < count; ++i) {
+        memcpy(&id, data.data() + sizeof(int) + sizeof(WId) * i, sizeof(WId));
+        ids << id;
+    }
+
+    if (ok) {
+        *ok = true;
+    }
+
+    return ids;
 }
- 
+
+WId Task::idFromMimeData(const QMimeData *mimeData, bool *ok)
+{
+    Q_ASSERT(mimeData);
+
+    if (ok) {
+        *ok = false;
+    }
+
+    if (!mimeData->hasFormat(mimetype())) {
+        return 0;
+    }
+
+    QByteArray data(mimeData->data(mimetype()));
+    if (data.size() != sizeof(WId)) {
+        return 0;
+    }
+
+    WId id;
+    memcpy(&id, data.data(), sizeof(WId));
+
+    if (ok) {
+        *ok = true;
+    }
+
+    return id;
+}
+
 } // TaskManager namespace
 
 #include "task.moc"
+
