@@ -23,7 +23,6 @@
 // Qt
 #include <QAction>
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QGraphicsView>
 #include <QCheckBox>
 #include <QVBoxLayout>
@@ -38,9 +37,10 @@
 #include <KProcess>
 
 // Plasma
-#include <plasma/widgets/icon.h>
-#include <plasma/containment.h>
-#include <plasma/view.h>
+#include <Plasma/IconWidget>
+#include <Plasma/Containment>
+#include <Plasma/View>
+#include <Plasma/ToolTipManager>
 
 // Local
 #include "ui/launcher.h"
@@ -48,41 +48,53 @@
 class LauncherApplet::Private
 {
 public:
-    Plasma::Icon *icon;
+    Private(LauncherApplet *lApplet) : launcher(0), switcher(0), q(lApplet) { }
+    ~Private() {
+        delete launcher;
+    }
+    void createLauncher();
+    void initToolTip();
+
     Kickoff::Launcher *launcher;
 
     QCheckBox *switchOnHoverCheckBox;
     KIntNumInput *visibleCountEdit;
     QList<QAction*> actions;
     QAction* switcher;
-
-    Private() : launcher(0), switcher(0) {}
-    ~Private() { delete launcher; }
-    void createLauncher(LauncherApplet *q);
+    LauncherApplet *q;
 };
 
-void LauncherApplet::Private::createLauncher(LauncherApplet *q)
+void LauncherApplet::Private::createLauncher()
 {
+    if (launcher) {
+        return;
+    }
+
     launcher = new Kickoff::Launcher(q);
-    launcher->setWindowFlags(launcher->windowFlags()|Qt::WindowStaysOnTopHint|Qt::Popup);
+    launcher->setAttribute(Qt::WA_NoSystemBackground);
     launcher->setAutoHide(true);
-    launcher->resize(launcher->sizeHint());
-    QObject::connect(launcher, SIGNAL(aboutToHide()), icon, SLOT(setUnpressed()));
-    QObject::connect(launcher, SIGNAL(configNeedsSaving()), q, SIGNAL(configNeedsSaving()));
+    QObject::connect(launcher, SIGNAL(aboutToHide()), q, SLOT(hidePopup()));
+    //launcher->resize(launcher->sizeHint());
+    //QObject::connect(launcher, SIGNAL(aboutToHide()), icon, SLOT(setUnpressed()));
+    //QObject::connect(launcher, SIGNAL(configNeedsSaving()), q, SIGNAL(configNeedsSaving()));
+}
+
+void LauncherApplet::Private::initToolTip()
+{
+    Plasma::ToolTipContent data(i18n("Kickoff Application Launcher"),
+                                i18n("Favorites, applications, computer places, "
+                                     "recently used items and desktop sessions"),
+                                q->popupIcon().pixmap(IconSize(KIconLoader::Desktop)));
+    Plasma::ToolTipManager::self()->setContent(q, data);
 }
 
 LauncherApplet::LauncherApplet(QObject *parent, const QVariantList &args)
-    : Plasma::Applet(parent,args),
-      d(new Private)
+        : Plasma::PopupApplet(parent, args),
+        d(new Private(this))
 {
     KGlobal::locale()->insertCatalog("plasma_applet_launcher");
-
     setHasConfigurationInterface(true);
-    setBackgroundHints(NoBackground);
-    d->icon = new Plasma::Icon(KIcon("start-here-kde"), QString(), this);
-    d->icon->setFlag(ItemIsMovable, false);
-    connect(d->icon, SIGNAL(pressed(bool)), this, SLOT(toggleMenu(bool)));
-    connect(this, SIGNAL(activate()), this, SLOT(toggleMenu()));
+    setPopupIcon("start-here-kde");
 }
 
 LauncherApplet::~LauncherApplet()
@@ -92,42 +104,23 @@ LauncherApplet::~LauncherApplet()
 
 void LauncherApplet::init()
 {
-    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
-    layout->setContentsMargins(0, 0, 0, 0);
-    layout->setSpacing(0);
-
-    layout->addItem(d->icon);
-
     if (KService::serviceByStorageId("kde4-kmenuedit.desktop")) {
         QAction* menueditor = new QAction(i18n("Menu Editor"), this);
         d->actions.append(menueditor);
         connect(menueditor, SIGNAL(triggered(bool)), this, SLOT(startMenuEditor()));
     }
 
-    setAspectRatioMode(Plasma::ConstrainedSquare);
-
-    Q_ASSERT( ! d->switcher );
+    Q_ASSERT(! d->switcher);
     d->switcher = new QAction(i18n("Switch to Classic Menu Style"), this);
-    d->switcher->setVisible(immutability() == Plasma::Mutable);
     d->actions.append(d->switcher);
     connect(d->switcher, SIGNAL(triggered(bool)), this, SLOT(switchMenuStyle()));
-    resize(IconSize(KIconLoader::Desktop),IconSize(KIconLoader::Desktop));
-    d->icon->resize(contentsRect().size());
+
+    constraintsEvent(Plasma::ImmutableConstraint);
+    Plasma::ToolTipManager::self()->registerWidget(this);
 }
 
 void LauncherApplet::constraintsEvent(Plasma::Constraints constraints)
 {
-    setBackgroundHints(NoBackground);
-    if (constraints & Plasma::FormFactorConstraint) {
-        if (formFactor() == Plasma::Planar ||
-            formFactor() == Plasma::MediaCenter) {
-            //FIXME set correct minimum size
-            //setMinimumSize(d->icon->sizeFromIconSize(IconSize(KIconLoader::Desktop)));
-        } else {
-            //setMinimumSize(d->icon->sizeFromIconSize(IconSize(KIconLoader::Small)));
-        }
-    }
-
     if ((constraints & Plasma::ImmutableConstraint) && d->switcher) {
         d->switcher->setVisible(immutability() == Plasma::Mutable);
     }
@@ -152,83 +145,58 @@ void LauncherApplet::createConfigurationInterface(KConfigDialog *parent)
     QVBoxLayout *layout = new QVBoxLayout(widget);
     widget->setLayout(layout);
 
-    QHBoxLayout *vl = new QHBoxLayout(widget);
-    layout->addLayout(vl);
-
     d->switchOnHoverCheckBox = new QCheckBox(i18n("Switch tabs on hover"), widget);
     layout->addWidget(d->switchOnHoverCheckBox);
+    layout->addStretch();
 
-    parent->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
-    parent->addPage(widget, parent->windowTitle(), icon());
-
-    if (!d->launcher) {
-        d->createLauncher(this);
-    }
+    parent->addPage(widget, i18n("General"), icon());
 
+    d->createLauncher();
     d->switchOnHoverCheckBox->setChecked(d->launcher->switchTabsOnHover());
 }
 
+void LauncherApplet::popupEvent(bool show)
+{
+    if (show) {
+        Plasma::ToolTipManager::self()->clearContent(this);
+        d->launcher->setLauncherOrigin(popupPlacement(), location());
+        d->createLauncher();
+    }
+}
+
+void LauncherApplet::toolTipAboutToShow()
+{
+    if (d->launcher->isVisible()) {
+        Plasma::ToolTipManager::self()->clearContent(this);
+    } else {
+        d->initToolTip();
+    }
+}
+
 void LauncherApplet::configAccepted()
 {
     bool switchTabsOnHover = d->switchOnHoverCheckBox->isChecked();
 
     // TODO: should this be moved into Launcher as well? perhaps even the config itself?
-    KConfigGroup cg = config();
+    KConfigGroup cg = globalConfig();
     cg.writeEntry("SwitchTabsOnHover", switchTabsOnHover);
     emit configNeedsSaving();
 
-    if (!d->launcher) {
-        d->createLauncher(this);
-    }
-
+    d->createLauncher();
     d->launcher->setSwitchTabsOnHover(switchTabsOnHover);
 }
 
-
-void LauncherApplet::toggleMenu(bool pressed)
-{
-    if (pressed) {
-        toggleMenu();
-    }
-}
-
-void LauncherApplet::toggleMenu()
-{
-    if (!d->launcher) {
-        d->createLauncher(this);
-    }
-
-    d->launcher->reset();
-
-    if (!d->launcher->isVisible()) {
-        // It's risky to calculate the popupPosition based on the size before
-        // calling setLauncherOrigin, which can change the size
-        // Probably just a problem on screens with strange aspect ratio or smaller than
-        // kickoff's size.
-        QPoint popupPosition = Applet::popupPosition(d->launcher->size());
-        d->launcher->move( popupPosition );
-        QPoint iconPosition = view()->mapToGlobal(
-                view()->mapFromScene( d->icon->scenePos() ) );
-
-        Plasma::View *pv = dynamic_cast<Plasma::View *>(view());
-        Plasma::Location loc = Plasma::Floating;
-        if (pv) {
-            loc = pv->containment()->location();
-        }
-
-        d->launcher->setLauncherOrigin( iconPosition, loc );
-        emit releaseVisualFocus();
-    }
-
-    d->launcher->setVisible(!d->launcher->isVisible());
-    d->icon->setPressed();
-}
-
 QList<QAction*> LauncherApplet::contextualActions()
 {
-  return d->actions;
+    return d->actions;
+}
+
+QWidget *LauncherApplet::widget()
+{
+    d->createLauncher();
+    return d->launcher;
 }
 
 #include "applet.moc"
