@@ -26,7 +26,6 @@
 #include <QStyleOptionGraphicsItem>
 #include <QFont>
 #include <QGraphicsSceneHoverEvent>
-#include <QDesktopWidget>
 #include <QTimer>
 #include <QX11Info>
 
@@ -40,23 +39,30 @@
 #include <KToolInvocation>
 #include <kmanagerselection.h>
 
-#include <plasma/svg.h>
-#include <plasma/panelsvg.h>
-#include <plasma/theme.h>
-#include <plasma/animator.h>
+#include <Plasma/Svg>
+#include <Plasma/FrameSvg>
+#include <Plasma/PaintUtils>
+#include <Plasma/Theme>
+#include <Plasma/ToolTipManager>
+#include <Plasma/Animator>
 
-const int WINDOW_UPDATE_DELAY = 500;
+#include <kephal/screens.h>
+
+#include <taskmanager/task.h>
+
+const int FAST_UPDATE_DELAY = 200;
+const int UPDATE_DELAY = 500;
 const int DRAG_SWITCH_DELAY = 1000;
 
 Pager::Pager(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args),
-      m_dialog(0),
-      m_displayedText(Number),
+      m_displayedText(None),
       m_showWindowIcons(false),
       m_showOwnBackground(false),
       m_rows(2),
       m_columns(0),
       m_hoverIndex(-1),
+      m_colorScheme(0),
       m_dragId(0),
       m_dirtyDesktop(-1),
       m_dragStartDesktop(-1),
@@ -67,9 +73,9 @@ Pager::Pager(QObject *parent, const QVariantList &args)
     setAcceptDrops(true);
     setHasConfigurationInterface(true);
 
-    m_background = new Plasma::PanelSvg(this);
+    m_background = new Plasma::FrameSvg(this);
     m_background->setImagePath("widgets/pager");
-    m_background->setCacheAllRenderedPanels(true);
+    m_background->setCacheAllRenderedFrames(true);
 
     // initialize with a decent default
     m_desktopCount = KWindowSystem::numberOfDesktops();
@@ -80,7 +86,7 @@ Pager::Pager(QObject *parent, const QVariantList &args)
 void Pager::init()
 {
     createMenu();
-    
+
     KConfigGroup cg = config();
     m_displayedText = (DisplayedText)cg.readEntry("displayedText", (int)m_displayedText);
     m_showWindowIcons = cg.readEntry("showWindowIcons", m_showWindowIcons);
@@ -109,7 +115,10 @@ void Pager::init()
     connect(KWindowSystem::self(), SIGNAL(stackingOrderChanged()), this, SLOT(stackingOrderChanged()));
     connect(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned int)), this, SLOT(windowChanged(WId,unsigned int)));
     connect(KWindowSystem::self(), SIGNAL(showingDesktopChanged(bool)), this, SLOT(showingDesktopChanged(bool)));
-    connect(QApplication::desktop(), SIGNAL(resized(int)), SLOT(desktopsSizeChanged()));
+    connect(Kephal::Screens::self(), SIGNAL(screenAdded(Kephal::Screen *)), SLOT(desktopsSizeChanged()));
+    connect(Kephal::Screens::self(), SIGNAL(screenRemoved(int)), SLOT(desktopsSizeChanged()));
+    connect(Kephal::Screens::self(), SIGNAL(screenResized(Kephal::Screen *, QSize, QSize)), SLOT(desktopsSizeChanged()));
+    connect(Kephal::Screens::self(), SIGNAL(screenMoved(Kephal::Screen *, QPoint, QPoint)), SLOT(desktopsSizeChanged()));
 
     m_desktopLayoutOwner = new KSelectionOwner( QString( "_NET_DESKTOP_LAYOUT_S%1" )
         .arg( QX11Info::appScreen()).toLatin1().constData(), QX11Info::appScreen(), this );
@@ -117,11 +126,18 @@ void Pager::init()
     if ( !m_desktopLayoutOwner->claim( false ))
         lostDesktopLayoutOwner();
 
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeRefresh()));
+
     recalculateGeometry();
 
     m_currentDesktop = KWindowSystem::currentDesktop();
 }
 
+Pager::~Pager()
+{
+    delete m_colorScheme;
+}
+
 void Pager::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & Plasma::SizeConstraint) {
@@ -129,7 +145,7 @@ void Pager::constraintsEvent(Plasma::Constraints constraints)
         recalculateWindowRects();
         if (m_background->hasElementPrefix(QString())) {
             m_background->setElementPrefix(QString());
-            m_background->resizePanel(size());
+            m_background->resizeFrame(size());
         }
     }
     if (constraints & Plasma::FormFactorConstraint) {
@@ -143,6 +159,15 @@ void Pager::constraintsEvent(Plasma::Constraints constraints)
     }
 }
 
+KColorScheme *Pager::colorScheme()
+{
+    if (!m_colorScheme) {
+        m_colorScheme = new KColorScheme(QPalette::Active, KColorScheme::View, Plasma::Theme::defaultTheme()->colorScheme());
+    }
+
+    return m_colorScheme;
+}
+
 void Pager::createMenu()
 {
     QAction* configureDesktop = new QAction(SmallIcon("configure"),i18n("&Configure Desktops..."), this);
@@ -165,52 +190,30 @@ void Pager::createConfigurationInterface(KConfigDialog *parent)
 {
     QWidget *widget = new QWidget();
     ui.setupUi(widget);
-    parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
-    parent->addPage(widget, parent->windowTitle(), icon());
+    parent->addPage(widget, i18n("General"), icon());
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+    connect(ui.configureDesktopsButton, SIGNAL(clicked()), SLOT(slotConfigureDesktop()));
+
+    switch (m_displayedText){
+        case Number:
+            ui. desktopNumberRadioButton->setChecked(true);
+            break;
+
+        case Name:
+            ui.desktopNameRadioButton->setChecked(true);
+            break;
+
+        case None:
+            ui.displayNoneRadioButton->setChecked(true);
+            break;
+    }
 
-    ui.displayedTextComboBox->clear();
-    ui.displayedTextComboBox->addItem(i18n("Desktop Number"));
-    ui.displayedTextComboBox->addItem(i18n("Desktop Name"));
-    ui.displayedTextComboBox->addItem(i18n("None"));
-    ui.displayedTextComboBox->setCurrentIndex((int)m_displayedText);
-    ui.displayedTextComboBox->setToolTip(i18n("What will appear when the mouse is over a desktop miniature"));
     ui.showWindowIconsCheckBox->setChecked(m_showWindowIcons);
     ui.spinRows->setValue(m_rows);
     ui.spinRows->setMaximum(m_desktopCount);
 }
 
-bool Pager::posOnDesktopRect(const QRectF& r, const QPointF& pos)
-{
-    qreal leftMargin;
-    qreal topMargin;
-    qreal rightMargin;
-    qreal bottomMargin;
-
-    if (m_showOwnBackground && m_background) {
-        m_background->setElementPrefix(QString());
-        m_background->getMargins(leftMargin, topMargin, rightMargin, bottomMargin);
-
-        if (r.left() > leftMargin) {
-            leftMargin = 0;
-        }
-        if (r.top() > topMargin) {
-            leftMargin = 0;
-        }
-        if (geometry().width() - r.right() < rightMargin) {
-            leftMargin = 0;
-        }
-        if (geometry().bottom() - r.bottom() < bottomMargin) {
-            leftMargin = 0;
-        }
-
-        return r.adjusted(-leftMargin, -topMargin, rightMargin, bottomMargin).contains(pos);
-    } else {
-        return r.contains(pos);
-    }
-}
-
 void Pager::recalculateGeometry()
 {
     if (!m_rects.isEmpty() && geometry().size() == m_size) {
@@ -221,23 +224,39 @@ void Pager::recalculateGeometry()
     int padding = 2; // Space between miniatures of desktops
     int textMargin = 3; // Space between name of desktop and border
     int columns = m_desktopCount / m_rows + m_desktopCount % m_rows;
+    int rows = m_rows;
 
-    qreal leftMargin;
-    qreal topMargin;
-    qreal rightMargin;
-    qreal bottomMargin;
+    //inverse rows and columns in vertical panel
+    if (formFactor() == Plasma::Vertical) {
+        rows = columns;
+        columns = m_rows;
+    }
+
+    qreal leftMargin = 0;
+    qreal topMargin = 0;
+    qreal rightMargin = 0;
+    qreal bottomMargin = 0;
 
     if (formFactor() == Plasma::Vertical || formFactor() == Plasma::Horizontal) {
         m_background->setElementPrefix(QString());
         m_background->getMargins(leftMargin, topMargin, rightMargin, bottomMargin);
 
-        qreal ratio = (qreal)QApplication::desktop()->width() / (qreal)QApplication::desktop()->height();
+        qreal ratio = (qreal)Kephal::ScreenUtils::desktopGeometry().width() / (qreal)Kephal::ScreenUtils::desktopGeometry().height();
 
-        //if the final size is going to be really tiny avoid to add extra margins
-        if (geometry().width() - leftMargin - rightMargin < KIconLoader::SizeSmall*ratio * columns + padding*(columns-1) ||
-            geometry().height() - topMargin - bottomMargin < KIconLoader::SizeSmall * m_rows + padding*(m_rows-1)) {
-            m_showOwnBackground = false;
-            leftMargin = topMargin = rightMargin = bottomMargin = padding = textMargin = 0;
+        if (formFactor() == Plasma::Vertical) {
+            qreal optimalSize = (geometry().width() - KIconLoader::SizeSmall*ratio * columns + padding*(columns-1)) / 2;
+
+            if (optimalSize < leftMargin || optimalSize < rightMargin) {
+                leftMargin = rightMargin = qMax(qreal(0), optimalSize);
+                m_showOwnBackground = false;
+            }
+        } else if (formFactor() == Plasma::Horizontal) {
+            qreal optimalSize = (geometry().height() - KIconLoader::SizeSmall*rows + padding*(rows-1)) / 2;
+
+            if (optimalSize < topMargin || optimalSize < bottomMargin) {
+                topMargin = bottomMargin =  qMax(qreal(0), optimalSize);
+                m_showOwnBackground = false;
+            }
         } else {
             m_showOwnBackground = true;
         }
@@ -249,14 +268,14 @@ void Pager::recalculateGeometry()
     qreal itemWidth;
 
     if (formFactor() == Plasma::Vertical) {
-        itemWidth = (geometry().width() - leftMargin - rightMargin - padding * (columns - 1)) / columns;
-        m_widthScaleFactor = itemWidth / QApplication::desktop()->width();
-        itemHeight = QApplication::desktop()->height() * m_widthScaleFactor;
+        itemWidth = (contentsRect().width() - leftMargin - rightMargin - padding * (columns - 1)) / columns;
+        m_widthScaleFactor = itemWidth / Kephal::ScreenUtils::desktopGeometry().width();
+        itemHeight = Kephal::ScreenUtils::desktopGeometry().height() * m_widthScaleFactor;
         m_heightScaleFactor = m_widthScaleFactor;
     } else {
-        itemHeight = (geometry().height() - topMargin -  bottomMargin - padding * (m_rows - 1)) / m_rows;
-        m_heightScaleFactor = itemHeight / QApplication::desktop()->height();
-        itemWidth = QApplication::desktop()->width() * m_heightScaleFactor;
+        itemHeight = (contentsRect().height() - topMargin - bottomMargin - padding * (rows - 1)) / rows;
+        m_heightScaleFactor = itemHeight / Kephal::ScreenUtils::desktopGeometry().height();
+        itemWidth = Kephal::ScreenUtils::desktopGeometry().width() * m_heightScaleFactor;
         if (m_displayedText == Name) {
             // When containment is in this position we are not limited by low width and we can
             // afford increasing width of applet to be able to display every name of desktops
@@ -268,17 +287,15 @@ void Pager::recalculateGeometry()
                 }
             }
         }
-        m_widthScaleFactor = itemWidth / QApplication::desktop()->width();
+        m_widthScaleFactor = itemWidth / Kephal::ScreenUtils::desktopGeometry().width();
     }
 
     m_rects.clear();
     m_animations.clear();
-    QRectF itemRect;
-    itemRect.setWidth(floor(itemWidth - 1));
-    itemRect.setHeight(floor(itemHeight - 1));
+    QRectF itemRect(QPoint(leftMargin, topMargin) , QSize(floor(itemWidth), floor(itemHeight)));
     for (int i = 0; i < m_desktopCount; i++) {
-        itemRect.moveLeft(leftMargin + floor((i % columns) * (itemWidth + padding)));
-        itemRect.moveTop(topMargin + floor((i / columns) * (itemHeight + padding)));
+        itemRect.moveLeft(leftMargin + floor(i % columns  * (itemWidth + padding)));
+        itemRect.moveTop(topMargin + floor(i / columns * (itemHeight + padding)));
         m_rects.append(itemRect);
         AnimInfo anim;
         anim.animId = -1;
@@ -290,19 +307,21 @@ void Pager::recalculateGeometry()
     //Resize background svgs as needed
     if (m_background->hasElementPrefix("normal")) {
         m_background->setElementPrefix("normal");
-        m_background->resizePanel(itemRect.size());
+        m_background->resizeFrame(itemRect.size());
     }
+
     if (m_background->hasElementPrefix("active")) {
         m_background->setElementPrefix("active");
-        m_background->resizePanel(itemRect.size());
+        m_background->resizeFrame(itemRect.size());
     }
+
     if (m_background->hasElementPrefix("hover")) {
         m_background->setElementPrefix("hover");
-        m_background->resizePanel(itemRect.size());
+        m_background->resizeFrame(itemRect.size());
     }
 
     m_size = QSizeF(ceil(columns * itemWidth + padding * (columns - 1) + leftMargin + rightMargin),
-                    ceil(m_rows * itemHeight + padding * (m_rows - 1) + topMargin + bottomMargin));
+                    ceil(rows * itemHeight + padding * (rows - 1) + topMargin + bottomMargin));
 
     //kDebug() << "new size set" << m_size << m_rows << m_columns << columns << itemWidth;
 
@@ -313,7 +332,7 @@ void Pager::recalculateGeometry()
         m_columns = columns;
         NET::Orientation orient = NET::OrientationHorizontal;
         NETRootInfo i( QX11Info::display(), 0 );
-        i.setDesktopLayout( orient, columns, m_rows, NET::DesktopLayoutCornerTopLeft );
+        i.setDesktopLayout( orient, columns, rows, NET::DesktopLayoutCornerTopLeft );
     }
 }
 
@@ -325,8 +344,9 @@ void Pager::recalculateWindowRects()
         m_windowRects.append(QList<QPair<WId, QRect> >());
     }
     m_activeWindows.clear();
+    m_windowInfo.clear();
     foreach(WId window, windows) {
-        KWindowInfo info = KWindowSystem::windowInfo(window, NET::WMGeometry | NET::WMFrameExtents | NET::WMWindowType | NET::WMDesktop | NET::WMState | NET::XAWMState);
+        KWindowInfo info = KWindowSystem::windowInfo(window, NET::WMGeometry | NET::WMFrameExtents | NET::WMWindowType | NET::WMDesktop | NET::WMState | NET::XAWMState | NET::WMVisibleName);
         NET::WindowType type = info.windowType(NET::NormalMask | NET::DialogMask | NET::OverrideMask |
                                                NET::UtilityMask | NET::DesktopMask | NET::DockMask |
                                                NET::TopMenuMask | NET::SplashMask | NET::ToolbarMask |
@@ -345,9 +365,13 @@ void Pager::recalculateWindowRects()
             if (!info.isOnDesktop(i+1)) {
                 continue;
             }
+
             QRect windowRect = info.frameGeometry();
-            if( KWindowSystem::mapViewport())
+
+            if (KWindowSystem::mapViewport()) {
                 windowRect = fixViewportPosition( windowRect );
+            }
+
             windowRect = QRectF(windowRect.x() * m_widthScaleFactor,
                                 windowRect.y() * m_heightScaleFactor,
                                 windowRect.width() * m_widthScaleFactor,
@@ -357,6 +381,7 @@ void Pager::recalculateWindowRects()
             if (window == KWindowSystem::activeWindow()) {
                 m_activeWindows.append(windowRect);
             }
+            m_windowInfo.append(info);
         }
     }
 
@@ -368,8 +393,20 @@ void Pager::configAccepted()
     KConfigGroup cg = config();
     bool changed = false;
 
-    if ((int)m_displayedText != ui.displayedTextComboBox->currentIndex()) {
-        m_displayedText = (DisplayedText)ui.displayedTextComboBox->currentIndex();
+    DisplayedText displayedText;
+
+    if (ui.desktopNumberRadioButton->isChecked()){
+        displayedText = Number;
+
+    }else if (ui.desktopNameRadioButton->isChecked()){
+        displayedText = Name;
+
+    }else{
+        displayedText = None;
+    }
+
+    if ((int)m_displayedText != (int)displayedText) {
+        m_displayedText = displayedText;
         cg.writeEntry("displayedText", (int)m_displayedText);
         changed = true;
     }
@@ -414,7 +451,7 @@ void Pager::currentDesktopChanged(int desktop)
     m_dirtyDesktop = -1;
 
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(FAST_UPDATE_DELAY);
     }
 }
 
@@ -426,7 +463,7 @@ void Pager::windowAdded(WId id)
     m_dirtyDesktop = info.desktop() - 1;
 
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(FAST_UPDATE_DELAY);
     }
 }
 
@@ -438,7 +475,7 @@ void Pager::windowRemoved(WId id)
     m_dirtyDesktop = info.desktop() - 1;
 
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(FAST_UPDATE_DELAY);
     }
 }
 
@@ -450,7 +487,7 @@ void Pager::activeWindowChanged(WId id)
     m_dirtyDesktop = info.desktop() - 1;
 
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(FAST_UPDATE_DELAY);
     }
 }
 
@@ -461,17 +498,15 @@ void Pager::numberOfDesktopsChanged(int num)
     }
 
     m_dirtyDesktop = -1;
-
     m_desktopCount = num;
+
     if (m_rows > m_desktopCount) {
         m_rows = m_desktopCount;
     }
+
     m_rects.clear();
     recalculateGeometry();
-
-    if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
-    }
+    recalculateWindowRects();
 }
 
 void Pager::desktopNamesChanged()
@@ -482,7 +517,7 @@ void Pager::desktopNamesChanged()
     recalculateGeometry();
 
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(UPDATE_DELAY);
     }
 }
 
@@ -491,7 +526,7 @@ void Pager::stackingOrderChanged()
     m_dirtyDesktop = -1;
 
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(FAST_UPDATE_DELAY);
     }
 }
 
@@ -509,7 +544,7 @@ void Pager::windowChanged(WId id, unsigned int properties)
     if (properties & NET::WMGeometry ||
         properties & NET::WMDesktop) {
         if (!m_timer->isActive()) {
-            m_timer->start(WINDOW_UPDATE_DELAY);
+            m_timer->start(UPDATE_DELAY);
         }
     }
 }
@@ -520,7 +555,7 @@ void Pager::showingDesktopChanged(bool showing)
 
     Q_UNUSED(showing)
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(UPDATE_DELAY);
     }
 }
 
@@ -532,7 +567,7 @@ void Pager::desktopsSizeChanged()
     recalculateGeometry();
 
     if (!m_timer->isActive()) {
-        m_timer->start(WINDOW_UPDATE_DELAY);
+        m_timer->start(UPDATE_DELAY);
     }
 }
 
@@ -540,8 +575,8 @@ void Pager::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
     if (event->buttons() != Qt::RightButton)
     {
-        for (int i = 0; i < m_desktopCount; i++) {
-            if (posOnDesktopRect(m_rects[i], event->pos())) {
+        for (int i = 0; i < m_rects.count(); ++i) {
+            if (m_rects[i].contains(event->pos())) {
                 m_dragStartDesktop = m_dragHighlightedDesktop = i;
                 m_dragOriginalPos = m_dragCurrentPos = event->pos();
                 if (m_dragOriginal.isEmpty()) {
@@ -583,25 +618,23 @@ void Pager::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
     if (m_dragId > 0) {
         m_dragCurrentPos = event->pos();
         m_dragHighlightedDesktop = -1;
-        for (int i = 0; i < m_desktopCount; i++) {
-            if (m_rects[i].contains(event->pos().toPoint())) {
-                m_dragHighlightedDesktop = i;
-                break;
-            }
-        }
         m_hoverRect = QRectF();
+        int i = 0;
         foreach (const QRectF &rect, m_rects) {
             if (rect.contains(event->pos())) {
+                m_dragHighlightedDesktop = i;
                 m_hoverRect = rect;
                 break;
             }
+
+            ++i;
         }
         update();
         event->accept();
         return;
-    } else if (m_dragId != -1 && m_dragStartDesktop != -1 &&
+    } else if (m_dragStartDesktop != -1 &&
                (event->pos() - m_dragOriginalPos).toPoint().manhattanLength() > KGlobalSettings::dndEventDelay()) {
-        m_dragId = -1; // prevent us from going through this more than once
+        m_dragId = 0; // prevent us from going through this more than once
         for (int k = m_windowRects[m_dragStartDesktop].count() - 1; k >= 0 ; k--) {
             if (m_windowRects[m_dragStartDesktop][k].second.contains(m_dragOriginalPos.toPoint())) {
                 m_dragOriginal = m_windowRects[m_dragStartDesktop][k].second;
@@ -619,18 +652,28 @@ void Pager::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
 void Pager::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
-    if (m_dragId > 0) {
+    if (m_dragId) {
         if (m_dragHighlightedDesktop != -1) {
             QPointF dest = m_dragCurrentPos - m_rects[m_dragHighlightedDesktop].topLeft() - m_dragOriginalPos + m_dragOriginal.topLeft();
             dest = QPointF(dest.x()/m_widthScaleFactor, dest.y()/m_heightScaleFactor);
             // don't move windows to negative positions
             dest = QPointF(qMax(dest.x(), qreal(0.0)), qMax(dest.y(), qreal(0.0)));
-            if( !KWindowSystem::mapViewport()) {
-                KWindowSystem::setOnDesktop(m_dragId, m_dragHighlightedDesktop+1);
-                // use _NET_MOVERESIZE_WINDOW rather than plain move, so that the WM knows this is a pager request
-                NETRootInfo i( QX11Info::display(), 0 );
-                int flags = ( 0x20 << 12 ) | ( 0x03 << 8 ) | 1; // from tool, x/y, northwest gravity
-                i.moveResizeWindowRequest( m_dragId, flags, dest.toPoint().x(), dest.toPoint().y(), 0, 0 );
+            if (!KWindowSystem::mapViewport()) {
+                KWindowInfo info = KWindowSystem::windowInfo(m_dragId, NET::WMDesktop);
+
+                if (!info.onAllDesktops()) {
+                    KWindowSystem::setOnDesktop(m_dragId, m_dragHighlightedDesktop+1);
+                }
+
+                // only move the window if it is kept within the same desktop
+                // moving when dropping between desktop is too annoying due to
+                // the small drop area.
+                if (m_dragHighlightedDesktop == m_dragStartDesktop || info.onAllDesktops()) {
+                    // use _NET_MOVERESIZE_WINDOW rather than plain move, so that the WM knows this is a pager request
+                    NETRootInfo i( QX11Info::display(), 0 );
+                    int flags = ( 0x20 << 12 ) | ( 0x03 << 8 ) | 1; // from tool, x/y, northwest gravity
+                    i.moveResizeWindowRequest( m_dragId, flags, dest.toPoint().x(), dest.toPoint().y(), 0, 0 );
+                }
             } else {
                 // setOnDesktop() with viewports is also moving a window, and since it takes a moment
                 // for the WM to do the move, there's a race condition with figuring out how much to move,
@@ -643,7 +686,8 @@ void Pager::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
             }
         }
         m_timer->start();
-    } else if (m_dragStartDesktop != -1 && posOnDesktopRect(m_rects[m_dragStartDesktop], event->pos()) &&
+    } else if (m_dragStartDesktop != -1 && m_dragStartDesktop < m_rects.size() &&
+               m_rects[m_dragStartDesktop].contains(event->pos()) &&
                m_currentDesktop != m_dragStartDesktop + 1) {
         // only change the desktop if the user presses and releases the mouse on the same desktop
         KWindowSystem::setCurrentDesktop(m_dragStartDesktop + 1);
@@ -664,7 +708,7 @@ void Pager::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 // This method provides the common implementation for hoverMoveEvent and dragMoveEvent.
 void Pager::handleHoverMove(const QPointF& pos)
 {
-    bool changedHover = !posOnDesktopRect(m_hoverRect, pos);
+    bool changedHover = !m_hoverRect.contains(pos);
     Plasma::Animator *anim = Plasma::Animator::self();
 
     if (changedHover && m_hoverIndex > -1) {
@@ -682,7 +726,7 @@ void Pager::handleHoverMove(const QPointF& pos)
 
     int i = 0;
     foreach (const QRectF &rect, m_rects) {
-        if (posOnDesktopRect(rect, pos)) {
+        if (rect.contains(pos)) {
             if (m_hoverRect != rect) {
                 m_hoverRect = rect;
                 m_hoverIndex = i;
@@ -693,10 +737,11 @@ void Pager::handleHoverMove(const QPointF& pos)
                 m_animations[m_hoverIndex].alpha = 0;
                 m_animations[m_hoverIndex].animId = anim->customAnimation(40 / (1000 / s_FadeInDuration), s_FadeInDuration,Plasma::Animator::EaseInCurve, this,"animationUpdate");
                 update();
+                updateToolTip();
             }
             return;
         }
-        i++;
+        ++i;
     }
     m_hoverIndex = -1;
     m_hoverRect = QRectF();
@@ -756,8 +801,11 @@ void Pager::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
 
 void Pager::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 {
-    //TODO: we need to accept window id drops eventually
-    event->setAccepted(false);
+    if (event->mimeData()->hasFormat(TaskManager::Task::mimetype())) {
+        event->setAccepted(true);
+        return;
+    }
+    else event->setAccepted(false);
     handleHoverMove(event->pos());
 
     if (m_hoverIndex != -1) {
@@ -790,6 +838,21 @@ void Pager::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
     Applet::dragLeaveEvent(event);
 }
 
+void Pager::dropEvent(QGraphicsSceneDragDropEvent *event)
+{
+    bool ok;
+    WId id = TaskManager::Task::idFromMimeData(event->mimeData(), &ok);
+    if (ok) {
+        for (int i = 0; i < m_rects.count(); ++i) {
+            if (m_rects[i].contains(event->pos().toPoint())) {
+                KWindowSystem::setOnDesktop(id, i + 1);
+                m_dragSwitchDesktop = -1;
+                break;
+            }
+        }
+    }
+}
+
 void Pager::animationUpdate(qreal progress, int animId)
 {
     int i = 0;
@@ -829,7 +892,7 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
     Q_UNUSED( option );
     Q_UNUSED( contentsRect );
 
-    KColorScheme plasmaColorTheme = KColorScheme(QPalette::Active, KColorScheme::View, Plasma::Theme::defaultTheme()->colorScheme());
+    KColorScheme* plasmaColorTheme = colorScheme();
     painter->setFont(KGlobalSettings::taskbarFont());
 
     // Desktop background
@@ -838,37 +901,37 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
     hoverColor.setAlpha(64);
 
     // Inactive windows
-    QColor drawingColor = plasmaColorTheme.foreground(KColorScheme::InactiveText).color();
-    drawingColor.setAlpha(50);
+    QColor drawingColor = plasmaColorTheme->foreground(KColorScheme::InactiveText).color();
+    drawingColor.setAlpha(45);
     QBrush windowBrush(drawingColor);
     // Inactive windows Active desktop
-    drawingColor.setAlpha(192);
+    drawingColor.setAlpha(90);
     QBrush windowBrushActiveDesk(drawingColor);
 
     // Inactive window borders
-    drawingColor = plasmaColorTheme.foreground(KColorScheme::NeutralText).color();
-    drawingColor.setAlpha(238);
+    drawingColor = defaultTextColor;
+    drawingColor.setAlpha(130);
     QPen windowPen(drawingColor);
 
     // Active window borders
     QPen activeWindowPen(defaultTextColor);
 
     // Active windows
-    drawingColor.setAlpha(190);
+    drawingColor.setAlpha(130);
     QBrush activeWindowBrush(drawingColor);
     // Active windows Active desktop
-    drawingColor.setAlpha(228);
+    drawingColor.setAlpha(155);
     QBrush activeWindowBrushActiveDesk(drawingColor);
 
     if (m_showOwnBackground && (formFactor() == Plasma::Vertical || formFactor() == Plasma::Horizontal)) {
         m_background->setElementPrefix(QString());
-        m_background->paintPanel(painter, contentsRect);
+        m_background->paintFrame(painter);
     }
 
     // Draw backgrounds of desktops only when there are not the proper theme elements
     painter->setPen(Qt::NoPen);
     if (!m_background->hasElementPrefix("hover")) {
-        for (int i = 0; i < m_desktopCount; i++) {
+        for (int i = 0; i < m_rects.count(); i++) {
             if (m_rects[i] == m_hoverRect) {
                 QColor animHoverColor = hoverColor;
                 if (m_animations[i].animId > -1) {
@@ -885,7 +948,10 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
     for (int i = 0; i < m_windowRects.count(); i++) {
         for (int j = 0; j < m_windowRects[i].count(); j++) {
             QRect rect = m_windowRects[i][j].second;
-            if (!m_rects.isEmpty() && m_rects[m_currentDesktop-1].contains(rect)) {
+
+            if (m_currentDesktop > 0 &&
+                m_currentDesktop <= m_rects.count() &&
+                m_rects[m_currentDesktop-1].contains(rect)) {
                 if (m_activeWindows.contains(rect)) {
                     painter->setBrush(activeWindowBrushActiveDesk);
                     painter->setPen(activeWindowPen);
@@ -905,7 +971,7 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
             if (m_dragId == m_windowRects[i][j].first) {
                 rect.translate((m_dragCurrentPos - m_dragOriginalPos).toPoint());
                 painter->setClipRect(option->exposedRect);
-            } else {
+            } else if (i < m_rects.count()) {
                 painter->setClipRect(m_rects[i].adjusted(1, 1, -1, -1));
             }
             painter->drawRect(rect);
@@ -921,7 +987,7 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
     painter->setBrush(Qt::NoBrush);
 
     QString prefix;
-    for (int i = 0; i < m_desktopCount; i++) {
+    for (int i = 0; i < m_rects.count(); i++) {
         if (i + 1 == m_currentDesktop || i == m_dragHighlightedDesktop) {
             prefix = "active";
         } else {
@@ -932,36 +998,16 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
         if (m_background->hasElementPrefix(prefix)) {
             m_background->setElementPrefix(prefix);
             if (m_animations[i].animId > -1) {
-                QPixmap pixmap(m_rects[i].size().toSize());
-                QColor alphaColor(0,0,0);
-
-                //alpha of the not-hover element
-                alphaColor.setAlphaF(1.0-m_animations[i].alpha);
-                pixmap.fill(alphaColor);
-
-                QPainter buffPainter(&pixmap);
-                buffPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
-
-                m_background->paintPanel(&buffPainter, QRectF(QPointF(0, 0), m_rects[i].size()));
-                buffPainter.end();
-                painter->drawPixmap(m_rects[i].topLeft(), pixmap);
-
+                QPixmap normal = m_background->framePixmap();
                 m_background->setElementPrefix("hover");
-                //alpha of the not-hover element
-                alphaColor.setAlphaF(m_animations[i].alpha);
-                //FIXME: filling again the old pixmap makes a segfault
-                pixmap = QPixmap(m_rects[i].size().toSize());
-                pixmap.fill(alphaColor);
-                buffPainter.begin(&pixmap);
-                buffPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
-                m_background->paintPanel(&buffPainter, QRectF(QPointF(0, 0), m_rects[i].size()));
-                painter->drawPixmap(m_rects[i].topLeft(), pixmap);
+                QPixmap result = Plasma::PaintUtils::transition(normal, m_background->framePixmap(), m_animations[i].alpha);
+                painter->drawPixmap(m_rects[i].topLeft(), result);
             } else {
                 //no anims, simpler thing
                 if (m_rects[i] == m_hoverRect) {
                     m_background->setElementPrefix("hover");
                 }
-                m_background->paintPanel(painter, m_rects[i], m_rects[i].topLeft());
+                m_background->paintFrame(painter, m_rects[i].topLeft());
             }
         } else {
             QPen drawingPen;
@@ -970,7 +1016,7 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
                 defaultTextColor.setAlphaF(1);
                 drawingPen = QPen(defaultTextColor);
             } else {
-                drawingPen = QPen(plasmaColorTheme.foreground(KColorScheme::InactiveText).color());
+                drawingPen = QPen(plasmaColorTheme->foreground(KColorScheme::InactiveText).color());
             }
 
             painter->setPen(drawingPen);
@@ -984,11 +1030,42 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
         defaultTextColor.setAlphaF(m_animations[i].alpha / 2 + 0.5);
         painter->setPen(defaultTextColor);
 
-        if (m_displayedText==Number) { // Display number of desktop
-            painter->drawText(m_rects[i], Qt::AlignCenter, QString::number(i+1));
-        } else if (m_displayedText==Name) { // Display name of desktop
-            painter->drawText(m_rects[i], Qt::AlignCenter, KWindowSystem::desktopName(i+1));
-         }
+        QColor shadowColor(Qt::black);
+        if (defaultTextColor.value() < 128) {
+            shadowColor = Qt::white;
+        }
+
+        QString desktopText = QString();
+        if (m_displayedText == Number) { // Display number of desktop
+            desktopText = QString::number(i + 1);
+        } else if (m_displayedText == Name) { // Display name of desktop
+            desktopText = KWindowSystem::desktopName(i + 1);
+        }
+
+        if (!desktopText.isEmpty()) {
+            int radius = 2;
+            QPixmap result = Plasma::PaintUtils::shadowText(desktopText,
+                                                            KGlobalSettings::smallestReadableFont(),
+                                                            defaultTextColor,
+                                                            shadowColor, QPoint(0, 0), radius);
+            QRectF target = m_rects[i];
+            //take also shadow position and radius into account
+            //kDebug() << target << result.height();
+            if (target.height() >= result.height() - radius * 2) {
+                QPointF paintPoint = target.center() - (result.rect().center() + QPoint(radius, radius)); 
+
+                if (paintPoint.x() + radius < target.x() + 1) {
+                    paintPoint.setX(target.x() + 1 - radius);
+                }
+
+                if (paintPoint.y() + radius < target.y() + 1) {
+                    paintPoint.setY(target.y() + 1 - radius);
+                }
+
+                target.moveTopLeft(QPointF(0, 0));
+                painter->drawPixmap(paintPoint, result, target);
+            }
+        }
     }
 }
 
@@ -1003,13 +1080,73 @@ void Pager::lostDesktopLayoutOwner()
 // so the offscreen coordinates need to be fixed
 QRect Pager::fixViewportPosition( const QRect& r )
 {
-    int x = r.center().x() % qApp->desktop()->width();
-    int y = r.center().y() % qApp->desktop()->height();
-    if( x < 0 )
-        x = x + qApp->desktop()->width();
-    if( y < 0 )
-        y = y + qApp->desktop()->height();
+    QRect desktopGeom = Kephal::ScreenUtils::desktopGeometry();
+    int x = r.center().x() % desktopGeom.width();
+    int y = r.center().y() % desktopGeom.height();
+    if( x < 0 ) {
+        x = x + desktopGeom.width();
+    }
+    if( y < 0 ) {
+        y = y + desktopGeom.height();
+    }
     return QRect( x - r.width() / 2, y - r.height() / 2, r.width(), r.height());
 }
 
+void Pager::themeRefresh()
+{
+    delete m_colorScheme;
+    m_colorScheme = 0;
+}
+
+void Pager::updateToolTip()
+{
+    int hoverDesktopNumber = 0;
+
+    for (int i = 0; i < m_desktopCount; i++) {
+        if (m_rects[i] == m_hoverRect) {
+            hoverDesktopNumber = i + 1;
+        }
+    }
+
+    Plasma::ToolTipContent data;
+    QString subtext = QString();
+    int taskCounter = 0;
+    int displayedTaskCounter = 0;
+
+    QList<WId> windows;
+
+    foreach(KWindowInfo winInfo, m_windowInfo){
+        if (winInfo.isOnDesktop(hoverDesktopNumber) && !windows.contains(winInfo.win())) {
+            bool active = (winInfo.win() == KWindowSystem::activeWindow());
+            if ((taskCounter < 4) || active){    
+                QPixmap icon = KWindowSystem::icon(winInfo.win(), 16, 16, true);
+                if (icon.isNull()) {
+                     subtext += "<br />&bull;" + winInfo.visibleName();
+                } else {
+                    data.addResource(Plasma::ToolTipContent::ImageResource, QUrl("wicon://" + QString::number(taskCounter)), QVariant(icon));
+                    subtext += "<br /><img src=\"wicon://" + QString::number(taskCounter) + "\"/>";
+                }
+                subtext += (active ? "<u>" : "") + winInfo.visibleName() + (active ? "</u>" : "");
+
+                displayedTaskCounter++; 
+                windows.append(winInfo.win());
+            }
+            taskCounter++;
+        }
+    }
+
+    if (taskCounter) {
+        subtext.prepend(i18np("One window:", "%1 windows:", taskCounter));
+    }
+
+    if (taskCounter - displayedTaskCounter > 0) {
+        subtext.append("<br>&bull; <i>" + i18np("and 1 other", "and %1 others", taskCounter - displayedTaskCounter) + "</i>");
+    }
+
+    data.setMainText(KWindowSystem::desktopName(hoverDesktopNumber));
+    data.setSubText(subtext);
+
+    Plasma::ToolTipManager::self()->setContent(this, data);
+}
+
 #include "pager.moc"
