@@ -1,5 +1,5 @@
 /*
-*   Copyright 2007 by Alex Merry <huntedhacker@tiscali.co.uk>
+*   Copyright 2007 by Alex Merry <alex.merry@kdemail.net>
 *   Copyright 2008 by Alexis Ménard <darktears31@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <QComboBox>
 #include <QAction>
 #include <QGraphicsLayout>
+#include <QGraphicsSceneDragDropEvent>
 
 
 #include <KDebug>
@@ -39,23 +40,64 @@
 #include <KIntNumInput>
 #include <KMessageBox>
 
-#include <plasma/corona.h>
-#include <plasma/panelsvg.h>
-#include <plasma/theme.h>
-#include <plasma/view.h>
+#include <Plasma/Corona>
+#include <Plasma/FrameSvg>
+#include <Plasma/Theme>
+#include <Plasma/View>
+#include <Plasma/PaintUtils>
+
+#include <kephal/screens.h>
 
 using namespace Plasma;
 
+class Spacer : public QGraphicsWidget
+{
+public:
+    Spacer(QGraphicsWidget *parent)
+         : QGraphicsWidget(parent)
+    {
+        setAcceptDrops(true);
+    }
+
+    ~Spacer()
+    {}
+
+    Panel *panel;
+
+protected:
+    void dropEvent(QGraphicsSceneDragDropEvent *event)
+    {
+        event->setPos(mapToParent(event->pos()));
+        panel->dropEvent(event);
+    }
+
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget * widget = 0)
+    {
+        Q_UNUSED(option)
+        Q_UNUSED(widget)
+
+        //TODO: make this a pretty gradient?
+        painter->setRenderHint(QPainter::Antialiasing);
+        QPainterPath p = Plasma::PaintUtils::roundedRectangle(contentsRect().adjusted(1, 1, -2, -2), 4);
+        QColor c = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
+        c.setAlphaF(0.3);
+
+        painter->fillPath(p, c);
+    }
+};
+
 Panel::Panel(QObject *parent, const QVariantList &args)
     : Containment(parent, args),
       m_configureAction(0),
       m_addPanelAction(0),
-      m_currentSize(QSize(QApplication::desktop()->screenGeometry(screen()).width(), 38)),
-      m_lastViewGeom()
+      m_currentSize(QSize(Kephal::ScreenUtils::screenSize(screen()).width(), 35)),
+      m_maskDirty(true),
+      m_spacerIndex(-1),
+      m_spacer(0)
 {
-    m_background = new Plasma::PanelSvg(this);
+    m_background = new Plasma::FrameSvg(this);
     m_background->setImagePath("widgets/panel-background");
-    m_background->setEnabledBorders(Plasma::PanelSvg::AllBorders);
+    m_background->setEnabledBorders(Plasma::FrameSvg::AllBorders);
     connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(backgroundChanged()));
     setZValue(150);
     setContainmentType(Containment::PanelContainment);
@@ -81,8 +123,17 @@ void Panel::init()
     //setFlag(ItemClipsChildrenToShape, true);
 
     KConfigGroup cg = config("Configuration");
+
+    m_currentSize = cg.readEntry("minimumSize", m_currentSize);
+    if (formFactor() == Plasma::Vertical) {
+        m_currentSize.expandedTo(QSize(0, 35));
+    } else {
+        m_currentSize.expandedTo(QSize(35, 0));
+    }
+
     setMinimumSize(cg.readEntry("minimumSize", m_currentSize));
     setMaximumSize(cg.readEntry("maximumSize", m_currentSize));
+    setDrawWallpaper(false);
 }
 
 QList<QAction*> Panel::contextualActions()
@@ -120,11 +171,17 @@ void Panel::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
     Plasma::FormFactor f = formFactor();
     int insertIndex = -1;
 
-    //Enlarge the panel if possible
+    //Enlarge the panel if possible and needed
+    QSizeF appletHint = applet->preferredSize();
+    QSizeF panelHint = layout()->preferredSize();
     if (f == Plasma::Horizontal) {
-        resize(size().width() + applet->preferredWidth(), size().height());
+        if (panelHint.width() + appletHint.width() > size().width()) {
+            resize(panelHint.width() + appletHint.width(), size().height());
+        }
     } else {
-        resize(size().width(), size().height() + applet->preferredHeight());
+        if (panelHint.height() + appletHint.height() > size().height()) {
+            resize(size().width(), panelHint.height() + appletHint.height());
+        }
     }
     layout()->setMaximumSize(size());
 
@@ -159,10 +216,15 @@ void Panel::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
     } else {
         lay->insertItem(insertIndex, applet);
     }
+
+    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
 }
 
 void Panel::appletRemoved(Plasma::Applet* applet)
 {
+    QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(layout());
+    lay->removeItem(applet);
+
     //shrink the panel if possible
     if (formFactor() == Plasma::Horizontal) {
         resize(size().width() - applet->size().width(), size().height());
@@ -172,27 +234,70 @@ void Panel::appletRemoved(Plasma::Applet* applet)
     layout()->setMaximumSize(size());
 }
 
+void Panel::updateSize()
+{
+    Plasma::Applet *applet = qobject_cast<Plasma::Applet *>(sender());
+
+    if (applet) {
+        if (formFactor() == Plasma::Horizontal) {
+            const int delta = applet->preferredWidth() - applet->size().width();
+            //setting the preferred width when delta = 0 and preferredWidth() < minimumWidth()
+            // leads to the same thing as setPreferredWidth(minimumWidth())
+            if (delta != 0) {
+                setPreferredWidth(preferredWidth() + delta);
+            }
+        } else if (formFactor() == Plasma::Vertical) {
+            const int delta = applet->preferredHeight() - applet->size().height();
+            if (delta != 0) {
+                setPreferredHeight(preferredHeight() + delta);
+            }
+        }
+
+        resize(preferredSize());
+    }
+}
+
 void Panel::addPanel()
 {
     if (corona()) {
-        // make a panel at the top
         Containment* panel = corona()->addContainment("panel");
         panel->showConfigurationInterface();
 
         panel->setScreen(screen());
-        panel->setLocation(Plasma::TopEdge);
 
-        // trigger an instant layout so we immediately have a proper geometry 
+        QList<Plasma::Location> freeEdges = corona()->freeEdges(screen());
+        kDebug() << freeEdges;
+        Plasma::Location destination;
+        if (freeEdges.contains(Plasma::TopEdge)) {
+            destination = Plasma::TopEdge;
+        } else if (freeEdges.contains(Plasma::BottomEdge)) {
+            destination = Plasma::BottomEdge;
+        } else if (freeEdges.contains(Plasma::LeftEdge)) {
+            destination = Plasma::LeftEdge;
+        } else if (freeEdges.contains(Plasma::RightEdge)) {
+            destination = Plasma::RightEdge;
+        } else destination = Plasma::TopEdge;
+
+        panel->setLocation(destination);
+
+        // trigger an instant layout so we immediately have a proper geometry
         // rather than waiting around for the event loop
         panel->updateConstraints(Plasma::StartupCompletedConstraint);
         panel->flushPendingConstraintsEvents();
+
+        if (destination == Plasma::LeftEdge ||
+            destination == Plasma::RightEdge) {
+            panel->setMinimumSize(10, 35);
+            panel->setMaximumSize(35,Kephal::ScreenUtils::screenSize(screen()).height());
+            panel->resize(QSize(35, Kephal::ScreenUtils::screenSize(screen()).height()));
+        }
     }
 }
 
 void Panel::updateBorders(const QRect &geom)
 {
     Plasma::Location loc = location();
-    PanelSvg::EnabledBorders enabledBorders = PanelSvg::AllBorders;
+    FrameSvg::EnabledBorders enabledBorders = FrameSvg::AllBorders;
 
     int s = screen();
     //kDebug() << loc << s << formFactor() << geometry();
@@ -206,42 +311,42 @@ void Panel::updateBorders(const QRect &geom)
     if (s < 0) {
         // do nothing in this case, we want all the borders
     } else if (loc == BottomEdge || loc == TopEdge) {
-        QRect r = QApplication::desktop()->screenGeometry(s);
+        QRect r = Kephal::ScreenUtils::screenGeometry(s);
 
         if (loc == BottomEdge) {
-            enabledBorders ^= PanelSvg::BottomBorder;
+            enabledBorders ^= FrameSvg::BottomBorder;
             bottomHeight = 0;
         } else {
-            enabledBorders ^= PanelSvg::TopBorder;
+            enabledBorders ^= FrameSvg::TopBorder;
             topHeight = 0;
         }
 
         if (geom.x() <= r.x()) {
-            enabledBorders ^= PanelSvg::LeftBorder;
+            enabledBorders ^= FrameSvg::LeftBorder;
             leftWidth = 0;
         }
         if (geom.right() >= r.right()) {
-            enabledBorders ^= PanelSvg::RightBorder;
+            enabledBorders ^= FrameSvg::RightBorder;
             rightWidth = 0;
         }
 
         //kDebug() << "top/bottom: Width:" << width << ", height:" << height;
     } else if (loc == LeftEdge || loc == RightEdge) {
-        QRect r = QApplication::desktop()->screenGeometry(s);
+        QRect r = Kephal::ScreenUtils::screenGeometry(s);
 
         if (loc == RightEdge) {
-            enabledBorders ^= PanelSvg::RightBorder;
+            enabledBorders ^= FrameSvg::RightBorder;
             rightWidth = 0;
         } else {
-            enabledBorders ^= PanelSvg::LeftBorder;
+            enabledBorders ^= FrameSvg::LeftBorder;
             leftWidth = 0;
         }
         if (geom.y() <= r.y()) {
-            enabledBorders ^= PanelSvg::TopBorder;
+            enabledBorders ^= FrameSvg::TopBorder;
             topHeight = 0;
         }
         if (geom.bottom() >= r.bottom()) {
-            enabledBorders ^= PanelSvg::BottomBorder;
+            enabledBorders ^= FrameSvg::BottomBorder;
             bottomHeight = 0;
         }
 
@@ -255,26 +360,40 @@ void Panel::updateBorders(const QRect &geom)
     m_background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
 
     //calculation of extra margins has to be done after getMargins
-    if (formFactor() == Vertical) {
-        //hardcoded extra margin for the toolbox right now
-        if (immutability() == Mutable) {
-            bottomHeight += 20;
-        }
-    //Default to horizontal for now
-    } else {
-        //hardcoded extra margin for the toolbox for now
-        if (immutability() == Mutable) {
+    const QGraphicsItem *box = toolBoxItem();
+    if (box && immutability() == Mutable) {
+        QSizeF s = box->boundingRect().size();
+        if (formFactor() == Vertical) {
+            //hardcoded extra margin for the toolbox right now
+            bottomHeight += s.height();;
+            //Default to horizontal for now
+        } else {
             if (QApplication::layoutDirection() == Qt::RightToLeft) {
-                leftWidth += 20;
+                leftWidth += s.width();
             } else {
-                rightWidth += 20;
+                rightWidth += s.width();
             }
         }
     }
 
-
     //invalidate the layout and set again
     if (layout()) {
+        switch (location()) {
+        case LeftEdge:
+            rightWidth = qMin(rightWidth, qMax(qreal(1), size().width() - KIconLoader::SizeMedium));
+            break;
+        case RightEdge:
+            leftWidth = qMin(leftWidth, qMax(qreal(1), size().width() - KIconLoader::SizeMedium));
+            break;
+        case TopEdge:
+            bottomHeight = qMin(bottomHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
+            break;
+        case BottomEdge:
+            topHeight = qMin(topHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
+            break;
+        default:
+            break;
+        }
         layout()->setContentsMargins(leftWidth, topHeight, rightWidth, bottomHeight);
         layout()->invalidate();
     }
@@ -286,48 +405,65 @@ void Panel::constraintsEvent(Plasma::Constraints constraints)
 {
     kDebug() << "constraints updated with" << constraints << "!!!!!!";
 
+    m_maskDirty = true;
+
     if (constraints & Plasma::FormFactorConstraint) {
         Plasma::FormFactor form = formFactor();
-
+        Qt::Orientation layoutDirection = form == Plasma::Vertical ? Qt::Vertical : Qt::Horizontal;
         // create our layout!
-        if (form == Plasma::Horizontal || Plasma::Vertical) {
-            if (layout()) {
-                QGraphicsLayout *lay = layout();
-                QGraphicsLinearLayout * linearLay = dynamic_cast<QGraphicsLinearLayout *>(lay);
-                if (linearLay) {
-                    linearLay->setOrientation(form == Plasma::Horizontal ? Qt::Horizontal :
-                                                                           Qt::Vertical);
-                }
-            } else {
-                QGraphicsLinearLayout *lay = new QGraphicsLinearLayout(this);
-                lay->setOrientation(form == Plasma::Horizontal ? Qt::Horizontal : Qt::Vertical);
-                lay->setContentsMargins(0, 0, 0, 0);
-                lay->setSpacing(4);
-                lay->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
-                setLayout(lay);
-                updateBorders(geometry().toRect());
-
-                foreach (Applet *applet, applets()) {
-                    lay->addItem(applet);
-                }
+        if (layout()) {
+            QGraphicsLayout *lay = layout();
+            QGraphicsLinearLayout * linearLay = dynamic_cast<QGraphicsLinearLayout *>(lay);
+            if (linearLay) {
+                linearLay->setOrientation(layoutDirection);
             }
+            linearLay->setMaximumSize(size());
+        } else {
+            QGraphicsLinearLayout *lay = new QGraphicsLinearLayout(this);
+            lay->setOrientation(layoutDirection);
+            lay->setContentsMargins(0, 0, 0, 0);
+            lay->setSpacing(4);
+            lay->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
+            setLayout(lay);
+            updateBorders(geometry().toRect());
+
+            foreach (Applet *applet, applets()) {
+                lay->addItem(applet);
+            }
+
+            lay->setMaximumSize(size());
         }
     }
 
     //we need to know if the width or height is 100%
     if (constraints & Plasma::LocationConstraint || constraints & Plasma::SizeConstraint) {
         m_currentSize = geometry().size().toSize();
-        QRectF screenRect = screen() >= 0 ? QApplication::desktop()->screenGeometry(screen()) :
+        QRectF screenRect = screen() >= 0 ? Kephal::ScreenUtils::screenGeometry(screen()) :
             geometry();
 
-        if ((formFactor() == Horizontal && m_currentSize.width() >= screenRect.width()) || 
+        if ((formFactor() == Horizontal && m_currentSize.width() >= screenRect.width()) ||
                 (formFactor() == Vertical && m_currentSize.height() >= screenRect.height())) {
             m_background->setElementPrefix(location());
         } else {
-            m_background->setElementPrefix(QString());
+            switch (location()) {
+            case LeftEdge:
+                //this call will automatically fallback at no prefix if the element isn't available
+                m_background->setElementPrefix("west-mini");
+                break;
+            case RightEdge:
+                m_background->setElementPrefix("east-mini");
+                break;
+            case TopEdge:
+                m_background->setElementPrefix("north-mini");
+                break;
+            case BottomEdge:
+            default:
+                m_background->setElementPrefix("south-mini");
+                break;
+            }
         }
 
-        m_background->resizePanel(m_currentSize);
+        m_background->resizeFrame(m_currentSize);
     }
 
     //FIXME: this seems the only way to correctly resize the layout the first time when the
@@ -404,32 +540,34 @@ void Panel::paintInterface(QPainter *painter,
                            const QStyleOptionGraphicsItem *option,
                            const QRect& contentsRect)
 {
+    Q_UNUSED(contentsRect)
     //FIXME: this background drawing is bad and ugly =)
     // draw the background untransformed (saves lots of per-pixel-math)
     painter->save();
     painter->resetTransform();
 
     const Containment::StyleOption *containmentOpt = qstyleoption_cast<const Containment::StyleOption *>(option);
-    
+
     QRect viewGeom;
-    if (containmentOpt) {
+    if (containmentOpt && containmentOpt->view) {
         viewGeom = containmentOpt->view->geometry();
     }
 
-    if (viewGeom != m_lastViewGeom) {
+    if (m_maskDirty || m_lastViewGeom != viewGeom) {
+        m_maskDirty = false;
         m_lastViewGeom = viewGeom;
+
         updateBorders(viewGeom);
+        if (containmentOpt && containmentOpt->view) {
+            containmentOpt->view->setMask(m_background->mask());
+        }
     }
 
     // blit the background (saves all the per-pixel-products that blending does)
     painter->setCompositionMode(QPainter::CompositionMode_Source);
     painter->setRenderHint(QPainter::Antialiasing);
 
-    m_background->paintPanel(painter, contentsRect);
-
-    if (containmentOpt && containmentOpt->view) {
-        containmentOpt->view->setMask(m_background->mask());
-    }
+    m_background->paintFrame(painter, option->exposedRect);
 
     // restore transformation and composition mode
     painter->restore();
@@ -456,6 +594,80 @@ void Panel::setFormFactorFromLocation(Plasma::Location loc) {
     }
 }
 
+void Panel::showDropZone(const QPoint pos)
+{
+    if (!scene()) {
+        return;
+    }
+
+    // if the drop isn't happening on the outer edges and is instead
+    // actually poised over an applet, ignore it
+    if (((formFactor() == Plasma::Vertical && pos.y() > 1 && pos.y() > size().height() - 2) ||
+         (pos.x() > 1 && pos.x() < size().width() - 2)) &&
+        scene()->itemAt(mapToScene(pos)) != this) {
+        return;
+    }
+
+    QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(layout());
+
+    if (!lay) {
+        return;
+    }
+
+    if (pos == QPoint()) {
+        if (m_spacer) {
+            lay->removeItem(m_spacer);
+            m_spacer->hide();
+        }
+        return;
+    }
+
+    //lucky case: the spacer is already in the right position
+    if (m_spacer && m_spacer->geometry().contains(pos)) {
+        return;
+    }
+
+    Plasma::FormFactor f = formFactor();
+    int insertIndex = -1;
+
+    //FIXME: needed in two places, make it a function?
+    for (int i = 0; i < lay->count(); ++i) {
+        QRectF siblingGeometry = lay->itemAt(i)->geometry();
+
+        if (f == Plasma::Horizontal) {
+            qreal middle = (siblingGeometry.left() + siblingGeometry.right()) / 2.0;
+            if (pos.x() < middle) {
+                insertIndex = i;
+                break;
+            } else if (pos.x() <= siblingGeometry.right()) {
+                insertIndex = i + 1;
+                break;
+            }
+        } else { // Plasma::Vertical
+            qreal middle = (siblingGeometry.top() + siblingGeometry.bottom()) / 2.0;
+            if (pos.y() < middle) {
+                insertIndex = i;
+                break;
+            } else if (pos.y() <= siblingGeometry.bottom()) {
+                insertIndex = i + 1;
+                break;
+            }
+        }
+    }
+
+    m_spacerIndex = insertIndex;
+    if (insertIndex != -1) {
+        if (!m_spacer) {
+            m_spacer = new Spacer(this);
+            m_spacer->panel = this;
+        }
+        lay->removeItem(m_spacer);
+        m_spacer->show();
+        lay->insertItem(insertIndex, m_spacer);
+    }
+}
+
+
 K_EXPORT_PLASMA_APPLET(panel, Panel)
 
 #include "panel.moc"
