@@ -1,5 +1,6 @@
  /*  This file is part of the KDE project
     Copyright (C) 2006 Kevin Ottens <ervin@kde.org>
+    Copyright (C) 2008 Dario Freddi <drf54321@gmail.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -66,16 +67,9 @@ HalPower::HalPower(QObject *parent, const QStringList  & /*args */)
 
 HalPower::~HalPower()
 {
-    QList<Solid::Device *> devices;
-
-    devices << m_acAdapters.values();
-    devices << m_batteries.values();
-    devices << m_buttons.values();
-
-    foreach (Solid::Device *dev, devices)
-    {
-        delete dev;
-    }
+    qDeleteAll(m_acAdapters);
+    qDeleteAll(m_batteries);
+    qDeleteAll(m_buttons);
 }
 
 QStringList HalPower::supportedSchemes() const
@@ -168,6 +162,11 @@ int HalPower::batteryChargePercent() const
     return (m_currentBatteryCharge *100)/m_maxBatteryCharge;
 }
 
+int HalPower::batteryRemainingTime() const
+{
+    return m_estimatedBatteryTime;
+}
+
 Solid::Control::PowerManager::AcAdapterState HalPower::acAdapterState() const
 {
     if (m_acAdapters.size()==0)
@@ -240,7 +239,7 @@ Solid::Control::PowerManager::CpuFreqPolicies HalPower::supportedCpuFreqPolicies
         QStringList governors = reply;
         Solid::Control::PowerManager::CpuFreqPolicies policies = Solid::Control::PowerManager::UnknownCpuFreqPolicy;
 
-        foreach (QString governor, governors)
+        foreach (const QString& governor, governors)
         {
             if (governor == "ondemand")
             {
@@ -393,7 +392,7 @@ float HalPower::brightness(const QString &device)
     }
     if(m_halManager.call("FindDeviceByCapability", "keyboard_backlight").arguments().at(0).toStringList().contains(device))
     {
-        QDBusInterface deviceInterface("org.freedesktop.Hal", device, "org.freedesktop.Hal.Device.KeyboardBacklight", QDBusConnection::systemBus()); //TODO - I dont have a backlight enabled keyboard, so I'm guessing a bit here. Could someone please check this.
+        QDBusInterface deviceInterface("org.freedesktop.Hal", device, "org.freedesktop.Hal.Device.KeyboardBacklight", QDBusConnection::systemBus()); //TODO - I do not have a backlight enabled keyboard, so I'm guessing a bit here. Could someone please check this.
         brightness = deviceInterface.call("GetBrightness").arguments().at(0).toDouble();
         if(deviceInterface.lastError().isValid())
         {
@@ -427,7 +426,7 @@ bool HalPower::setBrightness(float brightness, const QString &device)
     {
         QDBusInterface propertyInterface("org.freedesktop.Hal", device, "org.freedesktop.Hal.Device", QDBusConnection::systemBus());
         int levels = propertyInterface.call("GetProperty", "keyboard_backlight.num_levels").arguments().at(0).toInt();
-        QDBusInterface deviceInterface("org.freedesktop.Hal", device, "org.freedesktop.Hal.Device.KeyboardBacklight", QDBusConnection::systemBus()); //TODO - I dont have a backlight enabled keyboard, so I'm guessing a bit here. Could someone please check this.
+        QDBusInterface deviceInterface("org.freedesktop.Hal", device, "org.freedesktop.Hal.Device.KeyboardBacklight", QDBusConnection::systemBus()); //TODO - I do not have a backlight enabled keyboard, so I'm guessing a bit here. Could someone please check this.
         deviceInterface.call("SetBrightness", qRound((levels-1)*(brightness/100.0)));
         if(!deviceInterface.lastError().isValid())
         {
@@ -467,6 +466,8 @@ void HalPower::computeBatteries()
         m_batteries[battery.udi()] = new Solid::Device(battery);
         connect(m_batteries[battery.udi()]->as<Solid::Battery>(), SIGNAL(chargePercentChanged(int, const QString &)),
                  this, SLOT(updateBatteryStats()));
+        connect(m_batteries[battery.udi()]->as<Solid::GenericInterface>(), SIGNAL(propertyChanged(const QMap<QString,int> &)),
+                 this, SLOT(slotBatteryPropertyChanged(const QMap<QString,int> &)));
     }
 
     updateBatteryStats();
@@ -492,8 +493,9 @@ void HalPower::updateBatteryStats()
     m_warningBatteryCharge = 0;
     m_lowBatteryCharge = 0;
     m_criticalBatteryCharge = 0;
+    m_estimatedBatteryTime = 0;
 
-    foreach (Solid::Device *d, m_batteries.values())
+    foreach (Solid::Device *d, m_batteries)
     {
         Solid::GenericInterface *interface = d->as<Solid::GenericInterface>();
 
@@ -503,6 +505,7 @@ void HalPower::updateBatteryStats()
         m_maxBatteryCharge+= interface->property("battery.charge_level.last_full").toInt();
         m_warningBatteryCharge+= interface->property("battery.charge_level.warning").toInt();
         m_lowBatteryCharge+= interface->property("battery.charge_level.low").toInt();
+        m_estimatedBatteryTime+= interface->property("battery.remaining_time").toInt() * 1000;
     }
 
     m_criticalBatteryCharge = m_lowBatteryCharge/2;
@@ -578,6 +581,8 @@ void HalPower::slotDeviceAdded(const QString &udi)
         m_batteries[udi] = device;
         connect(m_batteries[udi]->as<Solid::Battery>(), SIGNAL(chargePercentChanged(int, const QString &)),
                  this, SLOT(updateBatteryStats()));
+        connect(m_batteries[udi]->as<Solid::GenericInterface>(), SIGNAL(propertyChanged(const QMap<QString,int> &)),
+                 this, SLOT(slotBatteryPropertyChanged(const QMap<QString,int> &)));
     }
     else if (device->is<Solid::Button>())
     {
@@ -603,7 +608,7 @@ void HalPower::slotDeviceRemoved(const QString &udi)
 
         m_pluggedAdapterCount = 0;
 
-        foreach (Solid::Device *d, m_acAdapters.values())
+        foreach (Solid::Device *d, m_acAdapters)
         {
             if (d->as<Solid::AcAdapter>()!=0
               && d->as<Solid::AcAdapter>()->isPlugged())
@@ -633,4 +638,16 @@ void HalPower::slotDeviceRemoved(const QString &udi)
     }
 }
 
+void HalPower::slotBatteryPropertyChanged(const QMap<QString,int> &changes)
+{
+    /* This slot catches property changes on battery devices. At
+     * the moment it is used to find out remaining time on batteries.
+     */
+
+    if (changes.contains("battery.remaining_time")) {
+        updateBatteryStats();
+        emit batteryRemainingTimeChanged(batteryRemainingTime());
+    }
+}
+
 #include "halpower.moc"
